% This script allows you to check to what extent the simulation does the sequence of steps you want it to make

%  Here you can encode your "Expectation": Which quantum states are supposed to be realised when:

% before preparation
cA = [1 0];
cB = [1 0];
initialstate = [cA(1)*cB(1)  cA(1)*cB(2)  cA(2)*cB(1)  cA(2)*cB(2)]; 

% after preparation
phiA=pi/2;
cA = [exp(i*phiA/2)  -i*exp(-i*phiA/2)]/sqrt(2);
phiB=-pi/2;
%cB = [1 0];
cB = [exp(i*phiB/2)  -i*exp(-i*phiB/2)]/sqrt(2);

after_register_preparation = [cA(1)*cB(1)  cA(1)*cB(2)  cA(2)*cB(1)  cA(2)*cB(2)]; 
rhoprepared = transpose(after_register_preparation)*conj(after_register_preparation);

% after gate
after_gate_operation = [after_register_preparation(1) after_register_preparation(2) after_register_preparation(3) -after_register_preparation(4)];
rhoaftergate = transpose(after_gate_operation)*conj(after_gate_operation);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Here we extract the simulation density matrices, at some specific times, corresponding to the above

rho = rho_re_1 + i*rho_im_1;
rhomat= zeros(length(t_1),4,4);
rhomatlarge= zeros(length(t_1),5,5);

for tt=1:length(t_1),
%reshape density matrix into an actual matrix:
   rhomat(tt,:,:) = reshape(squeeze(rho(tt,:,:,:,:)),4,4);
   rhomat(tt,[2 3],:) =  rhomat(tt,[3 2],:); %reorder centre
   rhomat(tt,:,[2 3]) =  rhomat(tt,:,[3 2]); %reorder centre
   rhomatlarge(tt,1:4,1:4) = rhomat(tt,:,:); % add fake rows and columns for "pcolor"
end

% chose this time after your preparation is complete
t_afterprep = 1;
% chose this time after your gate is complete
t_aftergate = t_1(end);

[dump tind_afterprep] = min(abs(t_1 -t_afterprep));
[dump tind_aftergate] = min(abs(t_1 - t_aftergate));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% add fake rows and columns for "pcolor"
rhopreparedlarge = zeros(5,5);
rhopreparedlarge(1:4,1:4) = rhoprepared;
rhoaftergatelarge = zeros(5,5);
rhoaftergatelarge(1:4,1:4) = rhoaftergate;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% adjust this if it does not suit your states:
colorrange = [-0.5 0.5];

subplot(2,4,1)
pcolor(real(rhopreparedlarge))
daspect([1 1 1])
xlabel(['A'])
ylabel(['B'])
zlabel(['Re[\rho]'])
xticks([1:4]+0.5)
yticks([1:4]+0.5)
xticklabels({'00','01','10','11'})
yticklabels({'00','01','10','11'})
title(['after prep, target Re'])
caxis(colorrange)
colorbar

subplot(2,4,2)
pcolor(imag(rhopreparedlarge))
daspect([1 1 1])
xlabel(['A'])
ylabel(['B'])
zlabel(['Im[\rho]'])
xticks([1:4]+0.5)
yticks([1:4]+0.5)
xticklabels({'00','01','10','11'})
yticklabels({'00','01','10','11'})
title(['after prep, target Im'])
caxis(colorrange)
colorbar

subplot(2,4,3)
pcolor(real(squeeze(rhomatlarge(tind_afterprep,:,:))))
daspect([1 1 1])
xlabel(['A'])
ylabel(['B'])
zlabel(['Re[\rho]'])
xticks([1:4]+0.5)
yticks([1:4]+0.5)
xticklabels({'00','01','10','11'})
yticklabels({'00','01','10','11'})
title(['after prep, actual Re'])
caxis(colorrange)
colorbar



subplot(2,4,4)
pcolor(imag(squeeze(rhomatlarge(tind_afterprep,:,:))))
daspect([1 1 1])
xlabel(['A'])
ylabel(['B'])
zlabel(['Im'])
xticks([1:4]+0.5)
yticks([1:4]+0.5)
xticklabels({'00','01','10','11'})
yticklabels({'00','01','10','11'})
title(['after prep, actual Im'])
caxis(colorrange)
colorbar


subplot(2,4,5)
pcolor(real(rhoaftergatelarge))
daspect([1 1 1])
xlabel(['A'])
ylabel(['B'])
zlabel(['Re[\rho]'])
xticks([1:4]+0.5)
yticks([1:4]+0.5)
xticklabels({'00','01','10','11'})
yticklabels({'00','01','10','11'})
title(['after gate, target Re'])
caxis(colorrange)
colorbar


subplot(2,4,6)
pcolor(imag(rhoaftergatelarge))
daspect([1 1 1])
xlabel(['A'])
ylabel(['B'])
zlabel(['Im[\rho]'])
xticks([1:4]+0.5)
yticks([1:4]+0.5)
xticklabels({'00','01','10','11'})
yticklabels({'00','01','10','11'})
title(['after gate, target Im'])
caxis(colorrange)
colorbar


subplot(2,4,7)
pcolor(real(squeeze(rhomatlarge(tind_aftergate,:,:))))
daspect([1 1 1])
xlabel(['A'])
ylabel(['B'])
zlabel(['Re[\rho]'])
xticks([1:4]+0.5)
yticks([1:4]+0.5)
xticklabels({'00','01','10','11'})
yticklabels({'00','01','10','11'})
title(['after gate, actual Re'])
caxis(colorrange)
colorbar


subplot(2,4,8)
pcolor(imag(squeeze(rhomatlarge(tind_aftergate,:,:))))
daspect([1 1 1])
xlabel(['A'])
ylabel(['B'])
zlabel(['Im[\rho]'])
xticks([1:4]+0.5)
yticks([1:4]+0.5)
xticklabels({'00','01','10','11'})
yticklabels({'00','01','10','11'})
title(['after gate, actual Im'])
caxis(colorrange)
colorbar



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lw=2;
fs=20;

lineobj = findobj('type', 'Line');
set(lineobj,'LineWidth', lw);

patchobj = findobj('type', 'Patch');
set(patchobj,'LineWidth', lw);

textobj = findobj('type', 'axes');
set(textobj,'LineWidth', lw);
set(textobj, 'fontsize', fs);
set(textobj, 'FontName', 'Times');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  