
// ********************************************************
// simulation logging

#define _SAMPLE_LOG_LEVEL             (1 << 0)
#define _SEGMENT_LOG_LEVEL            (1 << 1)
#define _PATH_LOG_LEVEL               (1 << 2)
#define _SIMULATION_LOG_LEVEL         (1 << 3)
#define _WARNING_LOG_LEVEL            (1 << 4)
#define _ERROR_LOG_LEVEL              (1 << 5)
#define _NO_ERROR_TERMINATE_LOG_LEVEL (1 << 6)
#define _ALL_LOG_LEVELS        _SAMPLE_LOG_LEVEL|_SEGMENT_LOG_LEVEL|_PATH_LOG_LEVEL|_SIMULATION_LOG_LEVEL|_WARNING_LOG_LEVEL|_ERROR_LOG_LEVEL|_NO_ERROR_TERMINATE_LOG_LEVEL
#define _LOG_LEVELS_BEING_LOGGED (_ALL_LOG_LEVELS)

#define real Re
#define imag Im

#include <complex>

#undef real
#undef imag


#include <stdio.h>

#define _LOG(logLevel, ...) \
  do { \
    if (logLevel & _LOG_LEVELS_BEING_LOGGED) { \
      if (logLevel & (_ERROR_LOG_LEVEL | _WARNING_LOG_LEVEL)) \
          printf("%s:%i: ", __FILE__, __LINE__); \
      printf(__VA_ARGS__); \
      fflush(stdout); \
      if (logLevel & (_ERROR_LOG_LEVEL | _NO_ERROR_TERMINATE_LOG_LEVEL)) \
        exit(logLevel == _ERROR_LOG_LEVEL); \
    } \
  } while (0)

// ********************************************************
// simulation includes

#include <xpdeint_platform.h>
#include <cmath>
#include <string>
#include <cstring>
#include <fstream>
#include <sstream>
#include <cstdlib>

#if CFG_OSAPI == CFG_OSAPI_POSIX // These are POSIX headers (i.e. non-windows)
  #include <sys/time.h>
#endif // POSIX

#ifdef __APPLE__
  #include <Availability.h>
  #if __MAC_OS_X_VERSION_MIN_REQUIRED >= 1080
    #define OS_OBJECT_USE_OBJC 0 // Don't make dispatch and xpc objects Objective-C objects.
    #include <IOKit/pwr_mgt/IOPMLib.h> // To disable user idle sleep on Mountain Lion
  #endif
#endif

#include <time.h>
#include <list>
#include <vector>
#include <algorithm>

#include <utility>
#include <map>

#if (CFG_COMPILER == CFG_COMPILER_MSVC)
  #define FFTW_DLL
#endif

#include <fftw3.h>
#include <sys/stat.h>
#include <sys/types.h>

#define _xmds_malloc fftw_malloc
#define xmds_free fftw_free

#define H5_USE_16_API
#include <hdf5.h>

#if !defined(HAVE_H5LEXISTS)
htri_t H5Lexists(hid_t loc_id, const char *name, hid_t lapl_id)
{
  H5E_auto_t error_func;
  void* error_client_data;
  // Squelch errors generated by H5Gget_objinfo. It will report errors when it can't find an object
  // but that's the purpose of calling it.
  H5Eget_auto(&error_func, &error_client_data);
  H5Eset_auto(NULL, NULL);
  herr_t err = H5Gget_objinfo(loc_id, name, false, NULL);
  H5Eset_auto(error_func, error_client_data);
  if (err >= 0)
    return true;
  else
    return false;
}
#endif

#define H5T_NATIVE_REAL H5T_NATIVE_DOUBLE
#if defined(HAVE_HDF5_HL)
  #include <hdf5_hl.h>
#endif


typedef long integer;
typedef double real;
typedef std::complex<real> XMDSComplexType;

#include <xpdeint.h>

#define complex XMDSComplexType

const complex i(0.0, 1.0);

using namespace std;

#if CFG_COMPILER == CFG_COMPILER_ICC
  //
  // Disable ICC's warning: label was declared but never referenced
  //
  #pragma warning ( disable : 177 )
#endif

inline void *xmds_malloc(size_t size);

// ********************************************************
// DEFINES
// ********************************************************

// ********************************************************
//   Simulation defines
#define _EPSILON 1e-6
#ifndef INFINITY
#define INFINITY HUGE_VAL
#endif

#ifndef MAX
#define MAX(a, b) \
  ({ typeof(a) _a = (a); \
     typeof(b) _b = (b); \
     _a > _b ? _a : _b; })
#endif

#ifndef MIN
#define MIN(a, b) \
   ({ typeof(a) _a = (a); \
      typeof(b) _b = (b); \
      _a < _b ? _a : _b; })
#endif


// ********************************************************
//   Auto-vectorisation defines

#define _MAKE_AUTOVEC_VARIABLE(x) real* const __restrict__ x ## _autovec = (real*) x
#define _AUTOVEC(x) (x ## _autovec)

// ********************************************************
//   Geometry defines
#define _lattice_a ((int)2)
#define _min_a     ((long)0)
#define _max_a     ((long)1)
#define _da        ((long)1)

#define _lattice_b ((int)2)
#define _min_b     ((long)0)
#define _max_b     ((long)1)
#define _db        ((long)1)

#define _lattice_c ((int)2)
#define _min_c     ((long)0)
#define _max_c     ((long)1)
#define _dc        ((long)1)

#define _lattice_d ((int)2)
#define _min_d     ((long)0)
#define _max_d     ((long)1)
#define _dd        ((long)1)

#define _lattice_aprime ((int)2)
#define _min_aprime     ((long)0)
#define _max_aprime     ((long)1)
#define _daprime        ((long)1)

#define _lattice_bprime ((int)2)
#define _min_bprime     ((long)0)
#define _max_bprime     ((long)1)
#define _dbprime        ((long)1)

#define _lattice_cprime ((int)2)
#define _min_cprime     ((long)0)
#define _max_cprime     ((long)1)
#define _dcprime        ((long)1)

#define _lattice_dprime ((int)2)
#define _min_dprime     ((long)0)
#define _max_dprime     ((long)1)
#define _ddprime        ((long)1)

// ********************************************************
//   field abcd defines
#define _abcd_ndims 4


// vector density_matrix defines
#define _abcd_density_matrix_ncomponents 1

// ********************************************************
//   field dimensionless defines
#define _dimensionless_ndims 0


// vector time_dependent_coupling defines
#define _dimensionless_time_dependent_coupling_ncomponents 5

// ********************************************************
//   segment 1 (RK89 adaptive-step integrator) defines
// vector density_matrix defines
#define _segment1_operator0_delta_a_field_density_matrix_ncomponents 1

// ********************************************************
//   field mg0_sampling defines
#define _mg0_sampling_ndims 4


// ********************************************************
//   field mg0_output defines
#define _mg0_output_ndims 5


#define _mg0_output_lattice_t ((int)201)
#define _mg0_output_min_t     (_mg0_output_t[0])
#define _mg0_output_max_t     (_mg0_output_t[_mg0_output_lattice_t-1])
#define _mg0_output_dt        (_mg0_output_t[_index_t+1]-_mg0_output_t[_index_t])

// vector raw defines
#define _mg0_output_raw_ncomponents 2

// ********************************************************
//   field mg1_sampling defines
#define _mg1_sampling_ndims 4


// ********************************************************
//   field mg1_output defines
#define _mg1_output_ndims 3


#define _mg1_output_lattice_t ((int)201)
#define _mg1_output_min_t     (_mg1_output_t[0])
#define _mg1_output_max_t     (_mg1_output_t[_mg1_output_lattice_t-1])
#define _mg1_output_dt        (_mg1_output_t[_index_t+1]-_mg1_output_t[_index_t])

// vector raw defines
#define _mg1_output_raw_ncomponents 1

// ********************************************************
//   field mg2_sampling defines
#define _mg2_sampling_ndims 4


// ********************************************************
//   field mg2_output defines
#define _mg2_output_ndims 1


#define _mg2_output_lattice_t ((int)201)
#define _mg2_output_min_t     (_mg2_output_t[0])
#define _mg2_output_max_t     (_mg2_output_t[_mg2_output_lattice_t-1])
#define _mg2_output_dt        (_mg2_output_t[_index_t+1]-_mg2_output_t[_index_t])

// vector raw defines
#define _mg2_output_raw_ncomponents 1

// ********************************************************
//   field mg3_sampling defines
#define _mg3_sampling_ndims 4


// ********************************************************
//   field mg3_output defines
#define _mg3_output_ndims 1


#define _mg3_output_lattice_t ((int)201)
#define _mg3_output_min_t     (_mg3_output_t[0])
#define _mg3_output_max_t     (_mg3_output_t[_mg3_output_lattice_t-1])
#define _mg3_output_dt        (_mg3_output_t[_index_t+1]-_mg3_output_t[_index_t])

// vector raw defines
#define _mg3_output_raw_ncomponents 1

// ********************************************************
//   field mg4_sampling defines
#define _mg4_sampling_ndims 0


// ********************************************************
//   field mg4_output defines
#define _mg4_output_ndims 1


#define _mg4_output_lattice_t ((int)201)
#define _mg4_output_min_t     (_mg4_output_t[0])
#define _mg4_output_max_t     (_mg4_output_t[_mg4_output_lattice_t-1])
#define _mg4_output_dt        (_mg4_output_t[_index_t+1]-_mg4_output_t[_index_t])

// vector raw defines
#define _mg4_output_raw_ncomponents 5

// ********************************************************
//   field segment1_operator0_looping_field defines
#define _segment1_operator0_looping_field_ndims 4


// ********************************************************
//   field segment1_operator0_delta_a_field defines
#define _segment1_operator0_delta_a_field_ndims 4



// ********************************************************
// GLOBALS
// ********************************************************


// ********************************************************
//   Simulation globals

string gsArgsAndValues = "";


// gPathID is used to keep track of the current path number when doing multipath simulations.
// It's needed for the export_all_paths option. Note his counter is per rank, so when
// running the mpi-multi-path driver it runs from 0 to _n_paths / _size - 1.
long gPathID = 0;    

  
real t;

// ********************************************************
//   Transform Multiplexer globals
typedef pair<ptrdiff_t, ptrdiff_t> _basis_pair;
typedef void (*transform_function)(bool, real, real* const __restrict__, real* const __restrict__, ptrdiff_t, ptrdiff_t);

// Less than operator needed by the C++ map class
struct _basis_pair_less_than
{
  bool operator()(const _basis_pair& _x, const _basis_pair& _y) const {
    return (_x.first < _y.first) || ((_x.first == _y.first) && (_x.second < _y.second));
  }
};

struct _transform_step
{
  transform_function _func;
  bool _forward;
  bool _out_of_place;
  ptrdiff_t _prefix_lattice;
  ptrdiff_t _postfix_lattice;
};

// Structure to hold the basis change information
struct _basis_transform_t
{
  vector<_transform_step> _transform_steps;
  real _multiplier;
  
  _basis_transform_t(real _multiplier_in = 1.0) : _multiplier(_multiplier_in) {}
  
  _basis_transform_t(const _basis_transform_t& _b) : _transform_steps(_b._transform_steps), _multiplier(_b._multiplier) {}
  
  void append(transform_function _func, bool _forward, bool _out_of_place, ptrdiff_t _prefix_lattice, ptrdiff_t _postfix_lattice)
  {
    _transform_steps.push_back((_transform_step){_func, _forward, _out_of_place, _prefix_lattice, _postfix_lattice});
  }
};

// Map type for holding (old_basis, new_basis) -> _basis_transform_t mappings
typedef map<_basis_pair, _basis_transform_t, _basis_pair_less_than> _basis_map;


real *_auxiliary_array = NULL;

const char *_basis_identifiers[] = {
};

// ********************************************************
//   'Globals' element globals

#line 28 "Assignment5_program_solution_v2.xmds"

 // Constant Parameters, dimensionless units  
const double gammaA = 1.0/5.0;  
const double gammaB = 1.0/5.0;  

// Time dependent control 
const double delayA = 0.0;             // time until pulse starts
const double TpulseA = 0.25;            // duration of the pulse
const double pulsetypeA = M_PI/2.0;    // pi-half pulse 

const double delayB = 0.5;             // time until pulse starts
const double TpulseB = 0.25;            // duration of the pulse
const double pulsetypeB = M_PI/2.0;    // pi-half pulse 

const double delayDeltaA = 1.0;             // time until pulse starts
const double TpulseDeltaA = 0.25;            // duration of the pulse
const double targetphaseA = 0.3*M_PI;

const double delayDeltaB = 1.5;             // time until pulse starts
const double TpulseDeltaB = 0.25;            // duration of the pulse
const double targetphaseB = -M_PI/2.0;

const double delayU = 2.0;             // time until pulse starts
const double TpulseU = 0.25;            // duration of the pulse
const double targetphaseU = M_PI;

// Resultant amplitude of parameters      
const double OmegaA0 = pulsetypeA/TpulseA;
const double OmegaB0 = pulsetypeB/TpulseB;
const double DeltaA0 = targetphaseA/TpulseDeltaA;
const double DeltaB0 = targetphaseB/TpulseDeltaB;
const double U0 = targetphaseU/TpulseU;


#line 429 "Assignment5_program_solution_v2.cc"

// ********************************************************
//   FFTW3 globals
const real _inverse_sqrt_2pi = 1.0 / sqrt(2.0 * M_PI); 
string _fftwWisdomPath;

// ********************************************************
//   Geometry globals
long* _a = NULL;

long* _b = NULL;

long* _c = NULL;

long* _d = NULL;

long* _aprime = NULL;

long* _bprime = NULL;

long* _cprime = NULL;

long* _dprime = NULL;

// ********************************************************
//   field abcd globals
// vector density_matrix globals
size_t _abcd_density_matrix_alloc_size = 0;
complex* _abcd_density_matrix = NULL;
complex* _active_abcd_density_matrix = NULL;

// ********************************************************
//   field dimensionless globals
// vector time_dependent_coupling globals
size_t _dimensionless_time_dependent_coupling_alloc_size = 0;
real* _dimensionless_time_dependent_coupling = NULL;
real* _active_dimensionless_time_dependent_coupling = NULL;

// ********************************************************
//   segment 1 (RK89 adaptive-step integrator) globals
complex* _segment1_akafield_abcd_density_matrix;
complex* _segment1_akbfield_abcd_density_matrix;
complex* _segment1_akcfield_abcd_density_matrix;
complex* _segment1_akdfield_abcd_density_matrix;
complex* _segment1_akefield_abcd_density_matrix;
complex* _segment1_akffield_abcd_density_matrix;
complex* _segment1_akgfield_abcd_density_matrix;
complex* _segment1_akhfield_abcd_density_matrix;
complex* _segment1_akifield_abcd_density_matrix;
complex* _segment1_akjfield_abcd_density_matrix;
complex* _segment1_initial_abcd_density_matrix;

// vector density_matrix globals
size_t _segment1_operator0_delta_a_field_density_matrix_alloc_size = 0;
complex* _segment1_operator0_delta_a_field_density_matrix = NULL;
complex* _active_segment1_operator0_delta_a_field_density_matrix = NULL;

// ********************************************************
//   field mg0_output globals
real* _mg0_output_t = NULL;
unsigned long _mg0_output_index_t = 0;

// vector raw globals
size_t _mg0_output_raw_alloc_size = 0;
real* _mg0_output_raw = NULL;
real* _active_mg0_output_raw = NULL;

// ********************************************************
//   field mg1_output globals
real* _mg1_output_t = NULL;
unsigned long _mg1_output_index_t = 0;

// vector raw globals
size_t _mg1_output_raw_alloc_size = 0;
real* _mg1_output_raw = NULL;
real* _active_mg1_output_raw = NULL;

// ********************************************************
//   field mg2_output globals
real* _mg2_output_t = NULL;
unsigned long _mg2_output_index_t = 0;

// vector raw globals
size_t _mg2_output_raw_alloc_size = 0;
real* _mg2_output_raw = NULL;
real* _active_mg2_output_raw = NULL;

// ********************************************************
//   field mg3_output globals
real* _mg3_output_t = NULL;
unsigned long _mg3_output_index_t = 0;

// vector raw globals
size_t _mg3_output_raw_alloc_size = 0;
real* _mg3_output_raw = NULL;
real* _active_mg3_output_raw = NULL;

// ********************************************************
//   field mg4_output globals
real* _mg4_output_t = NULL;
unsigned long _mg4_output_index_t = 0;

// vector raw globals
size_t _mg4_output_raw_alloc_size = 0;
real* _mg4_output_raw = NULL;
real* _active_mg4_output_raw = NULL;


// ********************************************************
// FUNCTION PROTOTYPES
// ********************************************************

// ********************************************************
//   field abcd function prototypes
void _abcd_density_matrix_initialise();

// ********************************************************
//   field dimensionless function prototypes
void _dimensionless_time_dependent_coupling_initialise();
void _dimensionless_time_dependent_coupling_evaluate();

// ********************************************************
//   segment 0 (Top level sequence) function prototypes
void _segment0();

// ********************************************************
//   segment 1 (RK89 adaptive-step integrator) function prototypes
void _segment1();
inline void _segment1_calculate_delta_a(real _step);
inline void _segment1_ip_evolve(int _exponent);
inline void _segment1_calculate_nonconstant_ip_fields(real _step, int _exponent);
real _segment1_setup_sampling(bool* _next_sample_flag, long* _next_sample_counter);
real _segment1_abcd_density_matrix_timestep_error(complex* _checkfield);
bool _segment1_abcd_density_matrix_reset(complex* _reset_to);

void _segment1_abcd_operators_evaluate_operator0(real _step);
inline void _segment1_abcd_operators_operator0_copy_delta_a(real _step);

// ********************************************************
//   output function prototypes
void _write_output();

FILE* _open_xsil_file(const char* _filename);
void _close_xsil_file(FILE*& fp);
void _write_xsil_header(FILE* fp);
void _write_xsil_footer(FILE* fp);

// ********************************************************
//   moment group 0 function prototypes
void _mg0_sample();
void _mg0_process();
void _mg0_write_out(FILE* _outfile);

// ********************************************************
//   field mg0_output function prototypes
void _mg0_output_raw_initialise();

// ********************************************************
//   moment group 1 function prototypes
void _mg1_sample();
void _mg1_process();
void _mg1_write_out(FILE* _outfile);

// ********************************************************
//   field mg1_output function prototypes
void _mg1_output_raw_initialise();

// ********************************************************
//   moment group 2 function prototypes
void _mg2_sample();
void _mg2_process();
void _mg2_write_out(FILE* _outfile);

// ********************************************************
//   field mg2_output function prototypes
void _mg2_output_raw_initialise();

// ********************************************************
//   moment group 3 function prototypes
void _mg3_sample();
void _mg3_process();
void _mg3_write_out(FILE* _outfile);

// ********************************************************
//   field mg3_output function prototypes
void _mg3_output_raw_initialise();

// ********************************************************
//   moment group 4 function prototypes
void _mg4_sample();
void _mg4_process();
void _mg4_write_out(FILE* _outfile);

// ********************************************************
//   field mg4_output function prototypes
void _mg4_output_raw_initialise();

// ********************************************************
// MAIN ROUTINE
// ********************************************************
int main(int argc, char **argv)
{
  #ifdef __APPLE__
    #if __MAC_OS_X_VERSION_MIN_REQUIRED >= 1080
  {
    IOPMAssertionID _powerAssertionID = 0;
    IOReturn __io_result = IOPMAssertionCreateWithDescription(
      kIOPMAssertionTypePreventUserIdleSystemSleep,
      CFSTR("XMDS simulation 'Assignment5_program_solution_v2' preventing user idle sleep"), // Assertion name
      NULL, // Assertion details
      NULL, // Human-readable reason
      NULL, // Localization bundle path
      (CFTimeInterval)0, // never timeout
      kIOPMAssertionTimeoutActionRelease,
      &_powerAssertionID
      );
    if (__io_result != kIOReturnSuccess) {
      _LOG(_WARNING_LOG_LEVEL, "Failed to disable user idle sleep\n");
    }
    // Note, this power assertion is automatically released when the process quits.
  }
    #endif
  #endif
  
    
    
  
  _mg0_output_raw_alloc_size = MAX(_mg0_output_raw_alloc_size, (_mg0_output_lattice_t * _lattice_a * _lattice_b * _lattice_c * _lattice_d) * _mg0_output_raw_ncomponents);
  _dimensionless_time_dependent_coupling_alloc_size = MAX(_dimensionless_time_dependent_coupling_alloc_size, (1) * _dimensionless_time_dependent_coupling_ncomponents);
  _mg3_output_raw_alloc_size = MAX(_mg3_output_raw_alloc_size, (_mg3_output_lattice_t) * _mg3_output_raw_ncomponents);
  _mg4_output_raw_alloc_size = MAX(_mg4_output_raw_alloc_size, (_mg4_output_lattice_t) * _mg4_output_raw_ncomponents);
  _mg1_output_raw_alloc_size = MAX(_mg1_output_raw_alloc_size, (_mg1_output_lattice_t * _lattice_a * _lattice_b) * _mg1_output_raw_ncomponents);
  _abcd_density_matrix_alloc_size = MAX(_abcd_density_matrix_alloc_size, (_lattice_a * _lattice_b * _lattice_c * _lattice_d) * _abcd_density_matrix_ncomponents);
  _segment1_operator0_delta_a_field_density_matrix_alloc_size = MAX(_segment1_operator0_delta_a_field_density_matrix_alloc_size, (_lattice_a * _lattice_b * _lattice_c * _lattice_d) * _segment1_operator0_delta_a_field_density_matrix_ncomponents);
  _mg2_output_raw_alloc_size = MAX(_mg2_output_raw_alloc_size, (_mg2_output_lattice_t) * _mg2_output_raw_ncomponents);
  _a = (long*) xmds_malloc(sizeof(long) * (_lattice_a+1));
  
  _b = (long*) xmds_malloc(sizeof(long) * (_lattice_b+1));
  
  _c = (long*) xmds_malloc(sizeof(long) * (_lattice_c+1));
  
  _d = (long*) xmds_malloc(sizeof(long) * (_lattice_d+1));
  
  _aprime = (long*) xmds_malloc(sizeof(long) * (_lattice_aprime+1));
  
  _bprime = (long*) xmds_malloc(sizeof(long) * (_lattice_bprime+1));
  
  _cprime = (long*) xmds_malloc(sizeof(long) * (_lattice_cprime+1));
  
  _dprime = (long*) xmds_malloc(sizeof(long) * (_lattice_dprime+1));
  
  _abcd_density_matrix = (complex*) xmds_malloc(sizeof(complex) * MAX(_abcd_density_matrix_alloc_size,1));
  _active_abcd_density_matrix = _abcd_density_matrix;
  
  _dimensionless_time_dependent_coupling = (real*) xmds_malloc(sizeof(real) * MAX(_dimensionless_time_dependent_coupling_alloc_size,1));
  _active_dimensionless_time_dependent_coupling = _dimensionless_time_dependent_coupling;
  _mg0_output_t = (real*) xmds_malloc(sizeof(real) * (_mg0_output_lattice_t+1));
  
  
  _mg0_output_raw = (real*) xmds_malloc(sizeof(real) * MAX(_mg0_output_raw_alloc_size,1));
  _active_mg0_output_raw = _mg0_output_raw;
  _mg1_output_t = (real*) xmds_malloc(sizeof(real) * (_mg1_output_lattice_t+1));
  
  
  _mg1_output_raw = (real*) xmds_malloc(sizeof(real) * MAX(_mg1_output_raw_alloc_size,1));
  _active_mg1_output_raw = _mg1_output_raw;
  _mg2_output_t = (real*) xmds_malloc(sizeof(real) * (_mg2_output_lattice_t+1));
  
  
  _mg2_output_raw = (real*) xmds_malloc(sizeof(real) * MAX(_mg2_output_raw_alloc_size,1));
  _active_mg2_output_raw = _mg2_output_raw;
  _mg3_output_t = (real*) xmds_malloc(sizeof(real) * (_mg3_output_lattice_t+1));
  
  
  _mg3_output_raw = (real*) xmds_malloc(sizeof(real) * MAX(_mg3_output_raw_alloc_size,1));
  _active_mg3_output_raw = _mg3_output_raw;
  _mg4_output_t = (real*) xmds_malloc(sizeof(real) * (_mg4_output_lattice_t+1));
  
  
  _mg4_output_raw = (real*) xmds_malloc(sizeof(real) * MAX(_mg4_output_raw_alloc_size,1));
  _active_mg4_output_raw = _mg4_output_raw;
  
  
  for (long _index_a = 0; _index_a < _lattice_a; _index_a++)
    _a[_index_a] = _min_a + _index_a*_da;
  for (long _index_b = 0; _index_b < _lattice_b; _index_b++)
    _b[_index_b] = _min_b + _index_b*_db;
  for (long _index_c = 0; _index_c < _lattice_c; _index_c++)
    _c[_index_c] = _min_c + _index_c*_dc;
  for (long _index_d = 0; _index_d < _lattice_d; _index_d++)
    _d[_index_d] = _min_d + _index_d*_dd;
  for (long _index_aprime = 0; _index_aprime < _lattice_aprime; _index_aprime++)
    _aprime[_index_aprime] = _min_aprime + _index_aprime*_daprime;
  for (long _index_bprime = 0; _index_bprime < _lattice_bprime; _index_bprime++)
    _bprime[_index_bprime] = _min_bprime + _index_bprime*_dbprime;
  for (long _index_cprime = 0; _index_cprime < _lattice_cprime; _index_cprime++)
    _cprime[_index_cprime] = _min_cprime + _index_cprime*_dcprime;
  for (long _index_dprime = 0; _index_dprime < _lattice_dprime; _index_dprime++)
    _dprime[_index_dprime] = _min_dprime + _index_dprime*_ddprime;
  _active_mg0_output_raw = _mg0_output_raw;
  _mg0_output_raw_initialise();
  _active_mg1_output_raw = _mg1_output_raw;
  _mg1_output_raw_initialise();
  _active_mg2_output_raw = _mg2_output_raw;
  _mg2_output_raw_initialise();
  _active_mg3_output_raw = _mg3_output_raw;
  _mg3_output_raw_initialise();
  _active_mg4_output_raw = _mg4_output_raw;
  _mg4_output_raw_initialise();
  // load wisdom
  #if CFG_OSAPI == CFG_OSAPI_POSIX // Don't load wisdom on windows
  {
    char _hostName[256];
    gethostname(_hostName, 256);
    _hostName[255] = '\0'; // just in case
    
    string _pathToWisdom = getenv("HOME");
    _pathToWisdom += "/.xmds/wisdom/";
    
    string _wisdomFileName = _hostName;
    _wisdomFileName += ".wisdom";
    _wisdomFileName += ".fftw3";
    
    FILE *_fp = NULL;
    
    _fp = fopen(_pathToWisdom.c_str(), "r");
    if (_fp) {
      fclose(_fp);
    } else {
      int _result = mkdir((string(getenv("HOME")) + "/.xmds").c_str(), S_IRWXU);
      if (mkdir(_pathToWisdom.c_str(), S_IRWXU)) {
        // We failed to create the ~/.xmds/wisdom directory
        _LOG(_WARNING_LOG_LEVEL, "Warning: Cannot find enlightenment, the path to wisdom ~/.xmds/wisdom doesn't seem to exist and we couldn't create it.\n"
                                 "         I'll use the current path instead.\n");
        _pathToWisdom = ""; // present directory
      }
      
    }
    
    _fftwWisdomPath = _pathToWisdom + _wisdomFileName;
    
    FILE *_wisdomFile = NULL;
    if ( (_wisdomFile = fopen(_fftwWisdomPath.c_str(), "r")) != NULL) {
      _LOG(_SIMULATION_LOG_LEVEL, "Found enlightenment... (Importing wisdom)\n");
      fftw_import_wisdom_from_file(_wisdomFile);
      fclose(_wisdomFile);
    }
  }
  #endif // POSIX
  
  _basis_transform_t *_basis_transform = NULL;
  ptrdiff_t _auxiliary_array_size = 0;
  ptrdiff_t _max_vector_size = 0;
  real* _max_vector_array = NULL;
  
  if (_auxiliary_array_size) {
    _auxiliary_array = (real*) xmds_malloc(sizeof(real) * _auxiliary_array_size);
  }
  
  bool _allocated_temporary_array = false;
  if (!_max_vector_array && _max_vector_size > 0) {
    _max_vector_array = (real*) xmds_malloc(sizeof(real) * _max_vector_size);
    _allocated_temporary_array = true;
  }
  
  // Make all geometry-dependent transformations prepare plans, etc.
  
  if (_allocated_temporary_array) {
    xmds_free(_max_vector_array);
  }
  
  // Get the time at which the simulation started
  timeval _tim;
  gettimeofday(&_tim, NULL);
  double _startTime = _tim.tv_sec + (_tim.tv_usec/1e6);
  
  /* Code that actually does stuff goes here */
  _segment0();
  
  
  _write_output();
  
  // Work out how long the simulation has run for
  gettimeofday(&_tim, NULL);
  double _endTime = _tim.tv_sec + (_tim.tv_usec/1e6);
  _LOG(_SIMULATION_LOG_LEVEL, "Time elapsed for simulation is: %.2f seconds\n", _endTime - _startTime);
  if (_auxiliary_array) {
    xmds_free(_auxiliary_array);
  }
  
  // Save wisdom
  #if CFG_OSAPI == CFG_OSAPI_POSIX
  {
    FILE *_wisdomFile = NULL;
    if ( (_wisdomFile = fopen(_fftwWisdomPath.c_str(), "w")) != NULL) {
      fftw_export_wisdom_to_file(_wisdomFile);
      fclose(_wisdomFile);
    }
  }
  #endif // POSIX
  
  fftw_cleanup();
  
  xmds_free(_abcd_density_matrix);
  _active_abcd_density_matrix = _abcd_density_matrix = NULL;
  
  xmds_free(_dimensionless_time_dependent_coupling);
  _active_dimensionless_time_dependent_coupling = _dimensionless_time_dependent_coupling = NULL;
  
  xmds_free(_mg0_output_raw);
  _active_mg0_output_raw = _mg0_output_raw = NULL;
  
  xmds_free(_mg1_output_raw);
  _active_mg1_output_raw = _mg1_output_raw = NULL;
  
  xmds_free(_mg2_output_raw);
  _active_mg2_output_raw = _mg2_output_raw = NULL;
  
  xmds_free(_mg3_output_raw);
  _active_mg3_output_raw = _mg3_output_raw = NULL;
  
  xmds_free(_mg4_output_raw);
  _active_mg4_output_raw = _mg4_output_raw = NULL;
  
  
  return 0;
}

// ********************************************************
// FUNCTION IMPLEMENTATIONS
// ********************************************************

inline void *xmds_malloc(size_t size)
{
  void *retPointer = _xmds_malloc(size);
  if ( !retPointer )
    _LOG(_ERROR_LOG_LEVEL, "ERROR: Couldn't allocate %zu bytes of memory!", size);
  return retPointer;
}


// ********************************************************
//   Default Simulation Driver function implementations
void _segment0()
{
  t = 0.0;
  
  _mg0_output_raw_initialise();
  _mg1_output_raw_initialise();
  _mg2_output_raw_initialise();
  _mg3_output_raw_initialise();
  _mg4_output_raw_initialise();
  _active_abcd_density_matrix = _abcd_density_matrix;
  _abcd_density_matrix_initialise();
  _mg0_output_index_t = 0;
  _mg1_output_index_t = 0;
  _mg2_output_index_t = 0;
  _mg3_output_index_t = 0;
  _mg4_output_index_t = 0;
  _mg0_sample();
  _mg1_sample();
  _mg2_sample();
  _mg3_sample();
  _mg4_sample();
  _segment1();
  
  _mg0_process();
  _mg1_process();
  _mg2_process();
  _mg3_process();
  _mg4_process();
}


// ********************************************************
//   field abcd function implementations
// initialisation for vector density_matrix
void _abcd_density_matrix_initialise()
{
  
  long _abcd_density_matrix_index_pointer = 0;
  #define rho _active_abcd_density_matrix[_abcd_density_matrix_index_pointer + 0]
  #define a _a[_index_a + 0]
  #define da (_da * (1.0))
  
  for (long _index_a = 0; _index_a < _lattice_a; _index_a++) {
    #define b _b[_index_b + 0]
    #define db (_db * (1.0))
    
    for (long _index_b = 0; _index_b < _lattice_b; _index_b++) {
      #define c _c[_index_c + 0]
      #define dc (_dc * (1.0))
      
      for (long _index_c = 0; _index_c < _lattice_c; _index_c++) {
        #define d _d[_index_d + 0]
        #define dd (_dd * (1.0))
        
        for (long _index_d = 0; _index_d < _lattice_d; _index_d++) {
          // The purpose of the following define is to give a (somewhat helpful) compile-time error
          // if the user has attempted to use the propagation dimension variable in the initialisation
          // block of a <vector> element. If they're trying to do this, what they really want is a 
          // <computed_vector> instead.
          #define t Dont_use_propagation_dimension_t_in_vector_element_CDATA_block___Use_a_computed_vector_instead
          
          // ********** Initialisation code ***************
          #line 68 "Assignment5_program_solution_v2.xmds"
          
          rho = 0.0; 
          if(a==0 && b==0 && c==0 & d==0)
             rho = 1.0;
          
          #line 941 "Assignment5_program_solution_v2.cc"
          // **********************************************
          #undef t
          
          // Increment index pointers for vectors in field abcd (or having the same dimensions)
          _abcd_density_matrix_index_pointer += 1 * _abcd_density_matrix_ncomponents;
          
        }
        #undef d
        #undef dd
      }
      #undef c
      #undef dc
    }
    #undef b
    #undef db
  }
  #undef a
  #undef da
  #undef rho
}

// ********************************************************
//   field dimensionless function implementations
// initialisation for computed vector time_dependent_coupling
void _dimensionless_time_dependent_coupling_initialise()
{
}


void _dimensionless_time_dependent_coupling_evaluate()
{
  long _dimensionless_time_dependent_coupling_index_pointer = 0;
  #define OmegaA_t _active_dimensionless_time_dependent_coupling[_dimensionless_time_dependent_coupling_index_pointer + 0]
  #define OmegaB_t _active_dimensionless_time_dependent_coupling[_dimensionless_time_dependent_coupling_index_pointer + 1]
  #define DeltaA_t _active_dimensionless_time_dependent_coupling[_dimensionless_time_dependent_coupling_index_pointer + 2]
  #define DeltaB_t _active_dimensionless_time_dependent_coupling[_dimensionless_time_dependent_coupling_index_pointer + 3]
  #define U_t _active_dimensionless_time_dependent_coupling[_dimensionless_time_dependent_coupling_index_pointer + 4]
  // ************* Evaluation code ****************
  #line 80 "Assignment5_program_solution_v2.xmds"
  
  OmegaA_t = 0.0;
  if(t>delayA && t<= delayA+TpulseA)
      OmegaA_t = OmegaA0;
  
  OmegaB_t = 0.0;
  if(t>delayB && t<= delayB+TpulseB)
      OmegaB_t = OmegaB0;
  
  DeltaA_t = 0.0;
  if(t>delayDeltaA && t<= delayDeltaA+TpulseDeltaA)
      DeltaA_t = DeltaA0;
  
  DeltaB_t = 0.0;
  if(t>delayDeltaB && t<= delayDeltaB+TpulseDeltaB)
      DeltaB_t = DeltaB0;  
  
  U_t = 0.0;
  if(t>delayU && t<= delayU+TpulseU)
      U_t = U0;  
  
  #line 1002 "Assignment5_program_solution_v2.cc"
  // **********************************************
  #undef OmegaA_t
  #undef OmegaB_t
  #undef DeltaA_t
  #undef DeltaB_t
  #undef U_t
}

// ********************************************************
//   segment 1 (RK89 adaptive-step integrator) function implementations
void _segment1()
{
  real _step = 2.5/(real)1000;
  real _old_step = _step;
  real _min_step = _step;
  real _max_step = _step;
  long _attempted_steps = 0;
  long _unsuccessful_steps = 0;
  
  real _tolerance = 1e-10;
  
  real _error, _last_norm_error = 1.0;
  real _segment1_abcd_density_matrix_error;
  
  bool _discard = false;
  bool _break_next = false;
  
  bool _next_sample_flag[7];
  for (long _i0 = 0; _i0 < 7; _i0++)
    _next_sample_flag[_i0] = false;
  
  long _next_sample_counter[5];
  for (long _i0 = 0; _i0 < 5; _i0++)
    _next_sample_counter[_i0] = 1;
  
  real _t_local = 0.0;
  
  real _t_break_next = _segment1_setup_sampling(_next_sample_flag, _next_sample_counter);
  
  if ( (_t_local + _step)*(1.0 + _EPSILON) >= _t_break_next) {
    _break_next = true;
    _step = _t_break_next - _t_local;
  }
  
  
  _segment1_operator0_delta_a_field_density_matrix = (complex*) xmds_malloc(sizeof(complex) * MAX(_segment1_operator0_delta_a_field_density_matrix_alloc_size,1));
  _active_segment1_operator0_delta_a_field_density_matrix = _segment1_operator0_delta_a_field_density_matrix;
  _segment1_akafield_abcd_density_matrix = (complex*) xmds_malloc(sizeof(complex) * MAX(_abcd_density_matrix_alloc_size,1));
  _segment1_akbfield_abcd_density_matrix = (complex*) xmds_malloc(sizeof(complex) * MAX(_abcd_density_matrix_alloc_size,1));
  _segment1_akcfield_abcd_density_matrix = (complex*) xmds_malloc(sizeof(complex) * MAX(_abcd_density_matrix_alloc_size,1));
  _segment1_akdfield_abcd_density_matrix = (complex*) xmds_malloc(sizeof(complex) * MAX(_abcd_density_matrix_alloc_size,1));
  _segment1_akefield_abcd_density_matrix = (complex*) xmds_malloc(sizeof(complex) * MAX(_abcd_density_matrix_alloc_size,1));
  _segment1_akffield_abcd_density_matrix = (complex*) xmds_malloc(sizeof(complex) * MAX(_abcd_density_matrix_alloc_size,1));
  _segment1_akgfield_abcd_density_matrix = (complex*) xmds_malloc(sizeof(complex) * MAX(_abcd_density_matrix_alloc_size,1));
  _segment1_akhfield_abcd_density_matrix = (complex*) xmds_malloc(sizeof(complex) * MAX(_abcd_density_matrix_alloc_size,1));
  _segment1_akifield_abcd_density_matrix = (complex*) xmds_malloc(sizeof(complex) * MAX(_abcd_density_matrix_alloc_size,1));
  _segment1_akjfield_abcd_density_matrix = (complex*) xmds_malloc(sizeof(complex) * MAX(_abcd_density_matrix_alloc_size,1));
  _segment1_initial_abcd_density_matrix = (complex*) xmds_malloc(sizeof(complex) * MAX(_abcd_density_matrix_alloc_size,1));
  complex* _akafield_abcd_density_matrix = _segment1_akafield_abcd_density_matrix;
  complex* _akbfield_abcd_density_matrix = _segment1_akbfield_abcd_density_matrix;
  complex* _akcfield_abcd_density_matrix = _segment1_akcfield_abcd_density_matrix;
  complex* _akdfield_abcd_density_matrix = _segment1_akdfield_abcd_density_matrix;
  complex* _akefield_abcd_density_matrix = _segment1_akefield_abcd_density_matrix;
  complex* _akffield_abcd_density_matrix = _segment1_akffield_abcd_density_matrix;
  complex* _akgfield_abcd_density_matrix = _segment1_akgfield_abcd_density_matrix;
  complex* _akhfield_abcd_density_matrix = _segment1_akhfield_abcd_density_matrix;
  complex* _akifield_abcd_density_matrix = _segment1_akifield_abcd_density_matrix;
  complex* _akjfield_abcd_density_matrix = _segment1_akjfield_abcd_density_matrix;
  complex* _initial_abcd_density_matrix = _segment1_initial_abcd_density_matrix;
  
  
  // Runge Kutta method constants 
  real _a_raw[16];
  real _a[16];
  real _b[16][16];
  real _c[16];
  real _cs[16];
  real _d[16];
  
  for (unsigned long _i0 = 0; _i0 < 16; _i0++) {
    _a_raw[_i0] = _c[_i0] = _d[_i0] = 0.0;
    for (unsigned long _i1 = 0; _i1 < 16; _i1++)
     _b[_i0][_i1] = 0.0;
  }
  
  _a_raw[1]  = 0.02173913043478260869565217391304347;
  _a_raw[2]  = 0.09629581047800066670113001679819925;
  _a_raw[3]  = 0.14444371571700100005169502519729888;
  _a_raw[4]  = 0.52205882352941176470588235294117647;
  _a_raw[5]  = 0.22842443612863469578031459099794265;
  _a_raw[6]  = 0.54360353589933733219171338103002937;
  _a_raw[7]  = 0.64335664335664335664335664335664335;
  _a_raw[8]  = 0.48251748251748251748251748251748251;
  _a_raw[9]  = 0.06818181818181818181818181818181818;
  _a_raw[10] = 0.25060827250608272506082725060827250;
  _a_raw[11] = 0.66736715965600568968278165443304378;
  _a_raw[12] = 0.85507246376811594202898550724637681;
  _a_raw[13] = 0.89795918367346938775510204081632653;
  _a_raw[14] = 1.0;
  _a_raw[15] = 1.0;
  
  _a[0] = 0.0;
  for (unsigned long _i0 = 1; _i0 < 16; _i0++)
    _a[_i0] = _a_raw[_i0] - _a_raw[_i0 - 1];
  
  _b[1][0]   = 1.0/46.0;
  _b[2][0]   =-0.11698050118114486205818241524969622;
  _b[2][1]   = 0.21327631165914552875931243204789548;
  _b[3][0]   = 0.03611092892925025001292375629932472;
  _b[3][2]   = 0.10833278678775075003877126889797416;
  _b[4][0]   = 1.57329743908138605107331820072051125;
  _b[4][2]   =-5.98400943754042002888532938159655553;
  _b[4][3]   = 4.93277082198844574251789353381722074;
  _b[5][0]   = 0.05052046351120380909008334360006234;
  _b[5][3]   = 0.17686653884807108146683657390397612;
  _b[5][4]   = 0.00103743376935980522339467349390418;
  _b[6][0]   = 0.10543148021953768958529340893598138;
  _b[6][3]   =-0.16042415162569842979496486916719383;
  _b[6][4]   = 0.11643956912829316045688724281285250;
  _b[6][5]   = 0.48215663817720491194449759844838932;
  _b[7][0]   = 0.07148407148407148407148407148407148;
  _b[7][5]   = 0.32971116090443908023196389566296464;
  _b[7][6]   = 0.24216141096813279233990867620960722;
  _b[8][0]   = 0.07162368881118881118881118881118881;
  _b[8][5]   = 0.32859867301674234161492268975519694;
  _b[8][6]   = 0.11622213117906185418927311444060725;
  _b[8][7]   =-0.03392701048951048951048951048951048;
  _b[9][0]   = 0.04861540768024729180628870095388582;
  _b[9][5]   = 0.03998502200331629058445317782406268;
  _b[9][6]   = 0.10715724786209388876739304914053506;
  _b[9][7]   =-0.02177735985419485163815426357369818;
  _b[9][8]   =-0.10579849950964443770179884616296721;
  _b[10][0]  =-0.02540141041535143673515871979014924;
  _b[10][5]  = 1.0/30.0;
  _b[10][6]  =-0.16404854760069182073503553020238782;
  _b[10][7]  = 0.03410548898794737788891414566528526;
  _b[10][8]  = 0.15836825014108792658008718465091487;
  _b[10][9]  = 0.21425115805975734472868683695127609;
  _b[11][0]  = 0.00584833331460742801095934302256470;
  _b[11][5]  =-0.53954170547283522916525526480339109;
  _b[11][6]  = 0.20128430845560909506500331018201158;
  _b[11][7]  = 0.04347222773254789483240207937678906;
  _b[11][8]  =-0.00402998571475307250775349983910179;
  _b[11][9]  = 0.16541535721570612771420482097898952;
  _b[11][10] = 0.79491862412512344573322086551518180;
  _b[12][0]  =-0.39964965968794892497157706711861448;
  _b[12][5]  =-3.79096577568393158554742638116249372;
  _b[12][6]  =-0.40349325653530103387515807815498044;
  _b[12][7]  =-2.82463879530435263378049668286220715;
  _b[12][8]  = 1.04226892772185985533374283289821416;
  _b[12][9]  = 1.12510956420436603974237036536924078;
  _b[12][10] = 3.32746188718986816186934832571938138;
  _b[12][11] = 2.77897957186355606325818219255783627;
  _b[13][0]  = 0.39545306350085237157098218205756922;
  _b[13][5]  = 5.82534730759650564865380791881446903;
  _b[13][6]  =-0.36527452339161313311889856846974452;
  _b[13][7]  = 1.18860324058346533283780076203192232;
  _b[13][8]  = 0.57970467638357921347110271762687972;
  _b[13][9]  =-0.86824862589087693262676988867897834;
  _b[13][10] =-5.20227677296454721392873650976792184;
  _b[13][11] =-0.79895541420753382543211121058675915;
  _b[13][12] = 0.14360623206363792632792463778889008;
  _b[14][0]  = 8.49173149061346398013352206978380938;
  _b[14][5]  = 86.32213734729036800877634194386790750;
  _b[14][6]  = 1.02560575501091662034511526187393241;
  _b[14][7]  = 85.77427969817339941806831550695235092;
  _b[14][8]  =-13.98699305104110611795532466113248067;
  _b[14][9]  =-20.71537405501426352265946477613161883;
  _b[14][10] =-72.16597156619946800281180102605140463;
  _b[14][11] =-76.71211139107806345587696023064419687;
  _b[14][12] = 4.22319427707298828839851258893735507;
  _b[14][13] =-1.25649850482823521641825667745565428;
  _b[15][0]  =-0.42892119881959353241190195318730008;
  _b[15][5]  =-9.16865700950084689999297912545025359;
  _b[15][6]  = 1.08317616770620939241547721530003920;
  _b[15][7]  =-1.23501525358323653198215832293981810;
  _b[15][8]  =-1.21438272617593906232943856422371019;
  _b[15][9]  = 1.37226168507232166621351243731869914;
  _b[15][10] = 9.15723239697162418155377135344394113;
  _b[15][11] = 1.30616301842220047563298585480401671;
  _b[15][12] =-0.25285618808937955976690569433069974;
  _b[15][13] = 0.38099910799663987066763679926508552;
  
  _c[0]  = 0.01490902081978461022483617102382552;
  _c[7]  =-0.20408044692054151258349120934134791;
  _c[8]  = 0.22901438600570447264772469337066476;
  _c[9]  = 0.12800558251147375669208211573729202;
  _c[10] = 0.22380626846054143649770066956485937;
  _c[11] = 0.39553165293700054420552389156421651;
  _c[12] = 0.05416646758806981196568364538360743;
  _c[13] = 0.12691439652445903685643385312168037;
  _c[14] =-0.00052539244262118876455834655383035;
  _c[15] = 1.0/31.0;
  
  _cs[0]  = 0.00653047880643482012034413441159249;
  _cs[7]  =-2.31471038197461347517552506241529830;
  _cs[8]  = 0.43528227238866280799530900822377013;
  _cs[9]  = 0.14907947287101933118545845390618763;
  _cs[10] = 0.17905535442235532311850533252768020;
  _cs[11] = 2.53400872222767706921176214508820825;
  _cs[12] =-0.55430437423209112896721332268159015;
  _cs[13] = 0.56924788787870083224213506297615260;
  _cs[14] =-0.03644749690427461198884026816573513;
  _cs[15] = 1.0/31.0;
  
  _d[0] = 1.0-_b[15][5]/_b[14][5];
  _d[1] = _b[15][0]-_b[14][0]*_b[15][5]/_b[14][5];
  _d[2] = _b[15][5]/_b[14][5];
  _d[3] = _b[15][6]-_b[14][6]*_b[15][5]/_b[14][5];
  _d[4] = _b[15][7]-_b[14][7]*_b[15][5]/_b[14][5];
  _d[5] = _b[15][8]-_b[14][8]*_b[15][5]/_b[14][5];
  _d[6] = _b[15][9]-_b[14][9]*_b[15][5]/_b[14][5];
  _d[7] = _b[15][10]-_b[14][10]*_b[15][5]/_b[14][5];
  _d[8] = _b[15][11]-_b[14][11]*_b[15][5]/_b[14][5];
  _d[9] = _b[15][12]-_b[14][12]*_b[15][5]/_b[14][5];
  _d[10] = _b[15][13]-_b[14][13]*_b[15][5]/_b[14][5];
  
  do {
    
    do {
      
      
      // Step 1
      
      memcpy(_akafield_abcd_density_matrix, _abcd_density_matrix, sizeof(complex) * _abcd_density_matrix_alloc_size);
      
      _segment1_calculate_nonconstant_ip_fields(_step, 1);
      
      // a_i=D(a_2*dt)[y1]
      _segment1_ip_evolve(1);
      
      _active_abcd_density_matrix = _akafield_abcd_density_matrix;
      
      // a_k=G[a_k, t]
      _segment1_calculate_delta_a(_step);
      
      // a_i=D(a_2*dt)[y1]
      _segment1_ip_evolve(1);
      
      // Step 2
      
      t += _a[1] * _step;
      
      {
        _MAKE_AUTOVEC_VARIABLE(_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akafield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akbfield_abcd_density_matrix);
        #pragma ivdep
        for (long _i0 = 0; _i0 < 2 * (_lattice_a * _lattice_b * _lattice_c * _lattice_d) * _abcd_density_matrix_ncomponents; _i0++) {
          _AUTOVEC(_akbfield_abcd_density_matrix)[_i0] = _AUTOVEC(_abcd_density_matrix)[_i0] + _b[1][0]*_AUTOVEC(_akafield_abcd_density_matrix)[_i0];
        }
      
      }
      
      _active_abcd_density_matrix = _akbfield_abcd_density_matrix;
      
      _segment1_calculate_nonconstant_ip_fields(_step, 2);
      
      // a_i=D(a_2*dt)[y1]
      _segment1_ip_evolve(-2);
      
      // a_k=G[a_k, t]
      _segment1_calculate_delta_a(_step);
      
      // a_i=D(a_2*dt)[y1]
      _segment1_ip_evolve(2);
      
      // Step 3
      
      t += _a[2] * _step;
      
      {
        _MAKE_AUTOVEC_VARIABLE(_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akafield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akcfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akbfield_abcd_density_matrix);
        #pragma ivdep
        for (long _i0 = 0; _i0 < 2 * (_lattice_a * _lattice_b * _lattice_c * _lattice_d) * _abcd_density_matrix_ncomponents; _i0++) {
          _AUTOVEC(_akcfield_abcd_density_matrix)[_i0] = _AUTOVEC(_abcd_density_matrix)[_i0] + _b[2][0]*_AUTOVEC(_akafield_abcd_density_matrix)[_i0] + _b[2][1]*_AUTOVEC(_akbfield_abcd_density_matrix)[_i0];
        }
      
      }
      
      _active_abcd_density_matrix = _akcfield_abcd_density_matrix;
      
      _segment1_calculate_nonconstant_ip_fields(_step, 3);
      
      // a_i=D(a_2*dt)[y1]
      _segment1_ip_evolve(-3);
      
      // a_k=G[a_k, t]
      _segment1_calculate_delta_a(_step);
      
      // a_i=D(a_2*dt)[y1]
      _segment1_ip_evolve(3);
      
      // Step 4
      
      t += _a[3] * _step;
      
      {
        _MAKE_AUTOVEC_VARIABLE(_akcfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akbfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akdfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akafield_abcd_density_matrix);
        #pragma ivdep
        for (long _i0 = 0; _i0 < 2 * (_lattice_a * _lattice_b * _lattice_c * _lattice_d) * _abcd_density_matrix_ncomponents; _i0++) {
          _AUTOVEC(_akdfield_abcd_density_matrix)[_i0] = _AUTOVEC(_abcd_density_matrix)[_i0] + _b[3][0]*_AUTOVEC(_akafield_abcd_density_matrix)[_i0] + _b[3][1]*_AUTOVEC(_akbfield_abcd_density_matrix)[_i0]
              + _b[3][2]*_AUTOVEC(_akcfield_abcd_density_matrix)[_i0];
        }
      
      }
      
      _active_abcd_density_matrix = _akdfield_abcd_density_matrix;
      
      _segment1_calculate_nonconstant_ip_fields(_step, 4);
      
      // a_i=D(a_2*dt)[y1]
      _segment1_ip_evolve(-4);
      
      // a_k=G[a_k, t]
      _segment1_calculate_delta_a(_step);
      
      // a_i=D(a_2*dt)[y1]
      _segment1_ip_evolve(4);
      
      // Step 5
      
      t += _a[4] * _step;
      
      {
        _MAKE_AUTOVEC_VARIABLE(_akefield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akcfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akbfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akdfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akafield_abcd_density_matrix);
        #pragma ivdep
        for (long _i0 = 0; _i0 < 2 * (_lattice_a * _lattice_b * _lattice_c * _lattice_d) * _abcd_density_matrix_ncomponents; _i0++) {
          _AUTOVEC(_akefield_abcd_density_matrix)[_i0] = _AUTOVEC(_abcd_density_matrix)[_i0] + _b[4][0]*_AUTOVEC(_akafield_abcd_density_matrix)[_i0] + _b[4][1]*_AUTOVEC(_akbfield_abcd_density_matrix)[_i0]
              + _b[4][2]*_AUTOVEC(_akcfield_abcd_density_matrix)[_i0] + _b[4][3]*_AUTOVEC(_akdfield_abcd_density_matrix)[_i0];
        }
      
      }
      
      _active_abcd_density_matrix = _akefield_abcd_density_matrix;
      
      _segment1_calculate_nonconstant_ip_fields(_step, 5);
      
      // a_i=D(a_2*dt)[y1]
      _segment1_ip_evolve(-5);
      
      // a_k=G[a_k, t]
      _segment1_calculate_delta_a(_step);
      
      // a_i=D(a_2*dt)[y1]
      _segment1_ip_evolve(5);
      
      // Step 6
      
      t += _a[5] * _step;
      
      {
        _MAKE_AUTOVEC_VARIABLE(_akifield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akefield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akdfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akafield_abcd_density_matrix);
        #pragma ivdep
        for (long _i0 = 0; _i0 < 2 * (_lattice_a * _lattice_b * _lattice_c * _lattice_d) * _abcd_density_matrix_ncomponents; _i0++) {
          _AUTOVEC(_akifield_abcd_density_matrix)[_i0] = _AUTOVEC(_abcd_density_matrix)[_i0] + _b[5][0]*_AUTOVEC(_akafield_abcd_density_matrix)[_i0] + _b[5][3]*_AUTOVEC(_akdfield_abcd_density_matrix)[_i0]
              + _b[5][4]*_AUTOVEC(_akefield_abcd_density_matrix)[_i0];
        }
      
      }
      
      _active_abcd_density_matrix = _akifield_abcd_density_matrix;
      
      _segment1_calculate_nonconstant_ip_fields(_step, 6);
      
      // a_i=D(a_2*dt)[y1]
      _segment1_ip_evolve(-6);
      
      // a_k=G[a_k, t]
      _segment1_calculate_delta_a(_step);
      
      // a_i=D(a_2*dt)[y1]
      _segment1_ip_evolve(6);
      
      // Step 7
      
      t += _a[6] * _step;
      
      {
        _MAKE_AUTOVEC_VARIABLE(_akifield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akefield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akdfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akjfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akafield_abcd_density_matrix);
        #pragma ivdep
        for (long _i0 = 0; _i0 < 2 * (_lattice_a * _lattice_b * _lattice_c * _lattice_d) * _abcd_density_matrix_ncomponents; _i0++) {
          _AUTOVEC(_akjfield_abcd_density_matrix)[_i0] = _AUTOVEC(_abcd_density_matrix)[_i0] + _b[6][0]*_AUTOVEC(_akafield_abcd_density_matrix)[_i0] + _b[6][3]*_AUTOVEC(_akdfield_abcd_density_matrix)[_i0]
              + _b[6][4]*_AUTOVEC(_akefield_abcd_density_matrix)[_i0] + _b[6][5]*_AUTOVEC(_akifield_abcd_density_matrix)[_i0];
        }
      
      }
      
      _active_abcd_density_matrix = _akjfield_abcd_density_matrix;
      
      _segment1_calculate_nonconstant_ip_fields(_step, 7);
      
      // a_i=D(a_2*dt)[y1]
      _segment1_ip_evolve(-7);
      
      // a_k=G[a_k, t]
      _segment1_calculate_delta_a(_step);
      
      // a_i=D(a_2*dt)[y1]
      _segment1_ip_evolve(7);
      
      // Step 8
      
      t += _a[7] * _step;
      
      {
        _MAKE_AUTOVEC_VARIABLE(_akifield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akbfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akjfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akafield_abcd_density_matrix);
        #pragma ivdep
        for (long _i0 = 0; _i0 < 2 * (_lattice_a * _lattice_b * _lattice_c * _lattice_d) * _abcd_density_matrix_ncomponents; _i0++) {
          _AUTOVEC(_akbfield_abcd_density_matrix)[_i0] = _AUTOVEC(_abcd_density_matrix)[_i0] + _b[7][0]*_AUTOVEC(_akafield_abcd_density_matrix)[_i0] + _b[7][5]*_AUTOVEC(_akifield_abcd_density_matrix)[_i0]
              + _b[7][6]*_AUTOVEC(_akjfield_abcd_density_matrix)[_i0];
        }
      
      }
      
      _active_abcd_density_matrix = _akbfield_abcd_density_matrix;
      
      _segment1_calculate_nonconstant_ip_fields(_step, 8);
      
      // a_i=D(a_2*dt)[y1]
      _segment1_ip_evolve(-8);
      
      // a_k=G[a_k, t]
      _segment1_calculate_delta_a(_step);
      
      // a_i=D(a_2*dt)[y1]
      _segment1_ip_evolve(8);
      
      // Step 9
      
      t += _a[8] * _step;
      
      {
        _MAKE_AUTOVEC_VARIABLE(_akifield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akcfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akbfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akjfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akafield_abcd_density_matrix);
        #pragma ivdep
        for (long _i0 = 0; _i0 < 2 * (_lattice_a * _lattice_b * _lattice_c * _lattice_d) * _abcd_density_matrix_ncomponents; _i0++) {
          _AUTOVEC(_akcfield_abcd_density_matrix)[_i0] = _AUTOVEC(_abcd_density_matrix)[_i0] + _b[8][0]*_AUTOVEC(_akafield_abcd_density_matrix)[_i0] + _b[8][5]*_AUTOVEC(_akifield_abcd_density_matrix)[_i0]
              + _b[8][6]*_AUTOVEC(_akjfield_abcd_density_matrix)[_i0]+ _b[8][7]*_AUTOVEC(_akbfield_abcd_density_matrix)[_i0];
        }
      
      }
      
      _active_abcd_density_matrix = _akcfield_abcd_density_matrix;
      
      _segment1_calculate_nonconstant_ip_fields(_step, 9);
      
      // a_i=D(a_2*dt)[y1]
      _segment1_ip_evolve(-9);
      
      // a_k=G[a_k, t]
      _segment1_calculate_delta_a(_step);
      
      // a_i=D(a_2*dt)[y1]
      _segment1_ip_evolve(9);
      
      // Step 10
      
      t += _a[9] * _step;
      
      {
        _MAKE_AUTOVEC_VARIABLE(_akifield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akcfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akbfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akdfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akjfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akafield_abcd_density_matrix);
        #pragma ivdep
        for (long _i0 = 0; _i0 < 2 * (_lattice_a * _lattice_b * _lattice_c * _lattice_d) * _abcd_density_matrix_ncomponents; _i0++) {
          _AUTOVEC(_akdfield_abcd_density_matrix)[_i0] = _AUTOVEC(_abcd_density_matrix)[_i0] + _b[9][0]*_AUTOVEC(_akafield_abcd_density_matrix)[_i0] + _b[9][5]*_AUTOVEC(_akifield_abcd_density_matrix)[_i0]
              + _b[9][6]*_AUTOVEC(_akjfield_abcd_density_matrix)[_i0]+ _b[9][7]*_AUTOVEC(_akbfield_abcd_density_matrix)[_i0]+ _b[9][8]*_AUTOVEC(_akcfield_abcd_density_matrix)[_i0];
        }
      
      }
      
      _active_abcd_density_matrix = _akdfield_abcd_density_matrix;
      
      _segment1_calculate_nonconstant_ip_fields(_step, 10);
      
      // a_i=D(a_2*dt)[y1]
      _segment1_ip_evolve(-10);
      
      // a_k=G[a_k, t]
      _segment1_calculate_delta_a(_step);
      
      // a_i=D(a_2*dt)[y1]
      _segment1_ip_evolve(10);
      
      // Step 11
      
      t += _a[10] * _step;
      
      {
        _MAKE_AUTOVEC_VARIABLE(_akifield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akefield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akcfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akbfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akdfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akjfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akafield_abcd_density_matrix);
        #pragma ivdep
        for (long _i0 = 0; _i0 < 2 * (_lattice_a * _lattice_b * _lattice_c * _lattice_d) * _abcd_density_matrix_ncomponents; _i0++) {
          _AUTOVEC(_akefield_abcd_density_matrix)[_i0] = _AUTOVEC(_abcd_density_matrix)[_i0] + _b[10][0]*_AUTOVEC(_akafield_abcd_density_matrix)[_i0] + _b[10][5]*_AUTOVEC(_akifield_abcd_density_matrix)[_i0]
              + _b[10][6]*_AUTOVEC(_akjfield_abcd_density_matrix)[_i0]+ _b[10][7]*_AUTOVEC(_akbfield_abcd_density_matrix)[_i0] + _b[10][8]*_AUTOVEC(_akcfield_abcd_density_matrix)[_i0]
              + _b[10][9]*_AUTOVEC(_akdfield_abcd_density_matrix)[_i0];
        }
      
      }
      
      _active_abcd_density_matrix = _akefield_abcd_density_matrix;
      
      _segment1_calculate_nonconstant_ip_fields(_step, 11);
      
      // a_i=D(a_2*dt)[y1]
      _segment1_ip_evolve(-11);
      
      // a_k=G[a_k, t]
      _segment1_calculate_delta_a(_step);
      
      // a_i=D(a_2*dt)[y1]
      _segment1_ip_evolve(11);
      
      // Step 12
      
      t += _a[11] * _step;
      
      {
        _MAKE_AUTOVEC_VARIABLE(_akifield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akcfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akefield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akbfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akdfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akjfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akafield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akffield_abcd_density_matrix);
        #pragma ivdep
        for (long _i0 = 0; _i0 < 2 * (_lattice_a * _lattice_b * _lattice_c * _lattice_d) * _abcd_density_matrix_ncomponents; _i0++) {
          _AUTOVEC(_akffield_abcd_density_matrix)[_i0] = _AUTOVEC(_abcd_density_matrix)[_i0] + _b[11][0]*_AUTOVEC(_akafield_abcd_density_matrix)[_i0] + _b[11][5]*_AUTOVEC(_akifield_abcd_density_matrix)[_i0]
              + _b[11][6]*_AUTOVEC(_akjfield_abcd_density_matrix)[_i0] + _b[11][7]*_AUTOVEC(_akbfield_abcd_density_matrix)[_i0] + _b[11][8]*_AUTOVEC(_akcfield_abcd_density_matrix)[_i0]
              + _b[11][9]*_AUTOVEC(_akdfield_abcd_density_matrix)[_i0] + _b[11][10]*_AUTOVEC(_akefield_abcd_density_matrix)[_i0];
        }
      
      }
      
      _active_abcd_density_matrix = _akffield_abcd_density_matrix;
      
      _segment1_calculate_nonconstant_ip_fields(_step, 12);
      
      // a_i=D(a_2*dt)[y1]
      _segment1_ip_evolve(-12);
      
      // a_k=G[a_k, t]
      _segment1_calculate_delta_a(_step);
      
      // a_i=D(a_2*dt)[y1]
      _segment1_ip_evolve(12);
      
      // Step 13
      
      t += _a[12] * _step;
      
      {
        _MAKE_AUTOVEC_VARIABLE(_akifield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akcfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akefield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akbfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akgfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akdfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akjfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akafield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akffield_abcd_density_matrix);
        #pragma ivdep
        for (long _i0 = 0; _i0 < 2 * (_lattice_a * _lattice_b * _lattice_c * _lattice_d) * _abcd_density_matrix_ncomponents; _i0++) {
          _AUTOVEC(_akgfield_abcd_density_matrix)[_i0] = _AUTOVEC(_abcd_density_matrix)[_i0] + _b[12][0]*_AUTOVEC(_akafield_abcd_density_matrix)[_i0] + _b[12][5]*_AUTOVEC(_akifield_abcd_density_matrix)[_i0]
              + _b[12][6]*_AUTOVEC(_akjfield_abcd_density_matrix)[_i0]+ _b[12][7]*_AUTOVEC(_akbfield_abcd_density_matrix)[_i0] + _b[12][8]*_AUTOVEC(_akcfield_abcd_density_matrix)[_i0]
              + _b[12][9]*_AUTOVEC(_akdfield_abcd_density_matrix)[_i0] + _b[12][10]*_AUTOVEC(_akefield_abcd_density_matrix)[_i0] + _b[12][11]*_AUTOVEC(_akffield_abcd_density_matrix)[_i0];
        }
      
      }
      
      _active_abcd_density_matrix = _akgfield_abcd_density_matrix;
      
      _segment1_calculate_nonconstant_ip_fields(_step, 13);
      
      // a_i=D(a_2*dt)[y1]
      _segment1_ip_evolve(-13);
      
      // a_k=G[a_k, t]
      _segment1_calculate_delta_a(_step);
      
      // a_i=D(a_2*dt)[y1]
      _segment1_ip_evolve(13);
      
      // Step 14
      
      t += _a[13] * _step;
      
      {
        _MAKE_AUTOVEC_VARIABLE(_akifield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akcfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akefield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akhfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akbfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akgfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akdfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akjfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akafield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akffield_abcd_density_matrix);
        #pragma ivdep
        for (long _i0 = 0; _i0 < 2 * (_lattice_a * _lattice_b * _lattice_c * _lattice_d) * _abcd_density_matrix_ncomponents; _i0++) {
          _AUTOVEC(_akhfield_abcd_density_matrix)[_i0] = _AUTOVEC(_abcd_density_matrix)[_i0] + _b[13][0]*_AUTOVEC(_akafield_abcd_density_matrix)[_i0] + _b[13][5]*_AUTOVEC(_akifield_abcd_density_matrix)[_i0]
              + _b[13][6]*_AUTOVEC(_akjfield_abcd_density_matrix)[_i0]+ _b[13][7]*_AUTOVEC(_akbfield_abcd_density_matrix)[_i0] + _b[13][8]*_AUTOVEC(_akcfield_abcd_density_matrix)[_i0]
              + _b[13][9]*_AUTOVEC(_akdfield_abcd_density_matrix)[_i0] + _b[13][10]*_AUTOVEC(_akefield_abcd_density_matrix)[_i0] + _b[13][11]*_AUTOVEC(_akffield_abcd_density_matrix)[_i0]
              + _b[13][12]*_AUTOVEC(_akgfield_abcd_density_matrix)[_i0];
        }
      
      }
      
      _active_abcd_density_matrix = _akhfield_abcd_density_matrix;
      
      _segment1_calculate_nonconstant_ip_fields(_step, 14);
      
      // a_i=D(a_2*dt)[y1]
      _segment1_ip_evolve(-14);
      
      // a_k=G[a_k, t]
      _segment1_calculate_delta_a(_step);
      
      // a_i=D(a_2*dt)[y1]
      _segment1_ip_evolve(14);
      
      // Step 15 and 16 combined to reduce memory use 
      
      {
        _MAKE_AUTOVEC_VARIABLE(_akifield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akcfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akefield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akbfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akhfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akgfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akdfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akjfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akafield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akffield_abcd_density_matrix);
        #pragma ivdep
        for (long _i0 = 0; _i0 < 2 * (_lattice_a * _lattice_b * _lattice_c * _lattice_d) * _abcd_density_matrix_ncomponents; _i0++) {
          _AUTOVEC(_akifield_abcd_density_matrix)[_i0] = _AUTOVEC(_abcd_density_matrix)[_i0] + _b[14][0]*_AUTOVEC(_akafield_abcd_density_matrix)[_i0] + _b[14][5]*_AUTOVEC(_akifield_abcd_density_matrix)[_i0]
              + _b[14][6]*_AUTOVEC(_akjfield_abcd_density_matrix)[_i0]+ _b[14][7]*_AUTOVEC(_akbfield_abcd_density_matrix)[_i0] + _b[14][8]*_AUTOVEC(_akcfield_abcd_density_matrix)[_i0]
              + _b[14][9]*_AUTOVEC(_akdfield_abcd_density_matrix)[_i0] + _b[14][10]*_AUTOVEC(_akefield_abcd_density_matrix)[_i0] + _b[14][11]*_AUTOVEC(_akffield_abcd_density_matrix)[_i0]
              + _b[14][12]*_AUTOVEC(_akgfield_abcd_density_matrix)[_i0] + _b[14][13]*_AUTOVEC(_akhfield_abcd_density_matrix)[_i0];
        }
      
      }
      
      {
        _MAKE_AUTOVEC_VARIABLE(_akifield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akcfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akefield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akbfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akhfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akgfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akdfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akjfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akafield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akffield_abcd_density_matrix);
        #pragma ivdep
        for (long _i0 = 0; _i0 < 2 * (_lattice_a * _lattice_b * _lattice_c * _lattice_d) * _abcd_density_matrix_ncomponents; _i0++) {
          _AUTOVEC(_akjfield_abcd_density_matrix)[_i0] = _d[0]*_AUTOVEC(_abcd_density_matrix)[_i0]
                + _d[1]*_AUTOVEC(_akafield_abcd_density_matrix)[_i0]
                + _d[2]*_AUTOVEC(_akifield_abcd_density_matrix)[_i0]
                + _d[3]*_AUTOVEC(_akjfield_abcd_density_matrix)[_i0]
                + _d[4]*_AUTOVEC(_akbfield_abcd_density_matrix)[_i0]
                + _d[5]*_AUTOVEC(_akcfield_abcd_density_matrix)[_i0]
                + _d[6]*_AUTOVEC(_akdfield_abcd_density_matrix)[_i0]
                + _d[7]*_AUTOVEC(_akefield_abcd_density_matrix)[_i0]
                + _d[8]*_AUTOVEC(_akffield_abcd_density_matrix)[_i0]
                + _d[9]*_AUTOVEC(_akgfield_abcd_density_matrix)[_i0]
                + _d[10]*_AUTOVEC(_akhfield_abcd_density_matrix)[_i0];
        }
      
      }
      
      t += _a[14] * _step;
      
      _active_abcd_density_matrix = _akifield_abcd_density_matrix;
      
      // a_k=G[a_k, t]
      _segment1_calculate_delta_a(_step);
      
      t += _a[15] * _step;
      
      _active_abcd_density_matrix = _akjfield_abcd_density_matrix;
      
      // a_k=G[a_k, t]
      _segment1_calculate_delta_a(_step);
      
      // Take full step
      
      // ai = a
      memcpy(_initial_abcd_density_matrix, _abcd_density_matrix, sizeof(complex) * _abcd_density_matrix_alloc_size);
      
      // a = a + etc
      {
        _MAKE_AUTOVEC_VARIABLE(_akifield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akcfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akefield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akbfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akhfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akgfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akdfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akjfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akafield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akffield_abcd_density_matrix);
        #pragma ivdep
        for (long _i0 = 0; _i0 < 2 * (_lattice_a * _lattice_b * _lattice_c * _lattice_d) * _abcd_density_matrix_ncomponents; _i0++) {
          _AUTOVEC(_abcd_density_matrix)[_i0] += _c[0]*_AUTOVEC(_akafield_abcd_density_matrix)[_i0] + _c[7]*_AUTOVEC(_akbfield_abcd_density_matrix)[_i0] + _c[8]*_AUTOVEC(_akcfield_abcd_density_matrix)[_i0]
              + _c[9]*_AUTOVEC(_akdfield_abcd_density_matrix)[_i0] + _c[10]*_AUTOVEC(_akefield_abcd_density_matrix)[_i0] + _c[11]*_AUTOVEC(_akffield_abcd_density_matrix)[_i0]
              + _c[12]*_AUTOVEC(_akgfield_abcd_density_matrix)[_i0] + _c[13]*_AUTOVEC(_akhfield_abcd_density_matrix)[_i0] + _c[14]*_AUTOVEC(_akifield_abcd_density_matrix)[_i0]
              + _c[15]*_AUTOVEC(_akjfield_abcd_density_matrix)[_i0];
        }
      
      }
      
      // a* = a + etc
      {
        _MAKE_AUTOVEC_VARIABLE(_akifield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akcfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akefield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akbfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akhfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_initial_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akgfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akdfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akjfield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akafield_abcd_density_matrix);
        _MAKE_AUTOVEC_VARIABLE(_akffield_abcd_density_matrix);
        #pragma ivdep
        for (long _i0 = 0; _i0 < 2 * (_lattice_a * _lattice_b * _lattice_c * _lattice_d) * _abcd_density_matrix_ncomponents; _i0++) {
          _AUTOVEC(_akafield_abcd_density_matrix)[_i0] = _AUTOVEC(_initial_abcd_density_matrix)[_i0] + _cs[0]*_AUTOVEC(_akafield_abcd_density_matrix)[_i0] + _cs[7]*_AUTOVEC(_akbfield_abcd_density_matrix)[_i0]
              + _cs[8]*_AUTOVEC(_akcfield_abcd_density_matrix)[_i0] + _cs[9]*_AUTOVEC(_akdfield_abcd_density_matrix)[_i0] + _cs[10]*_AUTOVEC(_akefield_abcd_density_matrix)[_i0]
              + _cs[11]*_AUTOVEC(_akffield_abcd_density_matrix)[_i0] + _cs[12]*_AUTOVEC(_akgfield_abcd_density_matrix)[_i0] + _cs[13]*_AUTOVEC(_akhfield_abcd_density_matrix)[_i0]
              + _cs[14]*_AUTOVEC(_akifield_abcd_density_matrix)[_i0] + _cs[15]*_AUTOVEC(_akjfield_abcd_density_matrix)[_i0];
        }
      
      }
      
      _active_abcd_density_matrix = _abcd_density_matrix;
      
      
      
      _error = 0.0;
      
      _segment1_abcd_density_matrix_error = _segment1_abcd_density_matrix_timestep_error(_akafield_abcd_density_matrix);
      if (_segment1_abcd_density_matrix_error > _error)
        _error = _segment1_abcd_density_matrix_error;
      
      _attempted_steps++;
      
      if (_error < _tolerance) {
        _t_local += _step;
        if (_step > _max_step)
          _max_step = _step;
        if (!_break_next && _step < _min_step)
          _min_step = _step;
        _discard = false;
      } else {
        t -= _step;
  
        if (_segment1_abcd_density_matrix_reset(_initial_abcd_density_matrix) == false) {
  
          _LOG(_WARNING_LOG_LEVEL, "WARNING: NaN present. Integration halted at t = %e.\n"
                             "         Non-finite number in integration vector \"density_matrix\" in segment 1.\n", t);
          if (_mg0_output_index_t < _mg0_output_lattice_t)
            _mg0_sample();
          if (_mg1_output_index_t < _mg1_output_lattice_t)
            _mg1_sample();
          if (_mg2_output_index_t < _mg2_output_lattice_t)
            _mg2_sample();
          if (_mg3_output_index_t < _mg3_output_lattice_t)
            _mg3_sample();
          if (_mg4_output_index_t < _mg4_output_lattice_t)
            _mg4_sample();
          
          goto _SEGMENT1_END;
        }
  
        _segment1_ip_evolve(-1);
        
        _discard = true;
        _break_next = false;
        _unsuccessful_steps++;
      }
      
      _old_step = _step;
      
      // Resize step
      if (_error < 0.5*_tolerance || _error > _tolerance) {
        const real _safetyFactor = 0.90;
        real _scalingFactor = _safetyFactor * pow(abs(_error/_tolerance), real(-0.7/9.0)) * pow(_last_norm_error, real(0.4/9.0));
        _scalingFactor = MAX(_scalingFactor, 1.0/5.0);
        _scalingFactor = MIN(_scalingFactor, 7.0);
        if (_error > _tolerance && _scalingFactor > 1.0) {
          // If our step failed don't try and increase our step size. That would be silly.
          _scalingFactor = _safetyFactor * pow(abs(_error/_tolerance), real(-1.0/9.0));
        }
        _old_step = _step;
        _last_norm_error = pow(_safetyFactor/_scalingFactor*pow(_last_norm_error, real(0.4/9.0)), real(9.0/0.7));
        _step *= _scalingFactor;
      }
      
    } while (_discard);
    
    if (_break_next) {
      if (_next_sample_flag[0]) {
        _mg0_sample();
        _next_sample_counter[0]++;
      }
      if (_next_sample_flag[1]) {
        _mg1_sample();
        _next_sample_counter[1]++;
      }
      if (_next_sample_flag[2]) {
        _mg2_sample();
        _next_sample_counter[2]++;
      }
      if (_next_sample_flag[3]) {
        _mg3_sample();
        _next_sample_counter[3]++;
      }
      if (_next_sample_flag[4]) {
        _mg4_sample();
        _next_sample_counter[4]++;
      }
      if (_next_sample_flag[5])
        _next_sample_flag[6] = true;
      else {
        _break_next = false;
        _t_break_next = _segment1_setup_sampling(_next_sample_flag, _next_sample_counter);
      }
    }
    
    if ( (_t_local + _step)*(1.0 + _EPSILON) > _t_break_next) {
      _break_next = true;
      _LOG(_SAMPLE_LOG_LEVEL, "Current timestep: %e\n", _old_step);
      _step = _t_break_next - _t_local;
    }
  } while (!_next_sample_flag[6]);
  
  _SEGMENT1_END:;
  
  xmds_free(_segment1_operator0_delta_a_field_density_matrix);
  _active_segment1_operator0_delta_a_field_density_matrix = _segment1_operator0_delta_a_field_density_matrix = NULL;
  xmds_free(_segment1_akafield_abcd_density_matrix);
  xmds_free(_segment1_akbfield_abcd_density_matrix);
  xmds_free(_segment1_akcfield_abcd_density_matrix);
  xmds_free(_segment1_akdfield_abcd_density_matrix);
  xmds_free(_segment1_akefield_abcd_density_matrix);
  xmds_free(_segment1_akffield_abcd_density_matrix);
  xmds_free(_segment1_akgfield_abcd_density_matrix);
  xmds_free(_segment1_akhfield_abcd_density_matrix);
  xmds_free(_segment1_akifield_abcd_density_matrix);
  xmds_free(_segment1_akjfield_abcd_density_matrix);
  xmds_free(_segment1_initial_abcd_density_matrix);
  
  _LOG(_SEGMENT_LOG_LEVEL, "Segment 1: minimum timestep: %e maximum timestep: %e\n", _min_step, _max_step);
  _LOG(_SEGMENT_LOG_LEVEL, "  Attempted %li steps, %.2f%% steps failed.\n", _attempted_steps, (100.0*_unsuccessful_steps)/_attempted_steps);
}


inline void _segment1_calculate_delta_a(real _step)
{
  _dimensionless_time_dependent_coupling_evaluate();
  
  
  // Delta A propagation operator for field abcd
  _segment1_abcd_operators_evaluate_operator0(_step);
  
}


inline void _segment1_ip_evolve(int _exponent)
{
}


inline void _segment1_calculate_nonconstant_ip_fields(real _step, int _exponent)
{
}
real _segment1_setup_sampling(bool* _next_sample_flag, long* _next_sample_counter)
{
  // The numbers of the moment groups that need to be sampled at the next sampling point.
  // An entry of N+1 means "reached end of integration interval"
  long _momentGroupNumbersNeedingSamplingNext[6];
  long _numberOfMomentGroupsToBeSampledNext = 1;
  
  long _previous_m = 1;
  long _previous_M = 1;
  
  real _t_break_next = (real)2.5;
  _momentGroupNumbersNeedingSamplingNext[0] = 5;
  
  // initialise all flags to false
  for (long _i0 = 0; _i0 < 6; _i0++)
    _next_sample_flag[_i0] = false;
  
  /* Check if moment group needs sampling at the same time as another already discovered sample (or the final time).
   * If so, add this moment group to the to-be-sampled list. If moment group demands sampling earlier than all
   * previously noted moment groups, erase all previous ones from list and set the sample time to this earlier one.
   */
  if (_next_sample_counter[0] * _previous_M == _previous_m * 200) {
    _momentGroupNumbersNeedingSamplingNext[_numberOfMomentGroupsToBeSampledNext] = 0;
    _numberOfMomentGroupsToBeSampledNext++;
  } else if (_next_sample_counter[0] * _previous_M < _previous_m * 200) {
    _t_break_next = _next_sample_counter[0] * ((real)2.5) / ((real)200);
    _numberOfMomentGroupsToBeSampledNext = 1;
    _momentGroupNumbersNeedingSamplingNext[0] = 0;
    _previous_M = 200;
    _previous_m = _next_sample_counter[0];
  }
  
  if (_next_sample_counter[1] * _previous_M == _previous_m * 200) {
    _momentGroupNumbersNeedingSamplingNext[_numberOfMomentGroupsToBeSampledNext] = 1;
    _numberOfMomentGroupsToBeSampledNext++;
  } else if (_next_sample_counter[1] * _previous_M < _previous_m * 200) {
    _t_break_next = _next_sample_counter[1] * ((real)2.5) / ((real)200);
    _numberOfMomentGroupsToBeSampledNext = 1;
    _momentGroupNumbersNeedingSamplingNext[0] = 1;
    _previous_M = 200;
    _previous_m = _next_sample_counter[1];
  }
  
  if (_next_sample_counter[2] * _previous_M == _previous_m * 200) {
    _momentGroupNumbersNeedingSamplingNext[_numberOfMomentGroupsToBeSampledNext] = 2;
    _numberOfMomentGroupsToBeSampledNext++;
  } else if (_next_sample_counter[2] * _previous_M < _previous_m * 200) {
    _t_break_next = _next_sample_counter[2] * ((real)2.5) / ((real)200);
    _numberOfMomentGroupsToBeSampledNext = 1;
    _momentGroupNumbersNeedingSamplingNext[0] = 2;
    _previous_M = 200;
    _previous_m = _next_sample_counter[2];
  }
  
  if (_next_sample_counter[3] * _previous_M == _previous_m * 200) {
    _momentGroupNumbersNeedingSamplingNext[_numberOfMomentGroupsToBeSampledNext] = 3;
    _numberOfMomentGroupsToBeSampledNext++;
  } else if (_next_sample_counter[3] * _previous_M < _previous_m * 200) {
    _t_break_next = _next_sample_counter[3] * ((real)2.5) / ((real)200);
    _numberOfMomentGroupsToBeSampledNext = 1;
    _momentGroupNumbersNeedingSamplingNext[0] = 3;
    _previous_M = 200;
    _previous_m = _next_sample_counter[3];
  }
  
  if (_next_sample_counter[4] * _previous_M == _previous_m * 200) {
    _momentGroupNumbersNeedingSamplingNext[_numberOfMomentGroupsToBeSampledNext] = 4;
    _numberOfMomentGroupsToBeSampledNext++;
  } else if (_next_sample_counter[4] * _previous_M < _previous_m * 200) {
    _t_break_next = _next_sample_counter[4] * ((real)2.5) / ((real)200);
    _numberOfMomentGroupsToBeSampledNext = 1;
    _momentGroupNumbersNeedingSamplingNext[0] = 4;
    _previous_M = 200;
    _previous_m = _next_sample_counter[4];
  }
  
  // _momentGroupNumbersNeedingSamplingNext now contains the complete list of moment groups that need
  // to be sampled at the next sampling point. Set their flags to true.
  for (long _i0 = 0; _i0 < _numberOfMomentGroupsToBeSampledNext; _i0++)
    _next_sample_flag[_momentGroupNumbersNeedingSamplingNext[_i0]] = true;
  
  return _t_break_next;
}

real _segment1_abcd_density_matrix_timestep_error(complex* _checkfield)
{
  real _error = 1e-24;
  real _temp_error = 0.0;
  real _temp_mod = 0.0;

  
  // Find the peak value for each component of the field
  real _cutoff[_abcd_density_matrix_ncomponents];
  
  for (long _i0 = 0; _i0 < _abcd_density_matrix_ncomponents; _i0++)
    _cutoff[_i0] = 0.0;
  
  {
    long _abcd_density_matrix_index_pointer = 0;
    #define rho _active_abcd_density_matrix[_abcd_density_matrix_index_pointer + 0]
    for (long _i0 = 0; _i0 < (_lattice_a * _lattice_b * _lattice_c * _lattice_d); _i0++) {
      for (long _i1 = 0; _i1 < _abcd_density_matrix_ncomponents; _i1++) {
        _temp_mod = mod2(_abcd_density_matrix[_abcd_density_matrix_index_pointer + _i1]);
        if (_xmds_isnonfinite(_temp_mod))
          _cutoff[_i1] = INFINITY;
        else if (_cutoff[_i1] < _temp_mod)
          _cutoff[_i1] = _temp_mod;
      }
    
      _abcd_density_matrix_index_pointer += _abcd_density_matrix_ncomponents;
    }
    #undef rho
  }
  
  for (long _i0 = 0; _i0 < _abcd_density_matrix_ncomponents; _i0++) {
    if (_xmds_isnonfinite(_cutoff[_i0]))
      // Return an error two times the tolerance in this case because the timestep must be reduced.
      return 2.0*1e-10;
    _cutoff[_i0] *= 0.001;
    _cutoff[_i0] *= 0.001;
  }
  
  {
    long _abcd_density_matrix_index_pointer = 0;
    #define rho _active_abcd_density_matrix[_abcd_density_matrix_index_pointer + 0]
    for (long _i0 = 0; _i0 < (_lattice_a * _lattice_b * _lattice_c * _lattice_d); _i0++) {
      for (long  _i1 = 0; _i1 < _abcd_density_matrix_ncomponents; _i1++) {
        if (mod2(_abcd_density_matrix[_abcd_density_matrix_index_pointer + _i1]) > _cutoff[_i1]) {
          _temp_error = abs(_abcd_density_matrix[_abcd_density_matrix_index_pointer + _i1] - _checkfield[_abcd_density_matrix_index_pointer + _i1]) / (0.5*abs(_abcd_density_matrix[_abcd_density_matrix_index_pointer + _i1]) + 0.5*abs(_checkfield[_abcd_density_matrix_index_pointer + _i1]));
          
          if (_xmds_isnonfinite(_temp_error)) {
            /* For _temp_error to be NaN, both the absolute value of the higher and lower order solutions
               must BOTH be zero. This therefore implies that their difference is zero, and that there is no error. */
            _temp_error = 0.0;
          }
          
          if (_error < _temp_error) // UNVECTORISABLE
            _error = _temp_error;
        }
      }
    
      _abcd_density_matrix_index_pointer += _abcd_density_matrix_ncomponents;
    }
    #undef rho
  }
  
  return _error;
}

bool _segment1_abcd_density_matrix_reset(complex* _reset_to_abcd_density_matrix)
{
  memcpy(_abcd_density_matrix, _reset_to_abcd_density_matrix, sizeof(complex) * _abcd_density_matrix_alloc_size);
  
  /* return false if there's a NaN somewhere in the vector, otherwise return true */
  bool bNoNaNsPresent = true;
  {
    long _abcd_density_matrix_index_pointer = 0;
    #define rho _active_abcd_density_matrix[_abcd_density_matrix_index_pointer + 0]
    for (long _i0 = 0; _i0 < (_lattice_a * _lattice_b * _lattice_c * _lattice_d); _i0++) {
        for (long _i1 = 0; _i1 < _abcd_density_matrix_ncomponents; _i1++) {
          if (_xmds_isnonfinite(_abcd_density_matrix[_abcd_density_matrix_index_pointer + _i1].Re())
            || _xmds_isnonfinite(_abcd_density_matrix[_abcd_density_matrix_index_pointer + _i1].Im())) bNoNaNsPresent = false;
        }
    
      _abcd_density_matrix_index_pointer += _abcd_density_matrix_ncomponents;
    }
    #undef rho
  }
  return bNoNaNsPresent;
}

// Delta A propagation operator for field abcd
void _segment1_abcd_operators_evaluate_operator0(real _step)
{
  long _abcd_density_matrix_index_pointer = 0;
  #define rho _active_abcd_density_matrix[_abcd_density_matrix_index_pointer + 0]
  long _segment1_operator0_delta_a_field_density_matrix_index_pointer = 0;
  #define drho_dt _active_segment1_operator0_delta_a_field_density_matrix[_segment1_operator0_delta_a_field_density_matrix_index_pointer + 0]
  long _dimensionless_time_dependent_coupling_index_pointer = 0;
  #define OmegaA_t _active_dimensionless_time_dependent_coupling[_dimensionless_time_dependent_coupling_index_pointer + 0]
  #define OmegaB_t _active_dimensionless_time_dependent_coupling[_dimensionless_time_dependent_coupling_index_pointer + 1]
  #define DeltaA_t _active_dimensionless_time_dependent_coupling[_dimensionless_time_dependent_coupling_index_pointer + 2]
  #define DeltaB_t _active_dimensionless_time_dependent_coupling[_dimensionless_time_dependent_coupling_index_pointer + 3]
  #define U_t _active_dimensionless_time_dependent_coupling[_dimensionless_time_dependent_coupling_index_pointer + 4]
  #define a _a[_index_a + 0]
  #define da (_da * (1.0))
  
  for (long _index_a = 0; _index_a < _lattice_a; _index_a++) {
    #define b _b[_index_b + 0]
    #define db (_db * (1.0))
    
    for (long _index_b = 0; _index_b < _lattice_b; _index_b++) {
      #define c _c[_index_c + 0]
      #define dc (_dc * (1.0))
      
      for (long _index_c = 0; _index_c < _lattice_c; _index_c++) {
        #define d _d[_index_d + 0]
        #define dd (_dd * (1.0))
        
        for (long _index_d = 0; _index_d < _lattice_d; _index_d++) {
          
          #define dt _step
          
          // ************* Propagation code ***************
          #define _rho_abcd(_index_a, _index_b, _index_c, _index_d) _active_abcd_density_matrix[0 + (0 \
             + _index_a * _lattice_b * _lattice_c * _lattice_d \
             + _index_b * _lattice_c * _lattice_d \
             + _index_c * _lattice_d \
             + _index_d * 1) * _abcd_density_matrix_ncomponents]
          #line 113 "Assignment5_program_solution_v2.xmds"
          
          drho_dt = 0.0;   
          //populations
          if(a==0 && b==0 && c==0 && d==0) //drho00;00_dt
             drho_dt = -i*0.5*(OmegaA_t*(_rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 0 */ ((0) - _min_d)) - _rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 0 */ ((0) - _min_d))) + OmegaB_t*(_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 0 */ ((0) - _min_d))- _rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 1 */ ((1) - _min_d))) );
          if(a==0 && b==1 && c==0 && d==1)  //drho01;01_dt
             drho_dt =  -i*0.5*(OmegaA_t*(_rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 1 */ ((1) - _min_d)) - _rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 1 */ ((1) - _min_d))) + OmegaB_t*(_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 1 */ ((1) - _min_d))- _rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 0 */ ((0) - _min_d))) );
          if(a==1 && b==0 && c==1 && d==0) //drho10;10_dt
             drho_dt =  -i*0.5*(OmegaA_t*(_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 0 */ ((0) - _min_d)) - _rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 0 */ ((0) - _min_d))) + OmegaB_t*(_rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 0 */ ((0) - _min_d))- _rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 1 */ ((1) - _min_d))) );
          if(a==1 && b==1 && c==1 && d==1) //drho11;11_dt
             drho_dt =  -i*0.5*(OmegaA_t*(_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 1 */ ((1) - _min_d)) - _rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 1 */ ((1) - _min_d))) + OmegaB_t*(_rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 1 */ ((1) - _min_d))- _rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 0 */ ((0) - _min_d))) );
          
          
          //coherences
          if(a==0 && b==0 && c==0 && d==1)  //drho00;01_dt
             drho_dt = - gammaB*_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 1 */ ((1) - _min_d)) +i*0.5*(2*DeltaB_t*_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 1 */ ((1) - _min_d))  + OmegaA_t*(_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 1 */ ((1) - _min_d)) - _rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 1 */ ((1) - _min_d))) + OmegaB_t*(_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 0 */ ((0) - _min_d)) - _rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 1 */ ((1) - _min_d))) );
          if(a==0 && b==1 && c==0 && d==0)  //drho01;00_dt
             drho_dt = - gammaB*_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 0 */ ((0) - _min_d)) -i*0.5*(2*DeltaB_t*_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 0 */ ((0) - _min_d))  + OmegaA_t*(_rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 0 */ ((0) - _min_d)) - _rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 0 */ ((0) - _min_d))) + OmegaB_t*(_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 0 */ ((0) - _min_d)) - _rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 1 */ ((1) - _min_d))) );
          
          if(a==0 && b==0 && c==1 && d==0)  //drho00;10_dt
             drho_dt = - gammaA*_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 0 */ ((0) - _min_d)) +i*0.5*(2*DeltaA_t*_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 0 */ ((0) - _min_d))  + OmegaA_t*(_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 0 */ ((0) - _min_d)) - _rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 0 */ ((0) - _min_d))) + OmegaB_t*(_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 1 */ ((1) - _min_d)) - _rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 0 */ ((0) - _min_d))) );
          if(a==1 && b==0 && c==0 && d==0)  //drho10;00_dt
             drho_dt = - gammaA*_rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 0 */ ((0) - _min_d)) -i*0.5*(2*DeltaA_t*_rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 0 */ ((0) - _min_d))  + OmegaA_t*(_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 0 */ ((0) - _min_d)) - _rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 0 */ ((0) - _min_d))) + OmegaB_t*(_rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 0 */ ((0) - _min_d)) - _rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 1 */ ((1) - _min_d))) );
          
          
          if(a==0 && b==0 && c==1 && d==1)  //drho00;11_dt
             drho_dt = - (gammaA+gammaB)*_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 1 */ ((1) - _min_d)) +i*0.5*(2*(DeltaA_t + DeltaB_t + U_t)*_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 1 */ ((1) - _min_d))  + OmegaA_t*(_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 1 */ ((1) - _min_d)) - _rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 1 */ ((1) - _min_d))) + OmegaB_t*(_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 0 */ ((0) - _min_d)) - _rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 1 */ ((1) - _min_d))) );
          if(a==1 && b==1 && c==0 && d==0)  //drho11;00_dt
             drho_dt = - (gammaA+gammaB)*_rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 0 */ ((0) - _min_d)) -i*0.5*(2*(DeltaA_t + DeltaB_t + U_t)*_rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 0 */ ((0) - _min_d))  + OmegaA_t*(_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 0 */ ((0) - _min_d)) - _rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 0 */ ((0) - _min_d))) + OmegaB_t*(_rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 0 */ ((0) - _min_d)) - _rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 1 */ ((1) - _min_d))) );
          
          ///////
           if(a==0 && b==1 && c==1 && d==0)  //drho01;10_dt
             drho_dt = - (gammaA+gammaB)*_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 0 */ ((0) - _min_d)) +i*0.5*(2*(DeltaA_t - DeltaB_t)*_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 0 */ ((0) - _min_d))  + OmegaA_t*(_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 0 */ ((0) - _min_d)) - _rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 0 */ ((0) - _min_d))) + OmegaB_t*(_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 1 */ ((1) - _min_d)) - _rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 0 */ ((0) - _min_d))) );
          if(a==1 && b==0 && c==0 && d==1)  //drho10;01_dt
             drho_dt = - (gammaA+gammaB)*_rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 1 */ ((1) - _min_d)) -i*0.5*(2*(DeltaA_t - DeltaB_t)*_rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 1 */ ((1) - _min_d))  + OmegaA_t*(_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 1 */ ((1) - _min_d)) - _rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 1 */ ((1) - _min_d))) + OmegaB_t*(_rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 1 */ ((1) - _min_d)) - _rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 0 */ ((0) - _min_d))) );
          
          
           if(a==0 && b==1 && c==1 && d==1)  //drho01;11_dt
             drho_dt = -gammaA*_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 1 */ ((1) - _min_d)) +i*0.5*(2*(DeltaA_t + U_t)*_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 1 */ ((1) - _min_d))  + OmegaA_t*(_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 1 */ ((1) - _min_d)) - _rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 1 */ ((1) - _min_d))) + OmegaB_t*(_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 0 */ ((0) - _min_d)) - _rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 1 */ ((1) - _min_d))) );
          
          if(a==1 && b==1 && c==0 && d==1)  //drho11;01_dt
             drho_dt = -gammaA*_rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 1 */ ((1) - _min_d)) -i*0.5*(2*(DeltaA_t + U_t)*_rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 1 */ ((1) - _min_d))  + OmegaA_t*(_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 1 */ ((1) - _min_d)) - _rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 1 */ ((1) - _min_d))) + OmegaB_t*(_rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 1 */ ((1) - _min_d)) - _rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 0 */ ((0) - _min_d))) );
          
           ///       
           if(a==1 && b==0 && c==1 && d==1)  //drho10;11_dt
             drho_dt = -gammaB*_rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 1 */ ((1) - _min_d)) +i*0.5*(2*(DeltaB_t + U_t)*_rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 1 */ ((1) - _min_d))  + OmegaA_t*(_rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 1 */ ((1) - _min_d)) - _rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 1 */ ((1) - _min_d))) + OmegaB_t*(_rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 0 */ ((0) - _min_d)) - _rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 1 */ ((1) - _min_d))) );
          
          if(a==1 && b==1 && c==1 && d==0)  //drho11;10_dt
             drho_dt =  -gammaB*_rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 0 */ ((0) - _min_d)) -i*0.5*(2*(DeltaB_t + U_t)*_rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 0 */ ((0) - _min_d))  + OmegaA_t*(_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 0 */ ((0) - _min_d)) - _rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 0 */ ((0) - _min_d))) + OmegaB_t*(_rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 0 */ ((0) - _min_d)) - _rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 1 */ ((1) - _min_d))) );
          
          
          #line 2189 "Assignment5_program_solution_v2.cc"
          #undef _rho_abcd
          // **********************************************
          
          #undef dt
          
          
          // Increment index pointers for vectors in field segment1_operator0_looping_field (or having the same dimensions)
          _abcd_density_matrix_index_pointer += 1 * _abcd_density_matrix_ncomponents;
          _segment1_operator0_delta_a_field_density_matrix_index_pointer += 1 * _segment1_operator0_delta_a_field_density_matrix_ncomponents;
          
        }
        #undef d
        #undef dd
      }
      #undef c
      #undef dc
    }
    #undef b
    #undef db
  }
  #undef a
  #undef da
  _segment1_abcd_operators_operator0_copy_delta_a(_step);
  #undef rho
  #undef drho_dt
  #undef OmegaA_t
  #undef OmegaB_t
  #undef DeltaA_t
  #undef DeltaB_t
  #undef U_t
}


inline void _segment1_abcd_operators_operator0_copy_delta_a(real _step)
{
  long _abcd_density_matrix_index_pointer = 0;
  #define rho _active_abcd_density_matrix[_abcd_density_matrix_index_pointer + 0]
  long _segment1_operator0_delta_a_field_density_matrix_index_pointer = 0;
  #define drho_dt _active_segment1_operator0_delta_a_field_density_matrix[_segment1_operator0_delta_a_field_density_matrix_index_pointer + 0]
  #define a _a[_index_a + 0]
  #define da (_da * (1.0))
  
  for (long _index_a = 0; _index_a < _lattice_a; _index_a++) {
    #define b _b[_index_b + 0]
    #define db (_db * (1.0))
    
    for (long _index_b = 0; _index_b < _lattice_b; _index_b++) {
      #define c _c[_index_c + 0]
      #define dc (_dc * (1.0))
      
      for (long _index_c = 0; _index_c < _lattice_c; _index_c++) {
        #define d _d[_index_d + 0]
        #define dd (_dd * (1.0))
        
        for (long _index_d = 0; _index_d < _lattice_d; _index_d++) {
          
          // UNVECTORISABLE
          // This code copies the increments for the components back into the vectors themselves.
          rho = drho_dt * _step;
          // Increment index pointers for vectors in field segment1_operator0_looping_field (or having the same dimensions)
          _abcd_density_matrix_index_pointer += 1 * _abcd_density_matrix_ncomponents;
          _segment1_operator0_delta_a_field_density_matrix_index_pointer += 1 * _segment1_operator0_delta_a_field_density_matrix_ncomponents;
          
        }
        #undef d
        #undef dd
      }
      #undef c
      #undef dc
    }
    #undef b
    #undef db
  }
  #undef a
  #undef da
  #undef rho
  #undef drho_dt
}

// ********************************************************
//   output function implementations
void _write_output()
{
  _LOG(_SIMULATION_LOG_LEVEL, "Generating output for Assignment5_program_solution_v2\n");
  
  
  char *_xsilFilename = (char*)malloc(256);
  snprintf(_xsilFilename, 256, "%s.xsil", ("Assignment5_program_solution_v2" + gsArgsAndValues).c_str());
  
  FILE* _outfile = _open_xsil_file(_xsilFilename);
  
  if (_outfile) {
    _write_xsil_header(_outfile);
    char _dataFilename[200];
    snprintf(_dataFilename, 200, "%s.h5", ("Assignment5_program_solution_v2" + gsArgsAndValues).c_str());
    
    H5Fclose(H5Fcreate(_dataFilename, H5F_ACC_TRUNC, H5P_DEFAULT, H5P_DEFAULT));
  }
  _mg0_write_out(_outfile);
  _mg1_write_out(_outfile);
  _mg2_write_out(_outfile);
  _mg3_write_out(_outfile);
  _mg4_write_out(_outfile);
  
  _write_xsil_footer(_outfile);
  _close_xsil_file(_outfile);
  free(_xsilFilename);
  _xsilFilename = NULL;
  _outfile = NULL;
  
}


FILE* _open_xsil_file(const char* _filename)
{
  
  FILE* fp = fopen(_filename, "w");
  
  if (fp == NULL)
    // _LOG will cause the simulation to exit
    _LOG(_ERROR_LOG_LEVEL, "Unable to open output file '%s'.\n"
                           "Exiting.\n", _filename);
  
  return fp;
}

void _close_xsil_file(FILE*& fp)
{
  if (fp)
    fclose(fp);
  fp = NULL;
  
}

void _write_xsil_header(FILE* fp)
{
  if (!fp)
    return;
  fprintf(fp, "<?xml version=\"1.0\" ?><!DOCTYPE simulation [\n");
  fprintf(fp, "<!ENTITY Nstates     \"2\">\n");
  fprintf(fp, "]><simulation xmds-version=\"2\">\n");
  fprintf(fp, "  <name>Assignment5_program_solution_v2</name>\n");
  fprintf(fp, "  \n");
  fprintf(fp, "  <author> Sebastian Wuester (Oct 2022)</author>\n");
  fprintf(fp, "  <description>\n");
  fprintf(fp, "    Numerical solution of dephasing Master-equation for quantum computer with Rabi pulses.\n");
  fprintf(fp, "  </description>\n");
  fprintf(fp, "  \n");
  fprintf(fp, "  <geometry>\n");
  fprintf(fp, "    <propagation_dimension> t </propagation_dimension>\n");
  fprintf(fp, "    <transverse_dimensions>\n");
  fprintf(fp, "      <dimension name=\"a\" type=\"integer\" lattice=\"2\" domain=\"(0,1)\" aliases=\"aprime\"/>  <!--qubit  basis, 0,1 -->\n");
  fprintf(fp, "      <dimension name=\"b\" type=\"integer\" lattice=\"2\" domain=\"(0,1)\" aliases=\"bprime\"/>  <!--qubit  basis, 0,1 -->\n");
  fprintf(fp, "      <dimension name=\"c\" type=\"integer\" lattice=\"2\" domain=\"(0,1)\" aliases=\"cprime\"/>  <!--qubit  basis, 0,1 -->\n");
  fprintf(fp, "      <dimension name=\"d\" type=\"integer\" lattice=\"2\" domain=\"(0,1)\" aliases=\"dprime\"/>  <!--qubit  basis, 0,1 -->\n");
  fprintf(fp, "     </transverse_dimensions>\n");
  fprintf(fp, "  </geometry>\n");
  fprintf(fp, "  \n");
  fprintf(fp, "  <features>\n");
  fprintf(fp, "    <benchmark/>\n");
  fprintf(fp, "    <auto_vectorise/>\n");
  fprintf(fp, "    <fftw/>\n");
  fprintf(fp, "    <globals>\n");
  fprintf(fp, "      <![CDATA[\n");
  fprintf(fp, "       // Constant Parameters, dimensionless units  \n");
  fprintf(fp, "      const double gammaA = 1.0/5.0;  \n");
  fprintf(fp, "      const double gammaB = 1.0/5.0;  \n");
  fprintf(fp, "      \n");
  fprintf(fp, "      // Time dependent control \n");
  fprintf(fp, "      const double delayA = 0.0;             // time until pulse starts\n");
  fprintf(fp, "      const double TpulseA = 0.25;            // duration of the pulse\n");
  fprintf(fp, "      const double pulsetypeA = M_PI/2.0;    // pi-half pulse \n");
  fprintf(fp, "      \n");
  fprintf(fp, "      const double delayB = 0.5;             // time until pulse starts\n");
  fprintf(fp, "      const double TpulseB = 0.25;            // duration of the pulse\n");
  fprintf(fp, "      const double pulsetypeB = M_PI/2.0;    // pi-half pulse \n");
  fprintf(fp, "      \n");
  fprintf(fp, "      const double delayDeltaA = 1.0;             // time until pulse starts\n");
  fprintf(fp, "      const double TpulseDeltaA = 0.25;            // duration of the pulse\n");
  fprintf(fp, "      const double targetphaseA = 0.3*M_PI;\n");
  fprintf(fp, "      \n");
  fprintf(fp, "      const double delayDeltaB = 1.5;             // time until pulse starts\n");
  fprintf(fp, "      const double TpulseDeltaB = 0.25;            // duration of the pulse\n");
  fprintf(fp, "      const double targetphaseB = -M_PI/2.0;\n");
  fprintf(fp, "           \n");
  fprintf(fp, "      const double delayU = 2.0;             // time until pulse starts\n");
  fprintf(fp, "      const double TpulseU = 0.25;            // duration of the pulse\n");
  fprintf(fp, "      const double targetphaseU = M_PI;\n");
  fprintf(fp, "\n");
  fprintf(fp, "      // Resultant amplitude of parameters      \n");
  fprintf(fp, "      const double OmegaA0 = pulsetypeA/TpulseA;\n");
  fprintf(fp, "      const double OmegaB0 = pulsetypeB/TpulseB;\n");
  fprintf(fp, "      const double DeltaA0 = targetphaseA/TpulseDeltaA;\n");
  fprintf(fp, "      const double DeltaB0 = targetphaseB/TpulseDeltaB;\n");
  fprintf(fp, "      const double U0 = targetphaseU/TpulseU;\n");
  fprintf(fp, "       \n");
  fprintf(fp, "      ]]>\n");
  fprintf(fp, "    </globals>\n");
  fprintf(fp, "  </features>\n");
  fprintf(fp, "  \n");
  fprintf(fp, "  <vector name=\"density_matrix\" dimensions=\"a b c d\" type=\"complex\">\n");
  fprintf(fp, "    <components> rho </components>\n");
  fprintf(fp, "    <initialisation>\n");
  fprintf(fp, "      <![CDATA[\n");
  fprintf(fp, "       rho = 0.0; \n");
  fprintf(fp, "       if(a==0 && b==0 && c==0 & d==0)\n");
  fprintf(fp, "          rho = 1.0;\n");
  fprintf(fp, "      ]]>\n");
  fprintf(fp, "    </initialisation>\n");
  fprintf(fp, "  </vector>\n");
  fprintf(fp, "  \n");
  fprintf(fp, "    <computed_vector name=\"time_dependent_coupling\" type=\"real\" dimensions=\"\">\n");
  fprintf(fp, "    <components> OmegaA_t OmegaB_t DeltaA_t DeltaB_t  U_t</components>\n");
  fprintf(fp, "    <evaluation>\n");
  fprintf(fp, "    <dependencies> </dependencies>    \n");
  fprintf(fp, "      <![CDATA[\n");
  fprintf(fp, "        OmegaA_t = 0.0;\n");
  fprintf(fp, "        if(t>delayA && t<= delayA+TpulseA)\n");
  fprintf(fp, "            OmegaA_t = OmegaA0;\n");
  fprintf(fp, "\n");
  fprintf(fp, "        OmegaB_t = 0.0;\n");
  fprintf(fp, "        if(t>delayB && t<= delayB+TpulseB)\n");
  fprintf(fp, "            OmegaB_t = OmegaB0;\n");
  fprintf(fp, "            \n");
  fprintf(fp, "        DeltaA_t = 0.0;\n");
  fprintf(fp, "        if(t>delayDeltaA && t<= delayDeltaA+TpulseDeltaA)\n");
  fprintf(fp, "            DeltaA_t = DeltaA0;\n");
  fprintf(fp, "        \n");
  fprintf(fp, "        DeltaB_t = 0.0;\n");
  fprintf(fp, "        if(t>delayDeltaB && t<= delayDeltaB+TpulseDeltaB)\n");
  fprintf(fp, "            DeltaB_t = DeltaB0;  \n");
  fprintf(fp, "            \n");
  fprintf(fp, "        U_t = 0.0;\n");
  fprintf(fp, "        if(t>delayU && t<= delayU+TpulseU)\n");
  fprintf(fp, "            U_t = U0;  \n");
  fprintf(fp, "      ]]>\n");
  fprintf(fp, "    </evaluation>\n");
  fprintf(fp, "  </computed_vector>  \n");
  fprintf(fp, "\n");
  fprintf(fp, "  <sequence>\n");
  fprintf(fp, "    <!-- ARK89 = use Adaptive 8/9th order Runge-Kutta for time stepping -->\n");
  fprintf(fp, "    <!--         it allows a relative error of \"tolerance\" per step  -->\n");
  fprintf(fp, "    <!-- interval = final time of simulation -->\n");
  fprintf(fp, "    <integrate algorithm=\"ARK89\" interval=\"2.5\" tolerance=\"1e-10\"> \n");
  fprintf(fp, "      <samples>200 200 200 200 200</samples>\n");
  fprintf(fp, "      <operators>\n");
  fprintf(fp, "        <integration_vectors> density_matrix </integration_vectors>\n");
  fprintf(fp, "        <dependencies> time_dependent_coupling </dependencies>       \n");
  fprintf(fp, "        <![CDATA[     \n");
  fprintf(fp, "         drho_dt = 0.0;   \n");
  fprintf(fp, "         //populations\n");
  fprintf(fp, "         if(a==0 && b==0 && c==0 && d==0) //drho00;00_dt\n");
  fprintf(fp, "            drho_dt = -i*0.5*(OmegaA_t*(rho(a=>1,b=>0,c=>0,d=>0) - rho(a=>0,b=>0,c=>1,d=>0)) + OmegaB_t*(rho(a=>0,b=>1,c=>0,d=>0)- rho(a=>0,b=>0,c=>0,d=>1)) );\n");
  fprintf(fp, "         if(a==0 && b==1 && c==0 && d==1)  //drho01;01_dt\n");
  fprintf(fp, "            drho_dt =  -i*0.5*(OmegaA_t*(rho(a=>1,b=>1,c=>0,d=>1) - rho(a=>0,b=>1,c=>1,d=>1)) + OmegaB_t*(rho(a=>0,b=>0,c=>0,d=>1)- rho(a=>0,b=>1,c=>0,d=>0)) );\n");
  fprintf(fp, "         if(a==1 && b==0 && c==1 && d==0) //drho10;10_dt\n");
  fprintf(fp, "            drho_dt =  -i*0.5*(OmegaA_t*(rho(a=>0,b=>0,c=>1,d=>0) - rho(a=>1,b=>0,c=>0,d=>0)) + OmegaB_t*(rho(a=>1,b=>1,c=>1,d=>0)- rho(a=>1,b=>0,c=>1,d=>1)) );\n");
  fprintf(fp, "         if(a==1 && b==1 && c==1 && d==1) //drho11;11_dt\n");
  fprintf(fp, "            drho_dt =  -i*0.5*(OmegaA_t*(rho(a=>0,b=>1,c=>1,d=>1) - rho(a=>1,b=>1,c=>0,d=>1)) + OmegaB_t*(rho(a=>1,b=>0,c=>1,d=>1)- rho(a=>1,b=>1,c=>1,d=>0)) );\n");
  fprintf(fp, "\n");
  fprintf(fp, "\n");
  fprintf(fp, "         //coherences\n");
  fprintf(fp, "         if(a==0 && b==0 && c==0 && d==1)  //drho00;01_dt\n");
  fprintf(fp, "            drho_dt = - gammaB*rho(a=>0,b=>0,c=>0,d=>1) +i*0.5*(2*DeltaB_t*rho(a=>0,b=>0,c=>0,d=>1)  + OmegaA_t*(rho(a=>0,b=>0,c=>1,d=>1) - rho(a=>1,b=>0,c=>0,d=>1)) + OmegaB_t*(rho(a=>0,b=>0,c=>0,d=>0) - rho(a=>0,b=>1,c=>0,d=>1)) );\n");
  fprintf(fp, "         if(a==0 && b==1 && c==0 && d==0)  //drho01;00_dt\n");
  fprintf(fp, "            drho_dt = - gammaB*rho(a=>0,b=>1,c=>0,d=>0) -i*0.5*(2*DeltaB_t*rho(a=>0,b=>1,c=>0,d=>0)  + OmegaA_t*(rho(a=>1,b=>1,c=>0,d=>0) - rho(a=>0,b=>1,c=>1,d=>0)) + OmegaB_t*(rho(a=>0,b=>0,c=>0,d=>0) - rho(a=>0,b=>1,c=>0,d=>1)) );\n");
  fprintf(fp, "  \n");
  fprintf(fp, "         if(a==0 && b==0 && c==1 && d==0)  //drho00;10_dt\n");
  fprintf(fp, "            drho_dt = - gammaA*rho(a=>0,b=>0,c=>1,d=>0) +i*0.5*(2*DeltaA_t*rho(a=>0,b=>0,c=>1,d=>0)  + OmegaA_t*(rho(a=>0,b=>0,c=>0,d=>0) - rho(a=>1,b=>0,c=>1,d=>0)) + OmegaB_t*(rho(a=>0,b=>0,c=>1,d=>1) - rho(a=>0,b=>1,c=>1,d=>0)) );\n");
  fprintf(fp, "         if(a==1 && b==0 && c==0 && d==0)  //drho10;00_dt\n");
  fprintf(fp, "            drho_dt = - gammaA*rho(a=>1,b=>0,c=>0,d=>0) -i*0.5*(2*DeltaA_t*rho(a=>1,b=>0,c=>0,d=>0)  + OmegaA_t*(rho(a=>0,b=>0,c=>0,d=>0) - rho(a=>1,b=>0,c=>1,d=>0)) + OmegaB_t*(rho(a=>1,b=>1,c=>0,d=>0) - rho(a=>1,b=>0,c=>0,d=>1)) );\n");
  fprintf(fp, "            \n");
  fprintf(fp, "                 \n");
  fprintf(fp, "         if(a==0 && b==0 && c==1 && d==1)  //drho00;11_dt\n");
  fprintf(fp, "            drho_dt = - (gammaA+gammaB)*rho(a=>0,b=>0,c=>1,d=>1) +i*0.5*(2*(DeltaA_t + DeltaB_t + U_t)*rho(a=>0,b=>0,c=>1,d=>1)  + OmegaA_t*(rho(a=>0,b=>0,c=>0,d=>1) - rho(a=>1,b=>0,c=>1,d=>1)) + OmegaB_t*(rho(a=>0,b=>0,c=>1,d=>0) - rho(a=>0,b=>1,c=>1,d=>1)) );\n");
  fprintf(fp, "         if(a==1 && b==1 && c==0 && d==0)  //drho11;00_dt\n");
  fprintf(fp, "            drho_dt = - (gammaA+gammaB)*rho(a=>1,b=>1,c=>0,d=>0) -i*0.5*(2*(DeltaA_t + DeltaB_t + U_t)*rho(a=>1,b=>1,c=>0,d=>0)  + OmegaA_t*(rho(a=>0,b=>1,c=>0,d=>0) - rho(a=>1,b=>1,c=>1,d=>0)) + OmegaB_t*(rho(a=>1,b=>0,c=>0,d=>0) - rho(a=>1,b=>1,c=>0,d=>1)) );\n");
  fprintf(fp, "            \n");
  fprintf(fp, "         ///////\n");
  fprintf(fp, "          if(a==0 && b==1 && c==1 && d==0)  //drho01;10_dt\n");
  fprintf(fp, "            drho_dt = - (gammaA+gammaB)*rho(a=>0,b=>1,c=>1,d=>0) +i*0.5*(2*(DeltaA_t - DeltaB_t)*rho(a=>0,b=>1,c=>1,d=>0)  + OmegaA_t*(rho(a=>0,b=>1,c=>0,d=>0) - rho(a=>1,b=>1,c=>1,d=>0)) + OmegaB_t*(rho(a=>0,b=>1,c=>1,d=>1) - rho(a=>0,b=>0,c=>1,d=>0)) );\n");
  fprintf(fp, "         if(a==1 && b==0 && c==0 && d==1)  //drho10;01_dt\n");
  fprintf(fp, "            drho_dt = - (gammaA+gammaB)*rho(a=>1,b=>0,c=>0,d=>1) -i*0.5*(2*(DeltaA_t - DeltaB_t)*rho(a=>1,b=>0,c=>0,d=>1)  + OmegaA_t*(rho(a=>0,b=>0,c=>0,d=>1) - rho(a=>1,b=>0,c=>1,d=>1)) + OmegaB_t*(rho(a=>1,b=>1,c=>0,d=>1) - rho(a=>1,b=>0,c=>0,d=>0)) );\n");
  fprintf(fp, "\n");
  fprintf(fp, "   \n");
  fprintf(fp, "          if(a==0 && b==1 && c==1 && d==1)  //drho01;11_dt\n");
  fprintf(fp, "            drho_dt = -gammaA*rho(a=>0,b=>1,c=>1,d=>1) +i*0.5*(2*(DeltaA_t + U_t)*rho(a=>0,b=>1,c=>1,d=>1)  + OmegaA_t*(rho(a=>0,b=>1,c=>0,d=>1) - rho(a=>1,b=>1,c=>1,d=>1)) + OmegaB_t*(rho(a=>0,b=>1,c=>1,d=>0) - rho(a=>0,b=>0,c=>1,d=>1)) );\n");
  fprintf(fp, "\n");
  fprintf(fp, "         if(a==1 && b==1 && c==0 && d==1)  //drho11;01_dt\n");
  fprintf(fp, "            drho_dt = -gammaA*rho(a=>1,b=>1,c=>0,d=>1) -i*0.5*(2*(DeltaA_t + U_t)*rho(a=>1,b=>1,c=>0,d=>1)  + OmegaA_t*(rho(a=>0,b=>1,c=>0,d=>1) - rho(a=>1,b=>1,c=>1,d=>1)) + OmegaB_t*(rho(a=>1,b=>0,c=>0,d=>1) - rho(a=>1,b=>1,c=>0,d=>0)) );\n");
  fprintf(fp, "            \n");
  fprintf(fp, "          ///       \n");
  fprintf(fp, "          if(a==1 && b==0 && c==1 && d==1)  //drho10;11_dt\n");
  fprintf(fp, "            drho_dt = -gammaB*rho(a=>1,b=>0,c=>1,d=>1) +i*0.5*(2*(DeltaB_t + U_t)*rho(a=>1,b=>0,c=>1,d=>1)  + OmegaA_t*(rho(a=>1,b=>0,c=>0,d=>1) - rho(a=>0,b=>0,c=>1,d=>1)) + OmegaB_t*(rho(a=>1,b=>0,c=>1,d=>0) - rho(a=>1,b=>1,c=>1,d=>1)) );\n");
  fprintf(fp, "\n");
  fprintf(fp, "         if(a==1 && b==1 && c==1 && d==0)  //drho11;10_dt\n");
  fprintf(fp, "            drho_dt =  -gammaB*rho(a=>1,b=>1,c=>1,d=>0) -i*0.5*(2*(DeltaB_t + U_t)*rho(a=>1,b=>1,c=>1,d=>0)  + OmegaA_t*(rho(a=>0,b=>1,c=>1,d=>0) - rho(a=>1,b=>1,c=>0,d=>0)) + OmegaB_t*(rho(a=>1,b=>0,c=>1,d=>0) - rho(a=>1,b=>1,c=>1,d=>1)) );\n");
  fprintf(fp, "\n");
  fprintf(fp, "         ]]>\n");
  fprintf(fp, "      </operators>\n");
  fprintf(fp, "    </integrate>\n");
  fprintf(fp, "  </sequence>\n");
  fprintf(fp, "  \n");
  fprintf(fp, "  <output format=\"hdf5\">\n");
  fprintf(fp, "    <group>\n");
  fprintf(fp, "      <sampling basis=\"a b c d\" initial_sample=\"yes\">\n");
  fprintf(fp, "        <moments> rho_re rho_im   </moments>\n");
  fprintf(fp, "        <dependencies> density_matrix </dependencies>\n");
  fprintf(fp, "        <![CDATA[\n");
  fprintf(fp, "        \n");
  fprintf(fp, "          rho_re = rho.Re();\n");
  fprintf(fp, "          rho_im = rho.Im();\n");
  fprintf(fp, "          \n");
  fprintf(fp, "        ]]>\n");
  fprintf(fp, "      </sampling>\n");
  fprintf(fp, "    </group>\n");
  fprintf(fp, "    <group>\n");
  fprintf(fp, "      <sampling basis=\"a b c(0) d(0)\" initial_sample=\"yes\">\n");
  fprintf(fp, "        <moments> pop   </moments>\n");
  fprintf(fp, "        <dependencies> density_matrix </dependencies>\n");
  fprintf(fp, "        <![CDATA[\n");
  fprintf(fp, "         pop = 0.0;\n");
  fprintf(fp, "          if(_index_a==_index_c && _index_b==_index_d)\n");
  fprintf(fp, "           pop = rho.Re();\n");
  fprintf(fp, "\n");
  fprintf(fp, "        ]]>\n");
  fprintf(fp, "      </sampling>\n");
  fprintf(fp, "    </group>\n");
  fprintf(fp, "    <group>\n");
  fprintf(fp, "      <sampling basis=\"a(0) b(0) c(0) d(0)\" initial_sample=\"yes\">\n");
  fprintf(fp, "        <moments> trace  </moments>\n");
  fprintf(fp, "        <dependencies>density_matrix</dependencies>\n");
  fprintf(fp, "        <![CDATA[\n");
  fprintf(fp, "          trace = 0.0;\n");
  fprintf(fp, "          if(_index_a==_index_c && _index_b==_index_d)\n");
  fprintf(fp, "             trace = rho.Re();\n");
  fprintf(fp, "           \n");
  fprintf(fp, "        ]]>\n");
  fprintf(fp, "      </sampling>\n");
  fprintf(fp, "    </group>\n");
  fprintf(fp, "    <group>\n");
  fprintf(fp, "      <sampling basis=\"a(0) b(0) c(0) d(0)\" initial_sample=\"yes\">\n");
  fprintf(fp, "        <moments> energy  </moments>\n");
  fprintf(fp, "        <dependencies> density_matrix time_dependent_coupling</dependencies>\n");
  fprintf(fp, "        <![CDATA[\n");
  fprintf(fp, "          energy = 0.0;\n");
  fprintf(fp, "          if(_index_a==0 && _index_b==0 && _index_c==0 && _index_d==0)   \n");
  fprintf(fp, "              energy += (U_t*rho(a=>1,b=>1,c=>1,d=>1) + 0.5*DeltaB_t*(-rho(a=>0,b=>0,c=>0,d=>0) + rho(a=>0,b=>1,c=>0,d=>1) - rho(a=>1,b=>0,c=>1,d=>0) + rho(a=>1,b=>1,c=>1,d=>1)) + 0.5*DeltaA_t*(-rho(a=>0,b=>0,c=>0,d=>0) - rho(a=>0,b=>1,c=>0,d=>1) + rho(a=>1,b=>0,c=>1,d=>0) + rho(a=>1,b=>1,c=>1,d=>1)) + 0.5*OmegaA_t*(rho(a=>0,b=>0,c=>1,d=>0) + rho(a=>1,b=>0,c=>0,d=>0) + rho(a=>0,b=>1,c=>1,d=>1) + rho(a=>1,b=>1,c=>0,d=>1)) + 0.5*OmegaB_t*(rho(a=>0,b=>0,c=>0,d=>1) + rho(a=>0,b=>1,c=>0,d=>0) + rho(a=>1,b=>0,c=>1,d=>1) + rho(a=>1,b=>1,c=>1,d=>0))).Re();\n");
  fprintf(fp, "           \n");
  fprintf(fp, "        ]]>\n");
  fprintf(fp, "      </sampling>\n");
  fprintf(fp, "    </group>\n");
  fprintf(fp, "    <group>\n");
  fprintf(fp, "      <sampling basis=\"\" initial_sample=\"yes\">\n");
  fprintf(fp, "        <moments> OmegaA OmegaB DeltaA DeltaB UU </moments>\n");
  fprintf(fp, "        <dependencies>time_dependent_coupling</dependencies>\n");
  fprintf(fp, "        <![CDATA[\n");
  fprintf(fp, "          OmegaA = OmegaA_t;\n");
  fprintf(fp, "          OmegaB = OmegaB_t;\n");
  fprintf(fp, "          DeltaA = DeltaA_t;\n");
  fprintf(fp, "          DeltaB = DeltaB_t;\n");
  fprintf(fp, "          UU = U_t;\n");
  fprintf(fp, "        ]]>\n");
  fprintf(fp, "      </sampling>\n");
  fprintf(fp, "    </group>\n");
  fprintf(fp, "  </output>\n");
  
  fprintf(fp, "\n<info>\n");
  fprintf(fp, "Script compiled with XMDS2 version 3.1.0 \"My kingdom for a tentacle\" (r)\n");
  fprintf(fp, "See http://www.xmds.org for more information.\n");
  fprintf(fp, "</info>\n");
  
}

// In addition to writing the footer (if 'fp' is not NULL)
// this function closes the fp file pointer.
void _write_xsil_footer(FILE* fp)
{
  if (fp) {
    fprintf(fp, "</simulation>\n");
  }
}

// ********************************************************
//   moment group 0 function implementations
void _mg0_sample()
{
  
  long _mg0_output_raw_index_pointer = 0;
  #define rho_re _active_mg0_output_raw[_mg0_output_raw_index_pointer + 0]
  #define rho_im _active_mg0_output_raw[_mg0_output_raw_index_pointer + 1]
  long _abcd_density_matrix_index_pointer = 0;
  #define rho _active_abcd_density_matrix[_abcd_density_matrix_index_pointer + 0]
  #define a _a[_index_a + 0]
  #define da (_da * (1.0))
  
  for (long _index_a = 0; _index_a < _lattice_a; _index_a++) {
    #define b _b[_index_b + 0]
    #define db (_db * (1.0))
    
    for (long _index_b = 0; _index_b < _lattice_b; _index_b++) {
      #define c _c[_index_c + 0]
      #define dc (_dc * (1.0))
      
      for (long _index_c = 0; _index_c < _lattice_c; _index_c++) {
        #define d _d[_index_d + 0]
        #define dd (_dd * (1.0))
        
        for (long _index_d = 0; _index_d < _lattice_d; _index_d++) {
          // Set index pointers explicitly for (some) vectors
          _mg0_output_raw_index_pointer = ( 0
             + _mg0_output_index_t  * _lattice_a * _lattice_b * _lattice_c * _lattice_d
             + _index_a * _lattice_b * _lattice_c * _lattice_d
             + _index_b * _lattice_c * _lattice_d
             + _index_c * _lattice_d
             + _index_d * 1 ) * _mg0_output_raw_ncomponents;
          #define _SAMPLE_COMPLEX(variable) \
                    variable ## R = variable.Re(); variable ## I = variable.Im();
          
          // *************** Sampling code ****************
          #line 173 "Assignment5_program_solution_v2.xmds"
          
          
          rho_re = rho.Re();
          rho_im = rho.Im();
          
          
          #line 2617 "Assignment5_program_solution_v2.cc"
          // **********************************************
          
          #undef _SAMPLE_COMPLEX
          // Increment index pointers for vectors in field mg0_sampling (or having the same dimensions)
          _abcd_density_matrix_index_pointer += 1 * _abcd_density_matrix_ncomponents;
          
        }
        #undef d
        #undef dd
      }
      #undef c
      #undef dc
    }
    #undef b
    #undef db
  }
  #undef a
  #undef da
  #undef rho_re
  #undef rho_im
  #undef rho
  
  _mg0_output_t[0 + _mg0_output_index_t++] = t;
  
  _LOG(_SAMPLE_LOG_LEVEL, "Sampled field (for moment group #1) at t = %e\n", t);
  
}


void _mg0_process()
{
  // No post processing needs to be done
}


void _mg0_write_out(FILE* _outfile)
{
  
  if (_outfile) {
    fprintf(_outfile, "\n");
    fprintf(_outfile, "<XSIL Name=\"moment_group_1\">\n");
    fprintf(_outfile, "  <Param Name=\"n_independent\">5</Param>\n");
    fprintf(_outfile, "  <Array Name=\"variables\" Type=\"Text\">\n");
    fprintf(_outfile, "    <Dim>7</Dim>\n");
    fprintf(_outfile, "    <Stream><Metalink Format=\"Text\" Delimiter=\" \\n\"/>\n");
    fprintf(_outfile, "t a b c d rho_re rho_im \n");
    fprintf(_outfile, "    </Stream>\n");
    fprintf(_outfile, "  </Array>\n");
    fprintf(_outfile, "  <Array Name=\"data\" Type=\"double\">\n");
    fprintf(_outfile, "    <Dim>%i</Dim>\n", _mg0_output_lattice_t);
    fprintf(_outfile, "    <Dim>%i</Dim>\n", _lattice_a);
    fprintf(_outfile, "    <Dim>%i</Dim>\n", _lattice_b);
    fprintf(_outfile, "    <Dim>%i</Dim>\n", _lattice_c);
    fprintf(_outfile, "    <Dim>%i</Dim>\n", _lattice_d);
    fprintf(_outfile, "    <Dim>7</Dim>\n");
  }
  
  
  char _h5Filename[200];
  snprintf(_h5Filename, 200, "%s.h5", ("Assignment5_program_solution_v2" + gsArgsAndValues).c_str());
  
  /* Open the file */
  hid_t hdf5_file = H5Fopen(_h5Filename, H5F_ACC_RDWR, H5P_DEFAULT);
  if (hdf5_file < 0) {
    _LOG(_WARNING_LOG_LEVEL, "Failed to open HDF5 file '%s', will try to create it.", _h5Filename);
    hdf5_file = H5Fcreate(_h5Filename, H5F_ACC_EXCL, H5P_DEFAULT, H5P_DEFAULT);
    if (hdf5_file < 0) {
      _LOG(_ERROR_LOG_LEVEL, "Failed to create HDF5 file '%s'. Bailing.", _h5Filename);
    }
  }
  
  /* Create the group for this data */
  hid_t group;
  if (!H5Lexists(hdf5_file, "/1", H5P_DEFAULT))
    group = H5Gcreate(hdf5_file, "/1", H5P_DEFAULT);
  else
    group = H5Gopen(hdf5_file, "/1");
  
  if (_outfile) {
    fprintf(_outfile, "    <Stream><Metalink Format=\"HDF5\" Type=\"Remote\" Group=\"/1\"/>\n");
    fprintf(_outfile, "%s.h5\n", ("Assignment5_program_solution_v2" + gsArgsAndValues).c_str());
    fprintf(_outfile, "    </Stream>\n");
  }
  
  /* Create the coordinate data sets */
  hsize_t coordinate_length;
  hid_t coordinate_dataspace;
  coordinate_length = _mg0_output_lattice_t;
  coordinate_dataspace = H5Screate_simple(1, &coordinate_length, NULL);
  hid_t dataset_t;
  if (!H5Lexists(hdf5_file, "/1/t", H5P_DEFAULT))
    dataset_t = H5Dcreate(hdf5_file, "/1/t", H5T_NATIVE_REAL, coordinate_dataspace, H5P_DEFAULT);
  else
    dataset_t = H5Dopen(hdf5_file, "/1/t");
  H5Dwrite(dataset_t, H5T_NATIVE_REAL, H5S_ALL, H5S_ALL, H5P_DEFAULT, _mg0_output_t);
  #if defined(HAVE_HDF5_HL)
    H5DSset_scale(dataset_t, "t");
  #endif
  
  H5Sclose(coordinate_dataspace);
  coordinate_length = _lattice_a;
  coordinate_dataspace = H5Screate_simple(1, &coordinate_length, NULL);
  hid_t dataset_a;
  if (!H5Lexists(hdf5_file, "/1/a", H5P_DEFAULT))
    dataset_a = H5Dcreate(hdf5_file, "/1/a", H5T_NATIVE_LONG, coordinate_dataspace, H5P_DEFAULT);
  else
    dataset_a = H5Dopen(hdf5_file, "/1/a");
  H5Dwrite(dataset_a, H5T_NATIVE_LONG, H5S_ALL, H5S_ALL, H5P_DEFAULT, _a);
  #if defined(HAVE_HDF5_HL)
    H5DSset_scale(dataset_a, "a");
  #endif
  
  H5Sclose(coordinate_dataspace);
  coordinate_length = _lattice_b;
  coordinate_dataspace = H5Screate_simple(1, &coordinate_length, NULL);
  hid_t dataset_b;
  if (!H5Lexists(hdf5_file, "/1/b", H5P_DEFAULT))
    dataset_b = H5Dcreate(hdf5_file, "/1/b", H5T_NATIVE_LONG, coordinate_dataspace, H5P_DEFAULT);
  else
    dataset_b = H5Dopen(hdf5_file, "/1/b");
  H5Dwrite(dataset_b, H5T_NATIVE_LONG, H5S_ALL, H5S_ALL, H5P_DEFAULT, _b);
  #if defined(HAVE_HDF5_HL)
    H5DSset_scale(dataset_b, "b");
  #endif
  
  H5Sclose(coordinate_dataspace);
  coordinate_length = _lattice_c;
  coordinate_dataspace = H5Screate_simple(1, &coordinate_length, NULL);
  hid_t dataset_c;
  if (!H5Lexists(hdf5_file, "/1/c", H5P_DEFAULT))
    dataset_c = H5Dcreate(hdf5_file, "/1/c", H5T_NATIVE_LONG, coordinate_dataspace, H5P_DEFAULT);
  else
    dataset_c = H5Dopen(hdf5_file, "/1/c");
  H5Dwrite(dataset_c, H5T_NATIVE_LONG, H5S_ALL, H5S_ALL, H5P_DEFAULT, _c);
  #if defined(HAVE_HDF5_HL)
    H5DSset_scale(dataset_c, "c");
  #endif
  
  H5Sclose(coordinate_dataspace);
  coordinate_length = _lattice_d;
  coordinate_dataspace = H5Screate_simple(1, &coordinate_length, NULL);
  hid_t dataset_d;
  if (!H5Lexists(hdf5_file, "/1/d", H5P_DEFAULT))
    dataset_d = H5Dcreate(hdf5_file, "/1/d", H5T_NATIVE_LONG, coordinate_dataspace, H5P_DEFAULT);
  else
    dataset_d = H5Dopen(hdf5_file, "/1/d");
  H5Dwrite(dataset_d, H5T_NATIVE_LONG, H5S_ALL, H5S_ALL, H5P_DEFAULT, _d);
  #if defined(HAVE_HDF5_HL)
    H5DSset_scale(dataset_d, "d");
  #endif
  
  H5Sclose(coordinate_dataspace);
  
  hsize_t file_dims[] = {_mg0_output_lattice_t, _lattice_a, _lattice_b, _lattice_c, _lattice_d};
  hid_t file_dataspace = H5Screate_simple(5, file_dims, NULL);
  
  hid_t dataset_rho_re;
  if (!H5Lexists(hdf5_file, "/1/rho_re", H5P_DEFAULT))
    dataset_rho_re = H5Dcreate(hdf5_file, "/1/rho_re", H5T_NATIVE_REAL, file_dataspace, H5P_DEFAULT);
  else
    dataset_rho_re = H5Dopen(hdf5_file, "/1/rho_re");
  #if defined(HAVE_HDF5_HL)
    H5DSattach_scale(dataset_rho_re, dataset_t, 0);
    H5DSattach_scale(dataset_rho_re, dataset_a, 1);
    H5DSattach_scale(dataset_rho_re, dataset_b, 2);
    H5DSattach_scale(dataset_rho_re, dataset_c, 3);
    H5DSattach_scale(dataset_rho_re, dataset_d, 4);
  #endif
  hid_t dataset_rho_im;
  if (!H5Lexists(hdf5_file, "/1/rho_im", H5P_DEFAULT))
    dataset_rho_im = H5Dcreate(hdf5_file, "/1/rho_im", H5T_NATIVE_REAL, file_dataspace, H5P_DEFAULT);
  else
    dataset_rho_im = H5Dopen(hdf5_file, "/1/rho_im");
  #if defined(HAVE_HDF5_HL)
    H5DSattach_scale(dataset_rho_im, dataset_t, 0);
    H5DSattach_scale(dataset_rho_im, dataset_a, 1);
    H5DSattach_scale(dataset_rho_im, dataset_b, 2);
    H5DSattach_scale(dataset_rho_im, dataset_c, 3);
    H5DSattach_scale(dataset_rho_im, dataset_d, 4);
  #endif
  H5Dclose(dataset_t);
  H5Dclose(dataset_a);
  H5Dclose(dataset_b);
  H5Dclose(dataset_c);
  H5Dclose(dataset_d);
  
  
  if ((_mg0_output_lattice_t * _lattice_a * _lattice_b * _lattice_c * _lattice_d)) {
    /* Create the data space */
    hsize_t file_start[5] = {(unsigned long long int)0, (unsigned long long int)0, (unsigned long long int)0, (unsigned long long int)0, (unsigned long long int)0};
    hsize_t mem_dims[6] = {(unsigned long long int)_mg0_output_lattice_t, (unsigned long long int)_lattice_a, (unsigned long long int)_lattice_b, (unsigned long long int)_lattice_c, (unsigned long long int)_lattice_d, (unsigned long long int)1};
    hsize_t mem_start[6] = {0, 0, 0, 0, 0, 0};
    hsize_t mem_stride[6] = {1, 1, 1, 1, 1, 1};
    hsize_t mem_count[6] = {(unsigned long long int)_mg0_output_lattice_t, (unsigned long long int)_lattice_a, (unsigned long long int)_lattice_b, (unsigned long long int)_lattice_c, (unsigned long long int)_lattice_d, (unsigned long long int)1};
    
    
    hid_t mem_dataspace;
    mem_dims[5] = 2;
    mem_dataspace = H5Screate_simple(6, mem_dims, NULL);
    mem_stride[5] = 2;
    
    // Select hyperslabs of memory and file data spaces for data transfer operation
    mem_start[5] = 0;
    H5Sselect_hyperslab(mem_dataspace, H5S_SELECT_SET, mem_start, mem_stride, mem_count, NULL);
    H5Sselect_hyperslab(file_dataspace, H5S_SELECT_SET, file_start, mem_stride, mem_count, NULL);
    
    if (dataset_rho_re)
      H5Dwrite(dataset_rho_re, H5T_NATIVE_REAL, mem_dataspace, file_dataspace, H5P_DEFAULT, _active_mg0_output_raw);
    mem_start[5] = 1;
    H5Sselect_hyperslab(mem_dataspace, H5S_SELECT_SET, mem_start, mem_stride, mem_count, NULL);
    H5Sselect_hyperslab(file_dataspace, H5S_SELECT_SET, file_start, mem_stride, mem_count, NULL);
    
    if (dataset_rho_im)
      H5Dwrite(dataset_rho_im, H5T_NATIVE_REAL, mem_dataspace, file_dataspace, H5P_DEFAULT, _active_mg0_output_raw);
    
    H5Sclose(mem_dataspace);
  }
  
  
  H5Dclose(dataset_rho_re);
  H5Dclose(dataset_rho_im);
  
  H5Sclose(file_dataspace);
  H5Gclose(group);
  H5Fclose(hdf5_file);
  
  
  if (_outfile) {
    fprintf(_outfile, "  </Array>\n");
    fprintf(_outfile, "</XSIL>\n");
  }
}

// ********************************************************
//   field mg0_output function implementations
// initialisation for vector raw
void _mg0_output_raw_initialise()
{
  
  bzero(_active_mg0_output_raw, sizeof(real) * _mg0_output_raw_alloc_size);
}

// ********************************************************
//   moment group 1 function implementations
void _mg1_sample()
{
  
  long _mg1_output_raw_index_pointer = 0;
  real pop;
  long _abcd_density_matrix_index_pointer = 0;
  #define rho _active_abcd_density_matrix[_abcd_density_matrix_index_pointer + 0]
  #define a _a[_index_a + 0]
  #define da (_da * (1.0))
  
  for (long _index_a = 0; _index_a < _lattice_a; _index_a++) {
    #define b _b[_index_b + 0]
    #define db (_db * (1.0))
    
    for (long _index_b = 0; _index_b < _lattice_b; _index_b++) {
      #define c _c[_index_c + 0]
      #define dc (_dc * (1.0))
      
      for (long _index_c = 0; _index_c < _lattice_c; _index_c++) {
        #define d _d[_index_d + 0]
        #define dd (_dd * (1.0))
        
        for (long _index_d = 0; _index_d < _lattice_d; _index_d++) {
          // Set index pointers explicitly for (some) vectors
          _mg1_output_raw_index_pointer = ( 0
             + _mg1_output_index_t  * _lattice_a * _lattice_b
             + _index_a * _lattice_b
             + _index_b * 1 ) * _mg1_output_raw_ncomponents;
          #define _SAMPLE_COMPLEX(variable) \
                    variable ## R = variable.Re(); variable ## I = variable.Im();
          
          // *************** Sampling code ****************
          #line 185 "Assignment5_program_solution_v2.xmds"
          
          pop = 0.0;
           if(_index_a==_index_c && _index_b==_index_d)
            pop = rho.Re();
          
          
          #line 2901 "Assignment5_program_solution_v2.cc"
          // **********************************************
          
          #undef _SAMPLE_COMPLEX
          
          _active_mg1_output_raw[_mg1_output_raw_index_pointer + 0] += pop * dc * dd;
          // Increment index pointers for vectors in field mg1_sampling (or having the same dimensions)
          _abcd_density_matrix_index_pointer += 1 * _abcd_density_matrix_ncomponents;
          
        }
        #undef d
        #undef dd
      }
      #undef c
      #undef dc
    }
    #undef b
    #undef db
  }
  #undef a
  #undef da
  #undef rho
  
  _mg1_output_t[0 + _mg1_output_index_t++] = t;
  
  _LOG(_SAMPLE_LOG_LEVEL, "Sampled field (for moment group #2) at t = %e\n", t);
  
}


void _mg1_process()
{
  // No post processing needs to be done
}


void _mg1_write_out(FILE* _outfile)
{
  
  if (_outfile) {
    fprintf(_outfile, "\n");
    fprintf(_outfile, "<XSIL Name=\"moment_group_2\">\n");
    fprintf(_outfile, "  <Param Name=\"n_independent\">3</Param>\n");
    fprintf(_outfile, "  <Array Name=\"variables\" Type=\"Text\">\n");
    fprintf(_outfile, "    <Dim>4</Dim>\n");
    fprintf(_outfile, "    <Stream><Metalink Format=\"Text\" Delimiter=\" \\n\"/>\n");
    fprintf(_outfile, "t a b pop \n");
    fprintf(_outfile, "    </Stream>\n");
    fprintf(_outfile, "  </Array>\n");
    fprintf(_outfile, "  <Array Name=\"data\" Type=\"double\">\n");
    fprintf(_outfile, "    <Dim>%i</Dim>\n", _mg1_output_lattice_t);
    fprintf(_outfile, "    <Dim>%i</Dim>\n", _lattice_a);
    fprintf(_outfile, "    <Dim>%i</Dim>\n", _lattice_b);
    fprintf(_outfile, "    <Dim>4</Dim>\n");
  }
  
  
  char _h5Filename[200];
  snprintf(_h5Filename, 200, "%s.h5", ("Assignment5_program_solution_v2" + gsArgsAndValues).c_str());
  
  /* Open the file */
  hid_t hdf5_file = H5Fopen(_h5Filename, H5F_ACC_RDWR, H5P_DEFAULT);
  if (hdf5_file < 0) {
    _LOG(_WARNING_LOG_LEVEL, "Failed to open HDF5 file '%s', will try to create it.", _h5Filename);
    hdf5_file = H5Fcreate(_h5Filename, H5F_ACC_EXCL, H5P_DEFAULT, H5P_DEFAULT);
    if (hdf5_file < 0) {
      _LOG(_ERROR_LOG_LEVEL, "Failed to create HDF5 file '%s'. Bailing.", _h5Filename);
    }
  }
  
  /* Create the group for this data */
  hid_t group;
  if (!H5Lexists(hdf5_file, "/2", H5P_DEFAULT))
    group = H5Gcreate(hdf5_file, "/2", H5P_DEFAULT);
  else
    group = H5Gopen(hdf5_file, "/2");
  
  if (_outfile) {
    fprintf(_outfile, "    <Stream><Metalink Format=\"HDF5\" Type=\"Remote\" Group=\"/2\"/>\n");
    fprintf(_outfile, "%s.h5\n", ("Assignment5_program_solution_v2" + gsArgsAndValues).c_str());
    fprintf(_outfile, "    </Stream>\n");
  }
  
  /* Create the coordinate data sets */
  hsize_t coordinate_length;
  hid_t coordinate_dataspace;
  coordinate_length = _mg1_output_lattice_t;
  coordinate_dataspace = H5Screate_simple(1, &coordinate_length, NULL);
  hid_t dataset_t;
  if (!H5Lexists(hdf5_file, "/2/t", H5P_DEFAULT))
    dataset_t = H5Dcreate(hdf5_file, "/2/t", H5T_NATIVE_REAL, coordinate_dataspace, H5P_DEFAULT);
  else
    dataset_t = H5Dopen(hdf5_file, "/2/t");
  H5Dwrite(dataset_t, H5T_NATIVE_REAL, H5S_ALL, H5S_ALL, H5P_DEFAULT, _mg1_output_t);
  #if defined(HAVE_HDF5_HL)
    H5DSset_scale(dataset_t, "t");
  #endif
  
  H5Sclose(coordinate_dataspace);
  coordinate_length = _lattice_a;
  coordinate_dataspace = H5Screate_simple(1, &coordinate_length, NULL);
  hid_t dataset_a;
  if (!H5Lexists(hdf5_file, "/2/a", H5P_DEFAULT))
    dataset_a = H5Dcreate(hdf5_file, "/2/a", H5T_NATIVE_LONG, coordinate_dataspace, H5P_DEFAULT);
  else
    dataset_a = H5Dopen(hdf5_file, "/2/a");
  H5Dwrite(dataset_a, H5T_NATIVE_LONG, H5S_ALL, H5S_ALL, H5P_DEFAULT, _a);
  #if defined(HAVE_HDF5_HL)
    H5DSset_scale(dataset_a, "a");
  #endif
  
  H5Sclose(coordinate_dataspace);
  coordinate_length = _lattice_b;
  coordinate_dataspace = H5Screate_simple(1, &coordinate_length, NULL);
  hid_t dataset_b;
  if (!H5Lexists(hdf5_file, "/2/b", H5P_DEFAULT))
    dataset_b = H5Dcreate(hdf5_file, "/2/b", H5T_NATIVE_LONG, coordinate_dataspace, H5P_DEFAULT);
  else
    dataset_b = H5Dopen(hdf5_file, "/2/b");
  H5Dwrite(dataset_b, H5T_NATIVE_LONG, H5S_ALL, H5S_ALL, H5P_DEFAULT, _b);
  #if defined(HAVE_HDF5_HL)
    H5DSset_scale(dataset_b, "b");
  #endif
  
  H5Sclose(coordinate_dataspace);
  
  hsize_t file_dims[] = {_mg1_output_lattice_t, _lattice_a, _lattice_b};
  hid_t file_dataspace = H5Screate_simple(3, file_dims, NULL);
  
  hid_t dataset_pop;
  if (!H5Lexists(hdf5_file, "/2/pop", H5P_DEFAULT))
    dataset_pop = H5Dcreate(hdf5_file, "/2/pop", H5T_NATIVE_REAL, file_dataspace, H5P_DEFAULT);
  else
    dataset_pop = H5Dopen(hdf5_file, "/2/pop");
  #if defined(HAVE_HDF5_HL)
    H5DSattach_scale(dataset_pop, dataset_t, 0);
    H5DSattach_scale(dataset_pop, dataset_a, 1);
    H5DSattach_scale(dataset_pop, dataset_b, 2);
  #endif
  H5Dclose(dataset_t);
  H5Dclose(dataset_a);
  H5Dclose(dataset_b);
  
  
  if ((_mg1_output_lattice_t * _lattice_a * _lattice_b)) {
    /* Create the data space */
    hsize_t file_start[3] = {(unsigned long long int)0, (unsigned long long int)0, (unsigned long long int)0};
    hsize_t mem_dims[4] = {(unsigned long long int)_mg1_output_lattice_t, (unsigned long long int)_lattice_a, (unsigned long long int)_lattice_b, (unsigned long long int)1};
    hsize_t mem_start[4] = {0, 0, 0, 0};
    hsize_t mem_stride[4] = {1, 1, 1, 1};
    hsize_t mem_count[4] = {(unsigned long long int)_mg1_output_lattice_t, (unsigned long long int)_lattice_a, (unsigned long long int)_lattice_b, (unsigned long long int)1};
    
    
    hid_t mem_dataspace;
    mem_dims[3] = 1;
    mem_dataspace = H5Screate_simple(4, mem_dims, NULL);
    mem_stride[3] = 1;
    
    // Select hyperslabs of memory and file data spaces for data transfer operation
    mem_start[3] = 0;
    H5Sselect_hyperslab(mem_dataspace, H5S_SELECT_SET, mem_start, mem_stride, mem_count, NULL);
    H5Sselect_hyperslab(file_dataspace, H5S_SELECT_SET, file_start, mem_stride, mem_count, NULL);
    
    if (dataset_pop)
      H5Dwrite(dataset_pop, H5T_NATIVE_REAL, mem_dataspace, file_dataspace, H5P_DEFAULT, _active_mg1_output_raw);
    
    H5Sclose(mem_dataspace);
  }
  
  
  H5Dclose(dataset_pop);
  
  H5Sclose(file_dataspace);
  H5Gclose(group);
  H5Fclose(hdf5_file);
  
  
  if (_outfile) {
    fprintf(_outfile, "  </Array>\n");
    fprintf(_outfile, "</XSIL>\n");
  }
}

// ********************************************************
//   field mg1_output function implementations
// initialisation for vector raw
void _mg1_output_raw_initialise()
{
  
  bzero(_active_mg1_output_raw, sizeof(real) * _mg1_output_raw_alloc_size);
}

// ********************************************************
//   moment group 2 function implementations
void _mg2_sample()
{
  
  long _abcd_density_matrix_index_pointer = 0;
  #define rho _active_abcd_density_matrix[_abcd_density_matrix_index_pointer + 0]
  long _mg2_output_raw_index_pointer = 0;
  real trace;
  #define a _a[_index_a + 0]
  #define da (_da * (1.0))
  
  for (long _index_a = 0; _index_a < _lattice_a; _index_a++) {
    #define b _b[_index_b + 0]
    #define db (_db * (1.0))
    
    for (long _index_b = 0; _index_b < _lattice_b; _index_b++) {
      #define c _c[_index_c + 0]
      #define dc (_dc * (1.0))
      
      for (long _index_c = 0; _index_c < _lattice_c; _index_c++) {
        #define d _d[_index_d + 0]
        #define dd (_dd * (1.0))
        
        for (long _index_d = 0; _index_d < _lattice_d; _index_d++) {
          // Set index pointers explicitly for (some) vectors
          _mg2_output_raw_index_pointer = ( 0
             + _mg2_output_index_t  * 1 ) * _mg2_output_raw_ncomponents;
          #define _SAMPLE_COMPLEX(variable) \
                    variable ## R = variable.Re(); variable ## I = variable.Im();
          
          // *************** Sampling code ****************
          #line 197 "Assignment5_program_solution_v2.xmds"
          
          trace = 0.0;
          if(_index_a==_index_c && _index_b==_index_d)
             trace = rho.Re();
          
          
          #line 3132 "Assignment5_program_solution_v2.cc"
          // **********************************************
          
          #undef _SAMPLE_COMPLEX
          
          _active_mg2_output_raw[_mg2_output_raw_index_pointer + 0] += trace * da * db * dc * dd;
          // Increment index pointers for vectors in field mg2_sampling (or having the same dimensions)
          _abcd_density_matrix_index_pointer += 1 * _abcd_density_matrix_ncomponents;
          
        }
        #undef d
        #undef dd
      }
      #undef c
      #undef dc
    }
    #undef b
    #undef db
  }
  #undef a
  #undef da
  #undef rho
  
  _mg2_output_t[0 + _mg2_output_index_t++] = t;
  
  _LOG(_SAMPLE_LOG_LEVEL, "Sampled field (for moment group #3) at t = %e\n", t);
  
}


void _mg2_process()
{
  // No post processing needs to be done
}


void _mg2_write_out(FILE* _outfile)
{
  
  if (_outfile) {
    fprintf(_outfile, "\n");
    fprintf(_outfile, "<XSIL Name=\"moment_group_3\">\n");
    fprintf(_outfile, "  <Param Name=\"n_independent\">1</Param>\n");
    fprintf(_outfile, "  <Array Name=\"variables\" Type=\"Text\">\n");
    fprintf(_outfile, "    <Dim>2</Dim>\n");
    fprintf(_outfile, "    <Stream><Metalink Format=\"Text\" Delimiter=\" \\n\"/>\n");
    fprintf(_outfile, "t trace \n");
    fprintf(_outfile, "    </Stream>\n");
    fprintf(_outfile, "  </Array>\n");
    fprintf(_outfile, "  <Array Name=\"data\" Type=\"double\">\n");
    fprintf(_outfile, "    <Dim>%i</Dim>\n", _mg2_output_lattice_t);
    fprintf(_outfile, "    <Dim>2</Dim>\n");
  }
  
  
  char _h5Filename[200];
  snprintf(_h5Filename, 200, "%s.h5", ("Assignment5_program_solution_v2" + gsArgsAndValues).c_str());
  
  /* Open the file */
  hid_t hdf5_file = H5Fopen(_h5Filename, H5F_ACC_RDWR, H5P_DEFAULT);
  if (hdf5_file < 0) {
    _LOG(_WARNING_LOG_LEVEL, "Failed to open HDF5 file '%s', will try to create it.", _h5Filename);
    hdf5_file = H5Fcreate(_h5Filename, H5F_ACC_EXCL, H5P_DEFAULT, H5P_DEFAULT);
    if (hdf5_file < 0) {
      _LOG(_ERROR_LOG_LEVEL, "Failed to create HDF5 file '%s'. Bailing.", _h5Filename);
    }
  }
  
  /* Create the group for this data */
  hid_t group;
  if (!H5Lexists(hdf5_file, "/3", H5P_DEFAULT))
    group = H5Gcreate(hdf5_file, "/3", H5P_DEFAULT);
  else
    group = H5Gopen(hdf5_file, "/3");
  
  if (_outfile) {
    fprintf(_outfile, "    <Stream><Metalink Format=\"HDF5\" Type=\"Remote\" Group=\"/3\"/>\n");
    fprintf(_outfile, "%s.h5\n", ("Assignment5_program_solution_v2" + gsArgsAndValues).c_str());
    fprintf(_outfile, "    </Stream>\n");
  }
  
  /* Create the coordinate data sets */
  hsize_t coordinate_length;
  hid_t coordinate_dataspace;
  coordinate_length = _mg2_output_lattice_t;
  coordinate_dataspace = H5Screate_simple(1, &coordinate_length, NULL);
  hid_t dataset_t;
  if (!H5Lexists(hdf5_file, "/3/t", H5P_DEFAULT))
    dataset_t = H5Dcreate(hdf5_file, "/3/t", H5T_NATIVE_REAL, coordinate_dataspace, H5P_DEFAULT);
  else
    dataset_t = H5Dopen(hdf5_file, "/3/t");
  H5Dwrite(dataset_t, H5T_NATIVE_REAL, H5S_ALL, H5S_ALL, H5P_DEFAULT, _mg2_output_t);
  #if defined(HAVE_HDF5_HL)
    H5DSset_scale(dataset_t, "t");
  #endif
  
  H5Sclose(coordinate_dataspace);
  
  hsize_t file_dims[] = {_mg2_output_lattice_t};
  hid_t file_dataspace = H5Screate_simple(1, file_dims, NULL);
  
  hid_t dataset_trace;
  if (!H5Lexists(hdf5_file, "/3/trace", H5P_DEFAULT))
    dataset_trace = H5Dcreate(hdf5_file, "/3/trace", H5T_NATIVE_REAL, file_dataspace, H5P_DEFAULT);
  else
    dataset_trace = H5Dopen(hdf5_file, "/3/trace");
  #if defined(HAVE_HDF5_HL)
    H5DSattach_scale(dataset_trace, dataset_t, 0);
  #endif
  H5Dclose(dataset_t);
  
  
  if ((_mg2_output_lattice_t)) {
    /* Create the data space */
    hsize_t file_start[1] = {(unsigned long long int)0};
    hsize_t mem_dims[2] = {(unsigned long long int)_mg2_output_lattice_t, (unsigned long long int)1};
    hsize_t mem_start[2] = {0, 0};
    hsize_t mem_stride[2] = {1, 1};
    hsize_t mem_count[2] = {(unsigned long long int)_mg2_output_lattice_t, (unsigned long long int)1};
    
    
    hid_t mem_dataspace;
    mem_dims[1] = 1;
    mem_dataspace = H5Screate_simple(2, mem_dims, NULL);
    mem_stride[1] = 1;
    
    // Select hyperslabs of memory and file data spaces for data transfer operation
    mem_start[1] = 0;
    H5Sselect_hyperslab(mem_dataspace, H5S_SELECT_SET, mem_start, mem_stride, mem_count, NULL);
    H5Sselect_hyperslab(file_dataspace, H5S_SELECT_SET, file_start, mem_stride, mem_count, NULL);
    
    if (dataset_trace)
      H5Dwrite(dataset_trace, H5T_NATIVE_REAL, mem_dataspace, file_dataspace, H5P_DEFAULT, _active_mg2_output_raw);
    
    H5Sclose(mem_dataspace);
  }
  
  
  H5Dclose(dataset_trace);
  
  H5Sclose(file_dataspace);
  H5Gclose(group);
  H5Fclose(hdf5_file);
  
  
  if (_outfile) {
    fprintf(_outfile, "  </Array>\n");
    fprintf(_outfile, "</XSIL>\n");
  }
}

// ********************************************************
//   field mg2_output function implementations
// initialisation for vector raw
void _mg2_output_raw_initialise()
{
  
  bzero(_active_mg2_output_raw, sizeof(real) * _mg2_output_raw_alloc_size);
}

// ********************************************************
//   moment group 3 function implementations
void _mg3_sample()
{
  _dimensionless_time_dependent_coupling_evaluate();
  
  long _mg3_output_raw_index_pointer = 0;
  real energy;
  long _abcd_density_matrix_index_pointer = 0;
  #define rho _active_abcd_density_matrix[_abcd_density_matrix_index_pointer + 0]
  long _dimensionless_time_dependent_coupling_index_pointer = 0;
  #define OmegaA_t _active_dimensionless_time_dependent_coupling[_dimensionless_time_dependent_coupling_index_pointer + 0]
  #define OmegaB_t _active_dimensionless_time_dependent_coupling[_dimensionless_time_dependent_coupling_index_pointer + 1]
  #define DeltaA_t _active_dimensionless_time_dependent_coupling[_dimensionless_time_dependent_coupling_index_pointer + 2]
  #define DeltaB_t _active_dimensionless_time_dependent_coupling[_dimensionless_time_dependent_coupling_index_pointer + 3]
  #define U_t _active_dimensionless_time_dependent_coupling[_dimensionless_time_dependent_coupling_index_pointer + 4]
  #define a _a[_index_a + 0]
  #define da (_da * (1.0))
  
  for (long _index_a = 0; _index_a < _lattice_a; _index_a++) {
    #define b _b[_index_b + 0]
    #define db (_db * (1.0))
    
    for (long _index_b = 0; _index_b < _lattice_b; _index_b++) {
      #define c _c[_index_c + 0]
      #define dc (_dc * (1.0))
      
      for (long _index_c = 0; _index_c < _lattice_c; _index_c++) {
        #define d _d[_index_d + 0]
        #define dd (_dd * (1.0))
        
        for (long _index_d = 0; _index_d < _lattice_d; _index_d++) {
          // Set index pointers explicitly for (some) vectors
          _mg3_output_raw_index_pointer = ( 0
             + _mg3_output_index_t  * 1 ) * _mg3_output_raw_ncomponents;
          #define _SAMPLE_COMPLEX(variable) \
                    variable ## R = variable.Re(); variable ## I = variable.Im();
          
          // *************** Sampling code ****************
          #define _rho_abcd(_index_a, _index_b, _index_c, _index_d) _active_abcd_density_matrix[0 + (0 \
             + _index_a * _lattice_b * _lattice_c * _lattice_d \
             + _index_b * _lattice_c * _lattice_d \
             + _index_c * _lattice_d \
             + _index_d * 1) * _abcd_density_matrix_ncomponents]
          #line 209 "Assignment5_program_solution_v2.xmds"
          
          energy = 0.0;
          if(_index_a==0 && _index_b==0 && _index_c==0 && _index_d==0)   
              energy += (U_t*_rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 1 */ ((1) - _min_d)) + 0.5*DeltaB_t*(-_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 0 */ ((0) - _min_d)) + _rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 1 */ ((1) - _min_d)) - _rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 0 */ ((0) - _min_d)) + _rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 1 */ ((1) - _min_d))) + 0.5*DeltaA_t*(-_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 0 */ ((0) - _min_d)) - _rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 1 */ ((1) - _min_d)) + _rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 0 */ ((0) - _min_d)) + _rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 1 */ ((1) - _min_d))) + 0.5*OmegaA_t*(_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 0 */ ((0) - _min_d)) + _rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 0 */ ((0) - _min_d)) + _rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 1 */ ((1) - _min_d)) + _rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 1 */ ((1) - _min_d))) + 0.5*OmegaB_t*(_rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 1 */ ((1) - _min_d)) + _rho_abcd(/* a => 0 */ ((0) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 0 */ ((0) - _min_c), /* d => 0 */ ((0) - _min_d)) + _rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 0 */ ((0) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 1 */ ((1) - _min_d)) + _rho_abcd(/* a => 1 */ ((1) - _min_a), /* b => 1 */ ((1) - _min_b), /* c => 1 */ ((1) - _min_c), /* d => 0 */ ((0) - _min_d)))).Re();
          
          
          #line 3343 "Assignment5_program_solution_v2.cc"
          #undef _rho_abcd
          // **********************************************
          
          #undef _SAMPLE_COMPLEX
          
          _active_mg3_output_raw[_mg3_output_raw_index_pointer + 0] += energy * da * db * dc * dd;
          // Increment index pointers for vectors in field mg3_sampling (or having the same dimensions)
          _abcd_density_matrix_index_pointer += 1 * _abcd_density_matrix_ncomponents;
          
        }
        #undef d
        #undef dd
      }
      #undef c
      #undef dc
    }
    #undef b
    #undef db
  }
  #undef a
  #undef da
  #undef rho
  #undef OmegaA_t
  #undef OmegaB_t
  #undef DeltaA_t
  #undef DeltaB_t
  #undef U_t
  
  _mg3_output_t[0 + _mg3_output_index_t++] = t;
  
  _LOG(_SAMPLE_LOG_LEVEL, "Sampled field (for moment group #4) at t = %e\n", t);
  
}


void _mg3_process()
{
  // No post processing needs to be done
}


void _mg3_write_out(FILE* _outfile)
{
  
  if (_outfile) {
    fprintf(_outfile, "\n");
    fprintf(_outfile, "<XSIL Name=\"moment_group_4\">\n");
    fprintf(_outfile, "  <Param Name=\"n_independent\">1</Param>\n");
    fprintf(_outfile, "  <Array Name=\"variables\" Type=\"Text\">\n");
    fprintf(_outfile, "    <Dim>2</Dim>\n");
    fprintf(_outfile, "    <Stream><Metalink Format=\"Text\" Delimiter=\" \\n\"/>\n");
    fprintf(_outfile, "t energy \n");
    fprintf(_outfile, "    </Stream>\n");
    fprintf(_outfile, "  </Array>\n");
    fprintf(_outfile, "  <Array Name=\"data\" Type=\"double\">\n");
    fprintf(_outfile, "    <Dim>%i</Dim>\n", _mg3_output_lattice_t);
    fprintf(_outfile, "    <Dim>2</Dim>\n");
  }
  
  
  char _h5Filename[200];
  snprintf(_h5Filename, 200, "%s.h5", ("Assignment5_program_solution_v2" + gsArgsAndValues).c_str());
  
  /* Open the file */
  hid_t hdf5_file = H5Fopen(_h5Filename, H5F_ACC_RDWR, H5P_DEFAULT);
  if (hdf5_file < 0) {
    _LOG(_WARNING_LOG_LEVEL, "Failed to open HDF5 file '%s', will try to create it.", _h5Filename);
    hdf5_file = H5Fcreate(_h5Filename, H5F_ACC_EXCL, H5P_DEFAULT, H5P_DEFAULT);
    if (hdf5_file < 0) {
      _LOG(_ERROR_LOG_LEVEL, "Failed to create HDF5 file '%s'. Bailing.", _h5Filename);
    }
  }
  
  /* Create the group for this data */
  hid_t group;
  if (!H5Lexists(hdf5_file, "/4", H5P_DEFAULT))
    group = H5Gcreate(hdf5_file, "/4", H5P_DEFAULT);
  else
    group = H5Gopen(hdf5_file, "/4");
  
  if (_outfile) {
    fprintf(_outfile, "    <Stream><Metalink Format=\"HDF5\" Type=\"Remote\" Group=\"/4\"/>\n");
    fprintf(_outfile, "%s.h5\n", ("Assignment5_program_solution_v2" + gsArgsAndValues).c_str());
    fprintf(_outfile, "    </Stream>\n");
  }
  
  /* Create the coordinate data sets */
  hsize_t coordinate_length;
  hid_t coordinate_dataspace;
  coordinate_length = _mg3_output_lattice_t;
  coordinate_dataspace = H5Screate_simple(1, &coordinate_length, NULL);
  hid_t dataset_t;
  if (!H5Lexists(hdf5_file, "/4/t", H5P_DEFAULT))
    dataset_t = H5Dcreate(hdf5_file, "/4/t", H5T_NATIVE_REAL, coordinate_dataspace, H5P_DEFAULT);
  else
    dataset_t = H5Dopen(hdf5_file, "/4/t");
  H5Dwrite(dataset_t, H5T_NATIVE_REAL, H5S_ALL, H5S_ALL, H5P_DEFAULT, _mg3_output_t);
  #if defined(HAVE_HDF5_HL)
    H5DSset_scale(dataset_t, "t");
  #endif
  
  H5Sclose(coordinate_dataspace);
  
  hsize_t file_dims[] = {_mg3_output_lattice_t};
  hid_t file_dataspace = H5Screate_simple(1, file_dims, NULL);
  
  hid_t dataset_energy;
  if (!H5Lexists(hdf5_file, "/4/energy", H5P_DEFAULT))
    dataset_energy = H5Dcreate(hdf5_file, "/4/energy", H5T_NATIVE_REAL, file_dataspace, H5P_DEFAULT);
  else
    dataset_energy = H5Dopen(hdf5_file, "/4/energy");
  #if defined(HAVE_HDF5_HL)
    H5DSattach_scale(dataset_energy, dataset_t, 0);
  #endif
  H5Dclose(dataset_t);
  
  
  if ((_mg3_output_lattice_t)) {
    /* Create the data space */
    hsize_t file_start[1] = {(unsigned long long int)0};
    hsize_t mem_dims[2] = {(unsigned long long int)_mg3_output_lattice_t, (unsigned long long int)1};
    hsize_t mem_start[2] = {0, 0};
    hsize_t mem_stride[2] = {1, 1};
    hsize_t mem_count[2] = {(unsigned long long int)_mg3_output_lattice_t, (unsigned long long int)1};
    
    
    hid_t mem_dataspace;
    mem_dims[1] = 1;
    mem_dataspace = H5Screate_simple(2, mem_dims, NULL);
    mem_stride[1] = 1;
    
    // Select hyperslabs of memory and file data spaces for data transfer operation
    mem_start[1] = 0;
    H5Sselect_hyperslab(mem_dataspace, H5S_SELECT_SET, mem_start, mem_stride, mem_count, NULL);
    H5Sselect_hyperslab(file_dataspace, H5S_SELECT_SET, file_start, mem_stride, mem_count, NULL);
    
    if (dataset_energy)
      H5Dwrite(dataset_energy, H5T_NATIVE_REAL, mem_dataspace, file_dataspace, H5P_DEFAULT, _active_mg3_output_raw);
    
    H5Sclose(mem_dataspace);
  }
  
  
  H5Dclose(dataset_energy);
  
  H5Sclose(file_dataspace);
  H5Gclose(group);
  H5Fclose(hdf5_file);
  
  
  if (_outfile) {
    fprintf(_outfile, "  </Array>\n");
    fprintf(_outfile, "</XSIL>\n");
  }
}

// ********************************************************
//   field mg3_output function implementations
// initialisation for vector raw
void _mg3_output_raw_initialise()
{
  
  bzero(_active_mg3_output_raw, sizeof(real) * _mg3_output_raw_alloc_size);
}

// ********************************************************
//   moment group 4 function implementations
void _mg4_sample()
{
  _dimensionless_time_dependent_coupling_evaluate();
  
  long _mg4_output_raw_index_pointer = 0;
  #define OmegaA _active_mg4_output_raw[_mg4_output_raw_index_pointer + 0]
  #define OmegaB _active_mg4_output_raw[_mg4_output_raw_index_pointer + 1]
  #define DeltaA _active_mg4_output_raw[_mg4_output_raw_index_pointer + 2]
  #define DeltaB _active_mg4_output_raw[_mg4_output_raw_index_pointer + 3]
  #define UU _active_mg4_output_raw[_mg4_output_raw_index_pointer + 4]
  long _dimensionless_time_dependent_coupling_index_pointer = 0;
  #define OmegaA_t _active_dimensionless_time_dependent_coupling[_dimensionless_time_dependent_coupling_index_pointer + 0]
  #define OmegaB_t _active_dimensionless_time_dependent_coupling[_dimensionless_time_dependent_coupling_index_pointer + 1]
  #define DeltaA_t _active_dimensionless_time_dependent_coupling[_dimensionless_time_dependent_coupling_index_pointer + 2]
  #define DeltaB_t _active_dimensionless_time_dependent_coupling[_dimensionless_time_dependent_coupling_index_pointer + 3]
  #define U_t _active_dimensionless_time_dependent_coupling[_dimensionless_time_dependent_coupling_index_pointer + 4]
  // Set index pointers explicitly for (some) vectors
  _mg4_output_raw_index_pointer = ( 0
     + _mg4_output_index_t  * 1 ) * _mg4_output_raw_ncomponents;
  #define _SAMPLE_COMPLEX(variable) \
            variable ## R = variable.Re(); variable ## I = variable.Im();
  
  // *************** Sampling code ****************
  #line 221 "Assignment5_program_solution_v2.xmds"
  
  OmegaA = OmegaA_t;
  OmegaB = OmegaB_t;
  DeltaA = DeltaA_t;
  DeltaB = DeltaB_t;
  UU = U_t;
  
  #line 3542 "Assignment5_program_solution_v2.cc"
  // **********************************************
  
  #undef _SAMPLE_COMPLEX
  #undef OmegaA
  #undef OmegaB
  #undef DeltaA
  #undef DeltaB
  #undef UU
  #undef OmegaA_t
  #undef OmegaB_t
  #undef DeltaA_t
  #undef DeltaB_t
  #undef U_t
  
  _mg4_output_t[0 + _mg4_output_index_t++] = t;
  
  _LOG(_SAMPLE_LOG_LEVEL, "Sampled field (for moment group #5) at t = %e\n", t);
  
}


void _mg4_process()
{
  // No post processing needs to be done
}


void _mg4_write_out(FILE* _outfile)
{
  
  if (_outfile) {
    fprintf(_outfile, "\n");
    fprintf(_outfile, "<XSIL Name=\"moment_group_5\">\n");
    fprintf(_outfile, "  <Param Name=\"n_independent\">1</Param>\n");
    fprintf(_outfile, "  <Array Name=\"variables\" Type=\"Text\">\n");
    fprintf(_outfile, "    <Dim>6</Dim>\n");
    fprintf(_outfile, "    <Stream><Metalink Format=\"Text\" Delimiter=\" \\n\"/>\n");
    fprintf(_outfile, "t OmegaA OmegaB DeltaA DeltaB UU \n");
    fprintf(_outfile, "    </Stream>\n");
    fprintf(_outfile, "  </Array>\n");
    fprintf(_outfile, "  <Array Name=\"data\" Type=\"double\">\n");
    fprintf(_outfile, "    <Dim>%i</Dim>\n", _mg4_output_lattice_t);
    fprintf(_outfile, "    <Dim>6</Dim>\n");
  }
  
  
  char _h5Filename[200];
  snprintf(_h5Filename, 200, "%s.h5", ("Assignment5_program_solution_v2" + gsArgsAndValues).c_str());
  
  /* Open the file */
  hid_t hdf5_file = H5Fopen(_h5Filename, H5F_ACC_RDWR, H5P_DEFAULT);
  if (hdf5_file < 0) {
    _LOG(_WARNING_LOG_LEVEL, "Failed to open HDF5 file '%s', will try to create it.", _h5Filename);
    hdf5_file = H5Fcreate(_h5Filename, H5F_ACC_EXCL, H5P_DEFAULT, H5P_DEFAULT);
    if (hdf5_file < 0) {
      _LOG(_ERROR_LOG_LEVEL, "Failed to create HDF5 file '%s'. Bailing.", _h5Filename);
    }
  }
  
  /* Create the group for this data */
  hid_t group;
  if (!H5Lexists(hdf5_file, "/5", H5P_DEFAULT))
    group = H5Gcreate(hdf5_file, "/5", H5P_DEFAULT);
  else
    group = H5Gopen(hdf5_file, "/5");
  
  if (_outfile) {
    fprintf(_outfile, "    <Stream><Metalink Format=\"HDF5\" Type=\"Remote\" Group=\"/5\"/>\n");
    fprintf(_outfile, "%s.h5\n", ("Assignment5_program_solution_v2" + gsArgsAndValues).c_str());
    fprintf(_outfile, "    </Stream>\n");
  }
  
  /* Create the coordinate data sets */
  hsize_t coordinate_length;
  hid_t coordinate_dataspace;
  coordinate_length = _mg4_output_lattice_t;
  coordinate_dataspace = H5Screate_simple(1, &coordinate_length, NULL);
  hid_t dataset_t;
  if (!H5Lexists(hdf5_file, "/5/t", H5P_DEFAULT))
    dataset_t = H5Dcreate(hdf5_file, "/5/t", H5T_NATIVE_REAL, coordinate_dataspace, H5P_DEFAULT);
  else
    dataset_t = H5Dopen(hdf5_file, "/5/t");
  H5Dwrite(dataset_t, H5T_NATIVE_REAL, H5S_ALL, H5S_ALL, H5P_DEFAULT, _mg4_output_t);
  #if defined(HAVE_HDF5_HL)
    H5DSset_scale(dataset_t, "t");
  #endif
  
  H5Sclose(coordinate_dataspace);
  
  hsize_t file_dims[] = {_mg4_output_lattice_t};
  hid_t file_dataspace = H5Screate_simple(1, file_dims, NULL);
  
  hid_t dataset_OmegaA;
  if (!H5Lexists(hdf5_file, "/5/OmegaA", H5P_DEFAULT))
    dataset_OmegaA = H5Dcreate(hdf5_file, "/5/OmegaA", H5T_NATIVE_REAL, file_dataspace, H5P_DEFAULT);
  else
    dataset_OmegaA = H5Dopen(hdf5_file, "/5/OmegaA");
  #if defined(HAVE_HDF5_HL)
    H5DSattach_scale(dataset_OmegaA, dataset_t, 0);
  #endif
  hid_t dataset_OmegaB;
  if (!H5Lexists(hdf5_file, "/5/OmegaB", H5P_DEFAULT))
    dataset_OmegaB = H5Dcreate(hdf5_file, "/5/OmegaB", H5T_NATIVE_REAL, file_dataspace, H5P_DEFAULT);
  else
    dataset_OmegaB = H5Dopen(hdf5_file, "/5/OmegaB");
  #if defined(HAVE_HDF5_HL)
    H5DSattach_scale(dataset_OmegaB, dataset_t, 0);
  #endif
  hid_t dataset_DeltaA;
  if (!H5Lexists(hdf5_file, "/5/DeltaA", H5P_DEFAULT))
    dataset_DeltaA = H5Dcreate(hdf5_file, "/5/DeltaA", H5T_NATIVE_REAL, file_dataspace, H5P_DEFAULT);
  else
    dataset_DeltaA = H5Dopen(hdf5_file, "/5/DeltaA");
  #if defined(HAVE_HDF5_HL)
    H5DSattach_scale(dataset_DeltaA, dataset_t, 0);
  #endif
  hid_t dataset_DeltaB;
  if (!H5Lexists(hdf5_file, "/5/DeltaB", H5P_DEFAULT))
    dataset_DeltaB = H5Dcreate(hdf5_file, "/5/DeltaB", H5T_NATIVE_REAL, file_dataspace, H5P_DEFAULT);
  else
    dataset_DeltaB = H5Dopen(hdf5_file, "/5/DeltaB");
  #if defined(HAVE_HDF5_HL)
    H5DSattach_scale(dataset_DeltaB, dataset_t, 0);
  #endif
  hid_t dataset_UU;
  if (!H5Lexists(hdf5_file, "/5/UU", H5P_DEFAULT))
    dataset_UU = H5Dcreate(hdf5_file, "/5/UU", H5T_NATIVE_REAL, file_dataspace, H5P_DEFAULT);
  else
    dataset_UU = H5Dopen(hdf5_file, "/5/UU");
  #if defined(HAVE_HDF5_HL)
    H5DSattach_scale(dataset_UU, dataset_t, 0);
  #endif
  H5Dclose(dataset_t);
  
  
  if ((_mg4_output_lattice_t)) {
    /* Create the data space */
    hsize_t file_start[1] = {(unsigned long long int)0};
    hsize_t mem_dims[2] = {(unsigned long long int)_mg4_output_lattice_t, (unsigned long long int)1};
    hsize_t mem_start[2] = {0, 0};
    hsize_t mem_stride[2] = {1, 1};
    hsize_t mem_count[2] = {(unsigned long long int)_mg4_output_lattice_t, (unsigned long long int)1};
    
    
    hid_t mem_dataspace;
    mem_dims[1] = 5;
    mem_dataspace = H5Screate_simple(2, mem_dims, NULL);
    mem_stride[1] = 5;
    
    // Select hyperslabs of memory and file data spaces for data transfer operation
    mem_start[1] = 0;
    H5Sselect_hyperslab(mem_dataspace, H5S_SELECT_SET, mem_start, mem_stride, mem_count, NULL);
    H5Sselect_hyperslab(file_dataspace, H5S_SELECT_SET, file_start, mem_stride, mem_count, NULL);
    
    if (dataset_OmegaA)
      H5Dwrite(dataset_OmegaA, H5T_NATIVE_REAL, mem_dataspace, file_dataspace, H5P_DEFAULT, _active_mg4_output_raw);
    mem_start[1] = 1;
    H5Sselect_hyperslab(mem_dataspace, H5S_SELECT_SET, mem_start, mem_stride, mem_count, NULL);
    H5Sselect_hyperslab(file_dataspace, H5S_SELECT_SET, file_start, mem_stride, mem_count, NULL);
    
    if (dataset_OmegaB)
      H5Dwrite(dataset_OmegaB, H5T_NATIVE_REAL, mem_dataspace, file_dataspace, H5P_DEFAULT, _active_mg4_output_raw);
    mem_start[1] = 2;
    H5Sselect_hyperslab(mem_dataspace, H5S_SELECT_SET, mem_start, mem_stride, mem_count, NULL);
    H5Sselect_hyperslab(file_dataspace, H5S_SELECT_SET, file_start, mem_stride, mem_count, NULL);
    
    if (dataset_DeltaA)
      H5Dwrite(dataset_DeltaA, H5T_NATIVE_REAL, mem_dataspace, file_dataspace, H5P_DEFAULT, _active_mg4_output_raw);
    mem_start[1] = 3;
    H5Sselect_hyperslab(mem_dataspace, H5S_SELECT_SET, mem_start, mem_stride, mem_count, NULL);
    H5Sselect_hyperslab(file_dataspace, H5S_SELECT_SET, file_start, mem_stride, mem_count, NULL);
    
    if (dataset_DeltaB)
      H5Dwrite(dataset_DeltaB, H5T_NATIVE_REAL, mem_dataspace, file_dataspace, H5P_DEFAULT, _active_mg4_output_raw);
    mem_start[1] = 4;
    H5Sselect_hyperslab(mem_dataspace, H5S_SELECT_SET, mem_start, mem_stride, mem_count, NULL);
    H5Sselect_hyperslab(file_dataspace, H5S_SELECT_SET, file_start, mem_stride, mem_count, NULL);
    
    if (dataset_UU)
      H5Dwrite(dataset_UU, H5T_NATIVE_REAL, mem_dataspace, file_dataspace, H5P_DEFAULT, _active_mg4_output_raw);
    
    H5Sclose(mem_dataspace);
  }
  
  
  H5Dclose(dataset_OmegaA);
  H5Dclose(dataset_OmegaB);
  H5Dclose(dataset_DeltaA);
  H5Dclose(dataset_DeltaB);
  H5Dclose(dataset_UU);
  
  H5Sclose(file_dataspace);
  H5Gclose(group);
  H5Fclose(hdf5_file);
  
  
  if (_outfile) {
    fprintf(_outfile, "  </Array>\n");
    fprintf(_outfile, "</XSIL>\n");
  }
}

// ********************************************************
//   field mg4_output function implementations
// initialisation for vector raw
void _mg4_output_raw_initialise()
{
  
  bzero(_active_mg4_output_raw, sizeof(real) * _mg4_output_raw_alloc_size);
}

