(* Content-type: application/vnd.wolfram.mathematica *)

(*** Wolfram Notebook File ***)
(* http://www.wolfram.com/nb *)

(* CreatedBy='Mathematica 12.3' *)

(*CacheID: 234*)
(* Internal cache information:
NotebookFileLineBreakTest
NotebookFileLineBreakTest
NotebookDataPosition[       158,          7]
NotebookDataLength[     88076,       1759]
NotebookOptionsPosition[     85747,       1713]
NotebookOutlinePosition[     86139,       1729]
CellTagsIndexPosition[     86096,       1726]
WindowFrame->Normal*)

(* Beginning of Notebook Content *)
Notebook[{

Cell[CellGroupData[{
Cell["Define bunch of environmental oscillators", "Subsection",
 CellChangeTimes->{{3.87393939457479*^9, 
  3.873939414399152*^9}},ExpressionUUID->"e836eb5e-a170-4a27-9e72-\
5c40d19d20e1"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"Nosc", "=", "2000"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"mvalues", "=", 
   RowBox[{"Table", "[", 
    RowBox[{"1", ",", 
     RowBox[{"{", 
      RowBox[{"nn", ",", "1", ",", "Nosc"}], "}"}]}], "]"}]}], ";", " ", 
  RowBox[{"(*", 
   RowBox[{
    RowBox[{"we", " ", "see", " ", "from", " ", 
     RowBox[{"Eq", ".", " ", 
      RowBox[{"(", "4.42", ")"}]}], " ", "that", " ", "m", " ", "and", " ", 
     "\[Kappa]", " ", "enter", " ", "the", " ", "correlation", " ", 
     "function", " ", "in", " ", "a", " ", "very", " ", "similar", " ", 
     RowBox[{"way", ".", " ", "Hence"}], " ", "for", " ", "this", " ", "toy", 
     " ", "model"}], ",", " ", 
    RowBox[{
    "varying", " ", "them", " ", "both", " ", "separately", " ", "is", " ", 
     "not", " ", "necessary", " ", "and", " ", "can", " ", "be", " ", 
     "confusing"}]}], "*)"}], "\[IndentingNewLine]", 
  RowBox[{"dist\[Kappa]", "=", 
   RowBox[{"NormalDistribution", "[", 
    RowBox[{"1", ",", "1"}], "]"}]}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{
   RowBox[{"\[Kappa]values", "=", 
    RowBox[{"RandomVariate", "[", 
     RowBox[{"dist\[Kappa]", ",", "Nosc"}], "]"}]}], ";"}], 
  " "}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dist", "=", 
   RowBox[{"MixtureDistribution", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{"2", ",", "0.1"}], "}"}], ",", 
     RowBox[{"{", 
      RowBox[{
       RowBox[{"ChiSquareDistribution", "[", "4", "]"}], ",", 
       RowBox[{"NormalDistribution", "[", 
        RowBox[{"5", ",", "0.5"}], "]"}]}], "}"}]}], "]"}]}], ";", 
  RowBox[{"\[Omega]values", "=", 
   RowBox[{"RandomVariate", "[", 
    RowBox[{"dist", ",", "Nosc"}], "]"}]}], ";", 
  RowBox[{"Nosc", "=", 
   RowBox[{"Length", "[", "mvalues", "]"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.873939417422063*^9, 3.873939566228876*^9}, {
  3.873939667861251*^9, 3.8739397239393673`*^9}, {3.873939801178925*^9, 
  3.873939810260768*^9}, {3.87394014952747*^9, 3.873940180856598*^9}, {
  3.87394064565364*^9, 3.8739406491699867`*^9}, {3.8739409427308397`*^9, 
  3.8739409431309233`*^9}, {3.873941104867923*^9, 3.8739411050422707`*^9}, {
  3.873941166894039*^9, 3.873941188209999*^9}, {3.8739413117223053`*^9, 
  3.8739413432120657`*^9}, {3.8739415927998962`*^9, 3.873941653316766*^9}},
 CellLabel->
  "In[239]:=",ExpressionUUID->"0eb3f57c-547b-4fcd-a513-7c006788fd85"],

Cell[BoxData[
 RowBox[{"(*", 
  RowBox[{"lets", " ", "check", " ", "what", " ", "we", " ", 
   RowBox[{"got", ":"}]}], "*)"}]], "Input",
 CellChangeTimes->{{3.873939883417098*^9, 
  3.8739398881765537`*^9}},ExpressionUUID->"9e7bd1cb-1b52-413e-a570-\
6145627e2944"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Show", "[", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"Histogram", "[", 
    RowBox[{"\[Kappa]values", ",", "40", ",", "\"\<ProbabilityDensity\>\""}], 
    "]"}], ",", "\[IndentingNewLine]", 
   RowBox[{"Plot", "[", 
    RowBox[{
     RowBox[{"PDF", "[", 
      RowBox[{"dist\[Kappa]", ",", "x"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", 
       RowBox[{"-", "9"}], ",", "20"}], "}"}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", "Thick"}], ",", 
     RowBox[{"PlotRange", "->", " ", "All"}]}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{3.873941581205275*^9, 3.8739416151116037`*^9},
 CellLabel->
  "In[238]:=",ExpressionUUID->"3736c89a-f14f-48b8-82a0-50c6ef71a78b"],

Cell[BoxData[
 GraphicsBox[{{
    {RGBColor[0.987148, 0.8073604000000001, 0.49470040000000004`], EdgeForm[{
     Opacity[0.44099999999999995`], Thickness[Small]}], {}, 
     {RGBColor[0.987148, 0.8073604000000001, 0.49470040000000004`], EdgeForm[{
      Opacity[0.44099999999999995`], Thickness[Small]}], 
      RectangleBox[{-2.4, 0}, {-2.2, 0.0025},
       RoundingRadius->0], RectangleBox[{-2.2, 0}, {-2., 0.005},
       RoundingRadius->0], RectangleBox[{-2., 0}, {-1.8, 0.005},
       RoundingRadius->0], RectangleBox[{-1.8, 0}, {-1.6, 0.0075},
       RoundingRadius->0], RectangleBox[{-1.6, 0}, {-1.4, 0.01},
       RoundingRadius->0], RectangleBox[{-1.4, 0}, {-1.2, 0.0375},
       RoundingRadius->0], RectangleBox[{-1.2, 0}, {-1., 0.0525},
       RoundingRadius->0], RectangleBox[{-1., 0}, {-0.8, 0.065},
       RoundingRadius->0], RectangleBox[{-0.8, 0}, {-0.6, 0.1},
       RoundingRadius->0], RectangleBox[{-0.6, 0}, {-0.4, 0.1225},
       RoundingRadius->0], RectangleBox[{-0.4, 0}, {-0.2, 0.1875},
       RoundingRadius->0], RectangleBox[{-0.2, 0}, {0., 0.22},
       RoundingRadius->0], RectangleBox[{0., 0}, {0.2, 0.2975},
       RoundingRadius->0], RectangleBox[{0.2, 0}, {0.4, 0.31},
       RoundingRadius->0], RectangleBox[{0.4, 0}, {0.6, 0.3425},
       RoundingRadius->0], RectangleBox[{0.6, 0}, {0.8, 0.415},
       RoundingRadius->0], RectangleBox[{0.8, 0}, {1., 0.3675},
       RoundingRadius->0], RectangleBox[{1., 0}, {1.2, 0.3975},
       RoundingRadius->0], RectangleBox[{1.2, 0}, {1.4, 0.3675},
       RoundingRadius->0], RectangleBox[{1.4, 0}, {1.6, 0.3525},
       RoundingRadius->0], RectangleBox[{1.6, 0}, {1.8, 0.325},
       RoundingRadius->0], RectangleBox[{1.8, 0}, {2., 0.2225},
       RoundingRadius->0], RectangleBox[{2., 0}, {2.2, 0.2225},
       RoundingRadius->0], RectangleBox[{2.2, 0}, {2.4, 0.1475},
       RoundingRadius->0], RectangleBox[{2.4, 0}, {2.6, 0.13},
       RoundingRadius->0], RectangleBox[{2.6, 0}, {2.8, 0.085},
       RoundingRadius->0], RectangleBox[{2.8, 0}, {3., 0.065},
       RoundingRadius->0], RectangleBox[{3., 0}, {3.2, 0.055},
       RoundingRadius->0], RectangleBox[{3.2, 0}, {3.4, 0.0225},
       RoundingRadius->0], RectangleBox[{3.4, 0}, {3.6, 0.025},
       RoundingRadius->0], RectangleBox[{3.6, 0}, {3.8, 0.0125},
       RoundingRadius->0], RectangleBox[{3.8, 0}, {4., 0.0125},
       RoundingRadius->0], RectangleBox[{4., 0}, {4.2, 0.0025},
       RoundingRadius->0], RectangleBox[{4.2, 0}, {4.4, 0.0025},
       RoundingRadius->0], RectangleBox[{4.4, 0}, {4.6, 0.005},
       RoundingRadius->
        0]}, {}, {}}, {{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, \
{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, \
{}, {}, {}}}, {{{}, {}, 
     TagBox[
      {RGBColor[0.368417, 0.506779, 0.709798], Thickness[Large], Opacity[1.], 
       LineBox[CompressedData["
1:eJwUV3c41t8btveer9d6KdJCkv35nKMUSoQGZWSEzLIS6mtkZieZURllhYwQ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        "]]},
      Annotation[#, "Charting`Private`Tag$76372#1"]& ]}, {}}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{True, True},
  AxesLabel->{None, None},
  AxesOrigin->{-2.54, 0},
  FrameLabel->{{None, None}, {None, None}},
  FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  PlotRange->{{-2.4, 4.6}, {All, All}},
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {
     Scaled[0.02], 
     Scaled[0.05]}},
  Ticks->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{{3.8739415821437397`*^9, 3.873941615450714*^9}},
 CellLabel->
  "Out[238]=",ExpressionUUID->"56658caa-0e6e-4f40-9dd9-8c4aae1fa4ed"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"Show", "[", "\[IndentingNewLine]", 
  RowBox[{
   RowBox[{"Histogram", "[", 
    RowBox[{"\[Omega]values", ",", "40", ",", "\"\<ProbabilityDensity\>\""}], 
    "]"}], ",", "\[IndentingNewLine]", 
   RowBox[{"Plot", "[", 
    RowBox[{
     RowBox[{"PDF", "[", 
      RowBox[{"dist", ",", "x"}], "]"}], ",", 
     RowBox[{"{", 
      RowBox[{"x", ",", 
       RowBox[{"-", "9"}], ",", "20"}], "}"}], ",", 
     RowBox[{"PlotStyle", "\[Rule]", "Thick"}], ",", 
     RowBox[{"PlotRange", "->", " ", "All"}]}], "]"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.873941190928317*^9, 3.873941233677493*^9}, {
   3.873941335918558*^9, 3.873941336124617*^9}, 3.873941656354644*^9, 
   3.873941897374098*^9},
 CellLabel->
  "In[262]:=",ExpressionUUID->"861ee568-f68f-4a88-966a-5f05dbf645c5"],

Cell[BoxData[
 GraphicsBox[{{
    {RGBColor[0.987148, 0.8073604000000001, 0.49470040000000004`], EdgeForm[{
     Opacity[0.504], Thickness[Small]}], {}, 
     {RGBColor[0.987148, 0.8073604000000001, 0.49470040000000004`], EdgeForm[{
      Opacity[0.504], Thickness[Small]}], RectangleBox[{0., 0}, {0.5, 0.063},
       RoundingRadius->0], RectangleBox[{0.5, 0}, {1., 0.117},
       RoundingRadius->0], RectangleBox[{1., 0}, {1.5, 0.156},
       RoundingRadius->0], RectangleBox[{1.5, 0}, {2., 0.168},
       RoundingRadius->0], RectangleBox[{2., 0}, {2.5, 0.162},
       RoundingRadius->0], RectangleBox[{2.5, 0}, {3., 0.155},
       RoundingRadius->0], RectangleBox[{3., 0}, {3.5, 0.164},
       RoundingRadius->0], RectangleBox[{3.5, 0}, {4., 0.159},
       RoundingRadius->0], RectangleBox[{4., 0}, {4.5, 0.137},
       RoundingRadius->0], RectangleBox[{4.5, 0}, {5., 0.151},
       RoundingRadius->0], RectangleBox[{5., 0}, {5.5, 0.121},
       RoundingRadius->0], RectangleBox[{5.5, 0}, {6., 0.085},
       RoundingRadius->0], RectangleBox[{6., 0}, {6.5, 0.068},
       RoundingRadius->0], RectangleBox[{6.5, 0}, {7., 0.048},
       RoundingRadius->0], RectangleBox[{7., 0}, {7.5, 0.042},
       RoundingRadius->0], RectangleBox[{7.5, 0}, {8., 0.034},
       RoundingRadius->0], RectangleBox[{8., 0}, {8.5, 0.028},
       RoundingRadius->0], RectangleBox[{8.5, 0}, {9., 0.021},
       RoundingRadius->0], RectangleBox[{9., 0}, {9.5, 0.021},
       RoundingRadius->0], RectangleBox[{9.5, 0}, {10., 0.022},
       RoundingRadius->0], RectangleBox[{10., 0}, {10.5, 0.016},
       RoundingRadius->0], RectangleBox[{10.5, 0}, {11., 0.014},
       RoundingRadius->0], RectangleBox[{11., 0}, {11.5, 0.007},
       RoundingRadius->0], RectangleBox[{11.5, 0}, {12., 0.009},
       RoundingRadius->0], RectangleBox[{12., 0}, {12.5, 0.007},
       RoundingRadius->0], RectangleBox[{12.5, 0}, {13., 0.005},
       RoundingRadius->0], RectangleBox[{13., 0}, {13.5, 0.003},
       RoundingRadius->0], RectangleBox[{13.5, 0}, {14., 0.003},
       RoundingRadius->0], RectangleBox[{14., 0}, {14.5, 0.003},
       RoundingRadius->0], RectangleBox[{14.5, 0}, {15., 0.002},
       RoundingRadius->0], RectangleBox[{15., 0}, {15.5, 0.001},
       RoundingRadius->0], RectangleBox[{15.5, 0}, {16., 0.002},
       RoundingRadius->0], RectangleBox[{16.5, 0}, {17., 0.003},
       RoundingRadius->0], RectangleBox[{17., 0}, {17.5, 0.001},
       RoundingRadius->0], RectangleBox[{21.5, 0}, {22., 0.001},
       RoundingRadius->0], RectangleBox[{26.5, 0}, {27., 0.001},
       RoundingRadius->
        0]}, {}, {}}, {{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, \
{}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, {}, \
{}, {}, {}, {}}}, {{{}, {}, 
     TagBox[
      {RGBColor[0.368417, 0.506779, 0.709798], Thickness[Large], Opacity[1.], 
       LineBox[CompressedData["
1:eJwBsQJO/SFib1JlAgAAACoAAAACAAAAQikk7P//IcB9JZicS9NKHA2aYCNy
+yHAIVUIUlsRVhzYCp1a5PYhwIepWRD0JWIcbuwVycjtIcDJXDn2VIZ4HJqv
B6aR2yHA9d7Kp+ROphzzNetfI7chwLbudmICLgIdpEKy00ZuIcCjLowvQrm2
HQVcQLuN3CDAEJNkK93XGx/CcTV4L0EfwO8eKM5bGwciN4wK1SPzHMBfouQG
qSGiJDsDdDqrsBrAkCP6vAHMESdyzlkDKz0YwNTwcr/dW5QpZfpfJ4vzFcCt
O42MafLNK4x64q7jeBPAUfKlROjKEy5AV/k+zwkRwNMsnA3+oS0wYylhVDaJ
DcBQHuNGFYoDMqtMyPG+nAjA6a7EE8XB4DNtMXBFCAQEwHPX4uQguX01mJ6B
VO8E/7/7G92d9JP0NiErFKyvPfW/yeQlnWMIazg6dVDg4jvov8azuOlW86g5
PKDg+sPs5788DXtv7yivOT7LcBWlnee/F5wcvDRzszlDIZFKZ//mvz6uuM9u
R745TM3RtOvC5b/VqtwdbUPSOV4lU4n0SeO/MuZx2CYb+jkCq6tkDLDcv0HQ
VQig00g6BgHMmc4R3L/+aZBsHqNOOgtX7M6Qc9u/Ymcr9JHlUjoUAy05FTfa
v2+qTpA6uVw6JluuDR6+17+270kHZIRwOkkLsbYvzNK/EfIll4V1lTog12wR
ptDBv8mT3xfH2+A6cLH3//OZwL/NE8zGm3vkOoEXBd2Dxr6/Kke/fJ7g6DrC
gDCXu+u5v67FlO8zVPI6RFOHCys2sL9Dd0QhMdADO8kPOtGNkau/Axe79u8A
CDsKeWWLxbamvxCBPA3jEQ07Fpd4/2kCmr8wVqUpf0sVO5hpz3PZTJC/Wkey
W/jCGTu8nIKXU/CCv+Hi24PBch07teRMcA==
        "]], LineBox[CompressedData["
1:eJwVl3c8Vf8bwO2Vcbn23ntcLpX5PFQUMjIi2VmVZIaIqIyGigoRlRKVMr5m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        "]]},
      Annotation[#, "Charting`Private`Tag$94572#1"]& ], {}}, {}}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{True, True},
  AxesLabel->{None, None},
  AxesOrigin->{-0.26, 0},
  FrameLabel->{{None, None}, {None, None}},
  FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  PlotRange->{{0, 13}, {All, All}},
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {
     Scaled[0.02], 
     Scaled[0.05]}},
  Ticks->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{{3.873941191393849*^9, 3.873941234288328*^9}, {
   3.873941314038578*^9, 3.873941345571207*^9}, 3.8739416565806847`*^9, 
   3.8739418980086193`*^9},
 CellLabel->
  "Out[262]=",ExpressionUUID->"0c88ac0b-a6ce-4202-9ff7-2ef56ca4c034"]
}, Open  ]],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"?", " ", "MixtureDistribution"}]], "Input",
 CellChangeTimes->{{3.873941290221347*^9, 3.873941290674046*^9}},
 CellLabel->
  "In[195]:=",ExpressionUUID->"c4b922b7-37fc-425a-86c7-ee7970f45bac"],

Cell[BoxData[
 InterpretationBox[
  StyleBox[
   FrameBox[
    DynamicModuleBox[{System`InformationDump`open$$ = False, 
     System`InformationDump`mouseOver$$ = False}, 
     PaneSelectorBox[{True->
      TagBox[GridBox[{
         {
          ItemBox[
           PaneBox[
            StyleBox["\<\" Symbol\"\>", "InformationTitleText",
             StripOnInput->False,
             BaseStyle -> None],
            FrameMargins->{{4, 0}, {-1, 1}}],
           BaseStyle->"InformationTitleBackground",
           StripOnInput->False], 
          ItemBox[
           PaneBox[
            TooltipBox[
             ButtonBox[
              PaneSelectorBox[{False->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "InformationHelpIcon"]], True->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "InformationHelpIconHot"]]}, Dynamic[
                CurrentValue["MouseOver"]]],
              Appearance->None,
              BaseStyle->"Link",
              ButtonData->"paclet:ref/MixtureDistribution",
              ButtonNote->"paclet:ref/MixtureDistribution"],
             "\"paclet:ref/MixtureDistribution\""],
            FrameMargins->{{0, 4}, {0, 2}}],
           BaseStyle->"InformationTitleBackground",
           StripOnInput->False]},
         {
          ItemBox[
           PaneBox[
            
            StyleBox["\<\"\\!\\(\\*RowBox[{\\\"MixtureDistribution\\\", \\\"[\
\\\", RowBox[{RowBox[{\\\"{\\\", RowBox[{SubscriptBox[StyleBox[\\\"w\\\", \
\\\"TI\\\"], StyleBox[\\\"1\\\", \\\"TR\\\"]], \\\",\\\", StyleBox[\\\"\
\[Ellipsis]\\\", \\\"TR\\\"], \\\",\\\", SubscriptBox[StyleBox[\\\"w\\\", \
\\\"TI\\\"], StyleBox[\\\"n\\\", \\\"TI\\\"]]}], \\\"}\\\"}], \\\",\\\", \
RowBox[{\\\"{\\\", RowBox[{SubscriptBox[StyleBox[\\\"dist\\\", \\\"TI\\\"], \
StyleBox[\\\"1\\\", \\\"TR\\\"]], \\\",\\\", StyleBox[\\\"\[Ellipsis]\\\", \\\
\"TR\\\"], \\\",\\\", SubscriptBox[StyleBox[\\\"dist\\\", \\\"TI\\\"], \
StyleBox[\\\"n\\\", \\\"TI\\\"]]}], \\\"}\\\"}]}], \\\"]\\\"}]\\) represents \
a mixture distribution whose CDF is given as a sum of the CDFs of the \
component distributions \\!\\(\\*SubscriptBox[StyleBox[\\\"dist\\\", \\\"TI\\\
\"], StyleBox[\\\"i\\\", \\\"TI\\\"]]\\), each with weight \
\\!\\(\\*SubscriptBox[StyleBox[\\\"w\\\", \\\"TI\\\"], StyleBox[\\\"i\\\", \\\
\"TI\\\"]]\\).\"\>", "InformationUsageText",
             StripOnInput->False,
             LineSpacing->{1.5, 1.5, 3.}],
            FrameMargins->{{10, 10}, {8, 10}}],
           BaseStyle->"InformationUsageSubtitleBackground",
           StripOnInput->False], 
          ItemBox["\<\"\"\>",
           BaseStyle->"InformationUsageSubtitleBackground",
           StripOnInput->False]},
         {
          PaneBox[GridBox[{
             {
              
              DynamicModuleBox[{System`InformationDump`open$$ = {
               False, False, False, False, False, False}}, 
               StyleBox[GridBox[{
                  {
                   TagBox[
                    TooltipBox[
                    StyleBox["\<\" Documentation\"\>", "InformationRowLabel",
                    StripOnInput->False],
                    "\"Documentation\"",
                    TooltipStyle->"TextStyling"],
                    Annotation[#, "Documentation", "Tooltip"]& ], 
                   TemplateBox[{
                    
                    TemplateBox[{
                    "\"Local \[RightGuillemet]\"", 
                    "paclet:ref/MixtureDistribution", 
                    "paclet:ref/MixtureDistribution", "Link", {
                    RGBColor[0.9686274509803922, 0.4666666666666667, 0.]}, 
                    BaseStyle -> {
                    RGBColor[0.0784313725490196, 0.1568627450980392, 0.6]}}, 
                    "HyperlinkTemplate"], "\"  \"", 
                    StyleBox[
                    "\"|\"", "InformationRowLabel", StripOnInput -> False], 
                    "\"  \"", 
                    TemplateBox[{"\"Web \[RightGuillemet]\"", {
                    URL[
                    "http://reference.wolfram.com/language/ref/\
MixtureDistribution.html"], None}, 
                    "http://reference.wolfram.com/language/ref/\
MixtureDistribution.html", "Hyperlink", {
                    RGBColor[0.9686274509803922, 0.4666666666666667, 0.]}, 
                    BaseStyle -> {
                    RGBColor[0.0784313725490196, 0.1568627450980392, 0.6]}}, 
                    "HyperlinkTemplate"]},
                    "RowDefault"]},
                  {
                   TagBox[
                    TooltipBox[
                    StyleBox["\<\" Attributes\"\>", "InformationRowLabel",
                    StripOnInput->False],
                    "\"Attributes\"",
                    TooltipStyle->"TextStyling"],
                    Annotation[#, "Attributes", "Tooltip"]& ], 
                   RowBox[{"{", 
                    RowBox[{"Protected", ",", "ReadProtected"}], "}"}]},
                  {
                   TagBox[
                    TooltipBox[
                    StyleBox["\<\" Full Name\"\>", "InformationRowLabel",
                    StripOnInput->False],
                    "\"FullName\"",
                    TooltipStyle->"TextStyling"],
                    
                    Annotation[#, "FullName", 
                    "Tooltip"]& ], "\<\"System`MixtureDistribution\"\>"}
                 },
                 AutoDelete->False,
                 GridBoxAlignment->{"Columns" -> {Right, Left}},
                 GridBoxDividers->None,
                 GridBoxItemSize->{"Columns" -> {Automatic, Automatic}},
                 GridBoxSpacings->{"Columns" -> {
                    Offset[0.27999999999999997`], {
                    Offset[0.5599999999999999]}, 
                    Offset[0.27999999999999997`]}, "Rows" -> {
                    Offset[0.2], {
                    Offset[0.8]}, 
                    Offset[0.2]}}], "DialogStyle",
                StripOnInput->False],
               DynamicModuleValues:>{}]}
            },
            DefaultBaseStyle->"Column",
            GridBoxAlignment->{"Columns" -> {{Left}}},
            GridBoxDividers->{"Columns" -> {{False}}, "Rows" -> {{False}}},
            
            GridBoxItemSize->{
             "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}},
            GridBoxSpacings->{"Columns" -> {
                Offset[0.27999999999999997`], {
                 Offset[0.5599999999999999]}, 
                Offset[0.27999999999999997`]}, "Rows" -> {
                Offset[0.2], {
                 Offset[3.6]}, 
                Offset[0.2]}}],
           FrameMargins->{{6, 6}, {6, 3}}], ""},
         {
          ItemBox[
           TagBox[
            ButtonBox[
             PaneSelectorBox[{False->
              
              DynamicBox[FEPrivate`FrontEndResource[
               "FEBitmaps", "UpPointerOpener"]], True->
              
              DynamicBox[FEPrivate`FrontEndResource[
               "FEBitmaps", "UpPointerOpenerHot"]]}, Dynamic[
              System`InformationDump`mouseOver$$]],
             Alignment->Left,
             Appearance->{"Default" -> None},
             
             ButtonFunction:>FEPrivate`Set[
              System`InformationDump`open$$, False],
             Evaluator->Automatic,
             FrameMargins->{{9, 0}, {0, 0}},
             ImageMargins->0,
             ImageSize->Full,
             Method->"Preemptive"],
            
            EventHandlerTag[{
             "MouseEntered" :> 
              FEPrivate`Set[System`InformationDump`mouseOver$$, True], 
              "MouseExited" :> 
              FEPrivate`Set[System`InformationDump`mouseOver$$, False], 
              Method -> "Preemptive", PassEventsDown -> Automatic, 
              PassEventsUp -> True}]],
           BaseStyle->"InformationTitleBackground",
           StripOnInput->False], "\[SpanFromLeft]"}
        },
        AutoDelete->False,
        FrameStyle->Directive[
          GrayLevel[0.8], 
          Thickness[Tiny]],
        GridBoxAlignment->{"Columns" -> {Left, Right}, "Rows" -> {{Center}}},
        GridBoxDividers->{
         "Columns" -> {{None}}, "Rows" -> {False, {True}, False}},
        GridBoxItemSize->{
         "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
       "Grid"], False->
      TagBox[GridBox[{
         {
          ItemBox[
           PaneBox[
            StyleBox["\<\" Symbol\"\>", "InformationTitleText",
             StripOnInput->False],
            FrameMargins->{{4, 0}, {-1, 1}}],
           BaseStyle->"InformationTitleBackground",
           StripOnInput->False], 
          ItemBox[
           PaneBox[
            TooltipBox[
             ButtonBox[
              PaneSelectorBox[{False->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "InformationHelpIcon"],
                ImageSizeCache->{16., {2., 14.}}], True->
               
               DynamicBox[FEPrivate`FrontEndResource[
                "FEBitmaps", "InformationHelpIconHot"],
                ImageSizeCache->{16., {2., 14.}}]}, Dynamic[
                CurrentValue["MouseOver"]]],
              Appearance->None,
              BaseStyle->"Link",
              ButtonData->"paclet:ref/MixtureDistribution",
              ButtonNote->"paclet:ref/MixtureDistribution"],
             "\"paclet:ref/MixtureDistribution\""],
            FrameMargins->{{0, 4}, {0, 2}}],
           BaseStyle->"InformationTitleBackground",
           StripOnInput->False]},
         {
          ItemBox[
           PaneBox[
            
            StyleBox["\<\"\\!\\(\\*RowBox[{\\\"MixtureDistribution\\\", \\\"[\
\\\", RowBox[{RowBox[{\\\"{\\\", RowBox[{SubscriptBox[StyleBox[\\\"w\\\", \
\\\"TI\\\"], StyleBox[\\\"1\\\", \\\"TR\\\"]], \\\",\\\", StyleBox[\\\"\
\[Ellipsis]\\\", \\\"TR\\\"], \\\",\\\", SubscriptBox[StyleBox[\\\"w\\\", \
\\\"TI\\\"], StyleBox[\\\"n\\\", \\\"TI\\\"]]}], \\\"}\\\"}], \\\",\\\", \
RowBox[{\\\"{\\\", RowBox[{SubscriptBox[StyleBox[\\\"dist\\\", \\\"TI\\\"], \
StyleBox[\\\"1\\\", \\\"TR\\\"]], \\\",\\\", StyleBox[\\\"\[Ellipsis]\\\", \\\
\"TR\\\"], \\\",\\\", SubscriptBox[StyleBox[\\\"dist\\\", \\\"TI\\\"], \
StyleBox[\\\"n\\\", \\\"TI\\\"]]}], \\\"}\\\"}]}], \\\"]\\\"}]\\) represents \
a mixture distribution whose CDF is given as a sum of the CDFs of the \
component distributions \\!\\(\\*SubscriptBox[StyleBox[\\\"dist\\\", \\\"TI\\\
\"], StyleBox[\\\"i\\\", \\\"TI\\\"]]\\), each with weight \
\\!\\(\\*SubscriptBox[StyleBox[\\\"w\\\", \\\"TI\\\"], StyleBox[\\\"i\\\", \\\
\"TI\\\"]]\\).\"\>", "InformationUsageText",
             StripOnInput->False,
             LineSpacing->{1.5, 1.5, 3.}],
            FrameMargins->{{10, 10}, {8, 10}}],
           BaseStyle->"InformationUsageSubtitleBackground",
           StripOnInput->False], 
          ItemBox["\<\"\"\>",
           BaseStyle->"InformationUsageSubtitleBackground",
           StripOnInput->False]},
         {
          ItemBox[
           TagBox[
            ButtonBox[
             PaneSelectorBox[{False->
              
              DynamicBox[FEPrivate`FrontEndResource[
               "FEBitmaps", "DownPointerOpener"],
               ImageSizeCache->{13., {2., 11.}}], True->
              
              DynamicBox[FEPrivate`FrontEndResource[
               "FEBitmaps", "DownPointerOpenerHot"],
               ImageSizeCache->{13., {2., 11.}}]}, Dynamic[
              System`InformationDump`mouseOver$$]],
             Alignment->Left,
             Appearance->{"Default" -> None},
             
             ButtonFunction:>FEPrivate`Set[
              System`InformationDump`open$$, True],
             Evaluator->Automatic,
             FrameMargins->{{9, 0}, {0, 0}},
             ImageMargins->0,
             ImageSize->Full,
             Method->"Preemptive"],
            
            EventHandlerTag[{
             "MouseEntered" :> 
              FEPrivate`Set[System`InformationDump`mouseOver$$, True], 
              "MouseExited" :> 
              FEPrivate`Set[System`InformationDump`mouseOver$$, False], 
              Method -> "Preemptive", PassEventsDown -> Automatic, 
              PassEventsUp -> True}]],
           BaseStyle->"InformationTitleBackground",
           StripOnInput->False], "\[SpanFromLeft]"}
        },
        AutoDelete->False,
        FrameStyle->Directive[
          GrayLevel[0.8], 
          Thickness[Tiny]],
        GridBoxAlignment->{"Columns" -> {Left, Right}, "Rows" -> {{Center}}},
        GridBoxDividers->{
         "Columns" -> {{None}}, "Rows" -> {False, {True}, False}},
        GridBoxItemSize->{
         "Columns" -> {{Automatic}}, "Rows" -> {{Automatic}}}],
       "Grid"]}, Dynamic[System`InformationDump`open$$],
      BaselinePosition->Baseline,
      FrameMargins->0,
      ImageSize->Automatic],
     DynamicModuleValues:>{}],
    BaseStyle->"InformationGridFrame",
    StripOnInput->False], "InformationGridPlain",
   StripOnInput->False],
  InformationData[
   Association[
   "ObjectType" -> "Symbol", "Usage" -> 
    "\!\(\*RowBox[{\"MixtureDistribution\", \"[\", RowBox[{RowBox[{\"{\", \
RowBox[{SubscriptBox[StyleBox[\"w\", \"TI\"], StyleBox[\"1\", \"TR\"]], \
\",\", StyleBox[\"\[Ellipsis]\", \"TR\"], \",\", SubscriptBox[StyleBox[\"w\", \
\"TI\"], StyleBox[\"n\", \"TI\"]]}], \"}\"}], \",\", RowBox[{\"{\", \
RowBox[{SubscriptBox[StyleBox[\"dist\", \"TI\"], StyleBox[\"1\", \"TR\"]], \
\",\", StyleBox[\"\[Ellipsis]\", \"TR\"], \",\", SubscriptBox[StyleBox[\"dist\
\", \"TI\"], StyleBox[\"n\", \"TI\"]]}], \"}\"}]}], \"]\"}]\) represents a \
mixture distribution whose CDF is given as a sum of the CDFs of the component \
distributions \!\(\*SubscriptBox[StyleBox[\"dist\", \"TI\"], StyleBox[\"i\", \
\"TI\"]]\), each with weight \!\(\*SubscriptBox[StyleBox[\"w\", \"TI\"], \
StyleBox[\"i\", \"TI\"]]\).", "Documentation" -> 
    Association[
     "Local" -> "paclet:ref/MixtureDistribution", "Web" -> 
      "http://reference.wolfram.com/language/ref/MixtureDistribution.html"], 
    "OwnValues" -> None, "UpValues" -> None, "DownValues" -> None, 
    "SubValues" -> None, "DefaultValues" -> None, "NValues" -> None, 
    "FormatValues" -> None, "Options" -> {}, 
    "Attributes" -> {Protected, ReadProtected}, "FullName" -> 
    "System`MixtureDistribution"], False]]], "Output",
 CellChangeTimes->{3.873941291002441*^9},
 CellLabel->
  "Out[195]=",ExpressionUUID->"aadf5efb-9373-47b1-b35b-8dc36f0574f5"]
}, Open  ]]
}, Open  ]],

Cell[CellGroupData[{

Cell["Assemble the correlation function", "Subsection",
 CellChangeTimes->{{3.873939526763006*^9, 3.873939529481845*^9}, {
  3.873939891599744*^9, 
  3.873939901652748*^9}},ExpressionUUID->"fb39ea1f-78f0-4d4c-9d6a-\
00d6fc574f6a"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"kbT", ":=", 
   SuperscriptBox["10", 
    RowBox[{"-", "11"}]]}], ";", " ", 
  RowBox[{"(*", 
   RowBox[{"zero", " ", "temperature"}], "*)"}], "\[IndentingNewLine]", 
  RowBox[{"tmax", ":=", "20"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Nsamples", "=", "200"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dt", "=", 
   RowBox[{"tmax", "/", "Nsamples"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.8739400026074343`*^9, 3.8739400458862143`*^9}, 
   3.87394017148479*^9, 3.873940224431078*^9, {3.8739407171772823`*^9, 
   3.873940717541111*^9}, 3.873941353248323*^9, {3.8739414530260553`*^9, 
   3.873941454478271*^9}, {3.8739416968632507`*^9, 3.87394171093222*^9}, {
   3.873941910114225*^9, 3.873941936693622*^9}},
 CellLabel->
  "In[275]:=",ExpressionUUID->"8f84a382-e215-49eb-85f0-5971914befd5"],

Cell[BoxData[
 RowBox[{
  RowBox[{
   RowBox[{"CC", "[", "\[Tau]_", "]"}], ":=", 
   RowBox[{"Sum", "[", 
    RowBox[{
     RowBox[{
      RowBox[{"(", 
       FractionBox[
        SuperscriptBox[
         RowBox[{"\[Kappa]values", "[", 
          RowBox[{"[", "nn", "]"}], "]"}], "2"], 
        RowBox[{"2", " ", 
         RowBox[{"mvalues", "[", 
          RowBox[{"[", "nn", "]"}], "]"}], " ", 
         RowBox[{"\[Omega]values", "[", 
          RowBox[{"[", "nn", "]"}], "]"}]}]], ")"}], 
      RowBox[{"(", 
       RowBox[{
        RowBox[{
         RowBox[{"Coth", "[", 
          RowBox[{
           RowBox[{"\[Omega]values", "[", 
            RowBox[{"[", "nn", "]"}], "]"}], "/", 
           RowBox[{"(", 
            RowBox[{"2", " ", "kbT"}], ")"}]}], "]"}], " ", 
         RowBox[{"Cos", "[", 
          RowBox[{
           RowBox[{"\[Omega]values", "[", 
            RowBox[{"[", "nn", "]"}], "]"}], " ", "\[Tau]"}], "]"}]}], " ", 
        "-", " ", 
        RowBox[{"I", " ", 
         RowBox[{"Sin", "[", 
          RowBox[{
           RowBox[{"\[Omega]values", "[", 
            RowBox[{"[", "nn", "]"}], "]"}], " ", "\[Tau]"}], "]"}]}]}], 
       ")"}]}], ",", 
     RowBox[{"{", 
      RowBox[{"nn", ",", "1", ",", "Nosc"}], "}"}]}], "]"}]}], ";"}]], "Input",\

 CellChangeTimes->{{3.8739399049241257`*^9, 3.873939998266199*^9}, 
   3.873940079887677*^9, {3.87394088840123*^9, 3.873940896429344*^9}},
 CellLabel->
  "In[267]:=",ExpressionUUID->"65406cf6-0e0d-4447-8f9d-fc641b08e5c9"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ReCtabled", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"nn", "-", "1"}], ")"}], " ", "dt"}], ",", 
       RowBox[{"Re", "[", 
        RowBox[{"CC", "[", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"nn", "-", "1"}], ")"}], " ", "dt"}], "]"}], "]"}]}], 
      "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"nn", ",", "1", ",", "Nsamples"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ImCtabled", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"nn", "-", "1"}], ")"}], " ", "dt"}], ",", 
       RowBox[{"Im", "[", 
        RowBox[{"CC", "[", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"nn", "-", "1"}], ")"}], " ", "dt"}], "]"}], "]"}]}], 
      "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"nn", ",", "1", ",", "Nsamples"}], "}"}]}], "]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.873941725127096*^9, 3.8739418347974157`*^9}},
 CellLabel->
  "In[278]:=",ExpressionUUID->"c0526c00-5eff-47b4-b825-9de58976406c"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ListPlot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ReCtabled", ",", "ImCtabled"}], "}"}], ",", 
   RowBox[{"Joined", "->", " ", "True"}], ",", 
   RowBox[{"PlotRange", "->", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.87394178019396*^9, 3.8739417821624613`*^9}, {
  3.873941842103313*^9, 3.8739418738909807`*^9}},
 CellLabel->
  "In[280]:=",ExpressionUUID->"26ea4d45-b2ba-4a63-a094-718cb4e2afad"],

Cell[BoxData[
 GraphicsBox[{{}, {{}, {}, 
    {RGBColor[0.368417, 0.506779, 0.709798], PointSize[0.007333333333333334], 
     AbsoluteThickness[1.6], LineBox[CompressedData["
1:eJw11Qs4lNkfB3ARFZEYd5FhxriMmXkHCXm/Ma7JLS1d/q2tTYl1aaObkm60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      "]]}, 
    {RGBColor[0.880722, 0.611041, 0.142051], PointSize[0.007333333333333334], 
     AbsoluteThickness[1.6], LineBox[CompressedData["
1:eJxd1Hk81VkfB3AZoTL2FEq4l3tdLnfxkKX5fSjaZd/b7De5IpQWyxQ1rZpI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      "]]}}, {{}, {}}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{True, True},
  AxesLabel->{None, None},
  AxesOrigin->{0, 0},
  DisplayFunction->Identity,
  Frame->{{False, False}, {False, False}},
  FrameLabel->{{None, None}, {None, None}},
  FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  Method->{
   "OptimizePlotMarkers" -> True, "OptimizePlotMarkers" -> True, 
    "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
        Identity[
         Part[#, 1]], 
        Identity[
         Part[#, 2]]}& ), "CopiedValueFunction" -> ({
        Identity[
         Part[#, 1]], 
        Identity[
         Part[#, 2]]}& )}},
  PlotRange->{{0, 19.9}, {-495.02731595289976`, 1014.70212042676}},
  PlotRangeClipping->True,
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {
     Scaled[0.05], 
     Scaled[0.05]}},
  Ticks->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{
  3.8739417875085087`*^9, {3.873941849545072*^9, 3.873941874239143*^9}, 
   3.8739419191151037`*^9, 3.87394194976112*^9},
 CellLabel->
  "Out[280]=",ExpressionUUID->"44e750a1-5b9a-46e1-805b-95b16d3dad96"]
}, Open  ]],

Cell[BoxData[{
 RowBox[{
  RowBox[{"kbT", ":=", "2"}], ";", " ", 
  RowBox[{"(*", 
   RowBox[{"finite", " ", "temperature"}], "*)"}], "\[IndentingNewLine]", 
  RowBox[{"tmax", ":=", "80"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"Nsamples", "=", "200"}], ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"dt", "=", 
   RowBox[{"tmax", "/", "Nsamples"}]}], ";"}]}], "Input",
 CellChangeTimes->{{3.8739400026074343`*^9, 3.8739400458862143`*^9}, 
   3.87394017148479*^9, 3.873940224431078*^9, {3.8739407171772823`*^9, 
   3.873940717541111*^9}, 3.873941353248323*^9, {3.8739414530260553`*^9, 
   3.873941454478271*^9}, {3.8739416968632507`*^9, 3.87394171093222*^9}, {
   3.873941910114225*^9, 3.873941936693622*^9}, {3.87394199102227*^9, 
   3.873942017815506*^9}, 3.873942092430079*^9},
 CellLabel->
  "In[303]:=",ExpressionUUID->"6c968fe9-cde8-4ece-9f33-141fa30bdba8"],

Cell[BoxData[{
 RowBox[{
  RowBox[{"ReCtabled", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"nn", "-", "1"}], ")"}], " ", "dt"}], ",", 
       RowBox[{"Re", "[", 
        RowBox[{"CC", "[", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"nn", "-", "1"}], ")"}], " ", "dt"}], "]"}], "]"}]}], 
      "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"nn", ",", "1", ",", "Nsamples"}], "}"}]}], "]"}]}], 
  ";"}], "\[IndentingNewLine]", 
 RowBox[{
  RowBox[{"ImCtabled", "=", 
   RowBox[{"Table", "[", 
    RowBox[{
     RowBox[{"{", 
      RowBox[{
       RowBox[{
        RowBox[{"(", 
         RowBox[{"nn", "-", "1"}], ")"}], " ", "dt"}], ",", 
       RowBox[{"Im", "[", 
        RowBox[{"CC", "[", 
         RowBox[{
          RowBox[{"(", 
           RowBox[{"nn", "-", "1"}], ")"}], " ", "dt"}], "]"}], "]"}]}], 
      "}"}], ",", 
     RowBox[{"{", 
      RowBox[{"nn", ",", "1", ",", "Nsamples"}], "}"}]}], "]"}]}], 
  ";"}]}], "Input",
 CellChangeTimes->{{3.873941725127096*^9, 3.8739418347974157`*^9}},
 CellLabel->
  "In[306]:=",ExpressionUUID->"77b7889b-e33e-4801-a6bc-ba26e8e07ac1"],

Cell[CellGroupData[{

Cell[BoxData[
 RowBox[{"ListPlot", "[", 
  RowBox[{
   RowBox[{"{", 
    RowBox[{"ReCtabled", ",", "ImCtabled"}], "}"}], ",", 
   RowBox[{"Joined", "->", " ", "True"}], ",", 
   RowBox[{"PlotRange", "->", " ", "All"}]}], "]"}]], "Input",
 CellChangeTimes->{{3.87394178019396*^9, 3.8739417821624613`*^9}, {
  3.873941842103313*^9, 3.8739418738909807`*^9}},
 CellLabel->
  "In[308]:=",ExpressionUUID->"6e2e886e-f132-4697-8960-7d3d4ffb8a94"],

Cell[BoxData[
 GraphicsBox[{{}, {{}, {}, 
    {RGBColor[0.368417, 0.506779, 0.709798], PointSize[0.007333333333333334], 
     AbsoluteThickness[1.6], LineBox[CompressedData["
1:eJw11Qs01HkbB3BtErsptk1s5bKEitwi5fZl3IZhbnShIl6ii0sui0qKKMpl
DGNQLtW0KkoiUhqVtpJSHYku3q5UW23pTbXkrfXM75w5c37n/zyf5/uf8//9
Rycokhfyg5yc3OVvn+/fsrW3sd6OZXsexeLvq8dhYPLxAtaEVtq/cGjaG9cw
ONACi3/XoMOSjFtT0hvP0PXPDiy3zshz9adBHJqtOMPyrxqpfjyevRwxPOd/
CqH/LgWsLWNV9Q7VU78SNAP6Gq62nETHte9rEp69KZUoNtWRpwK/TWFnNUtO
UL0q3ia+Eytya8mfirutka1l+sep/xc0DigpSuJqaJ4ajLvUGZr3jpKnjqfq
2v7Zr46Qp4Eb806olCsfIW8GCtKNy4K1D5M3EytLWhiOz/8gTxOunaovmpMO
kacNJ+Yxq9ubJVSvjZ8rc0N3fDhIvg7M2h6FSTIOUv9vuMe2vR7RfoDm6WJ6
zIl04aID5OlB7uOjDzUn95Onh0+RCr6JQ5XkzYbfxeJTXf0V5OljXfFsp+09
5eQZYGqmOZv1tow8Qyhk9I6ECfaRZ4inx14nNmvsJW8Otj6q4vf7lpA3F0Gf
7nt4povJmwcjw6EJe3pE5BnhnH7Cn9OlheQZYf5zq8t5GoXkGaOh7bVdwu0C
8ubD1Mh+lvn/hOSZoCL/U9XAaiF5pvi1xEwt+V0+1Zsi7fKmx18/C8g3RaT4
0UiXp4D6zWD4h3+fQlgezTND3vBF/9odueSZQ4FRx251zyHPHBKFkQ6nidnk
mcN81ErSa7iHPAs0Wa4V1wl3k2cBnaPVK1P0s8hbgJ/eF3ZmBmSRtwChXc6n
39RkkbcAvsE7q9Mqs8izRL/OilttE7PIs8Sj8913ci/sJM8KDT+mK+pezSDP
CkwH/vlZCunkWaHabJPb7xWp5C1EDSM43/9KCnkLITRcZ+FYuoU8a1SHJDQV
dSaQZw3uRoMDfZqx5FnjK3vv1F9ZUeQtwgKT5aFnXNaQtwj63PUKA/0B5C3G
XBM/Yf3NZeQtxlN2uZZyoR95ixHaq2plLQwkzwb/1I4k6ekEk2eDEQlftaLu
P+TZImvf1f0nfIPJs8XW1Bv5yQkB5NliU8jMl+yh1eTZQbHd0li/SObZ4dke
s2sz8kLIs8elCS4mqx6Gk2cPpsio0iQpijx72BU160d2RpPnAJfapflWldHk
OSAiyT+tXzeKPOBrh/WdPeFrqB4YLuCw9tstJx/Q7vr7mdfm+dQPdHRv7Rt0
40vH5gFdDqkeWwSh0jHPEdwIUdbt6xulY54jvsrlvXnLTZaOeY6Yp36zY5zx
DumY54ioZYLEp0GZ5DnCteVx4C61HPKcsE7+r4U1rgLynLDcJ6b/woIC8pyg
F9zxTC20iDwn/NCkxP9Zq5Q8J4gURw1C2GXkMbAxVUN54bFy8hiQRq2RO+tW
QR4DuaO9ESO7KshjIC3NfnGMqII8BmZMyTgujaggzxmacwOs2xkV5DlDEgOV
j2fLyXOG6sjHPfEx5eQ5Y7s0L72RWU6eM2zkDB5veyDL54L2FPnAH1Vl+VyQ
7B3nLvaSeS74b3i2QZudzHNBTl26er1A5rlA78kyy8GJsnyu2DUUO36Kviyf
K14lXL+mqye7X1c0dOs3f7gv81zhWNYZr5kt81zBSdcPexBSTp4b1hYNHFYZ
LCPPDVPkA+yU3MvIc8Pg7tOML137yHPD00y+Uve8feS5YYvGa86nxlLy3FF7
puFQnWoJee5gpELl+IRi8twxtTjJL8NXTJ47nNW1bL2DxOR923tOUm6Xislj
Yo6FbU+bZTF5TOTfK56mayPzmCj9siqnap7MY2LxT96Z0bOLyGOiQDTc0RAm
Is8DhjXJ92PiC8nzwNDMlpXtjULyPGD3+73uJTb55HngjUXN671aAvI88O6q
iGGilEeeJ3rfq0y3fJ9NnidKEm7ETnPPJM8T68tNmeIzaeR5QtL0muHZvJU8
T7QfY+p8eZhIHguZarZeo3djyWNBtL34H+MHG8ljoV/DwDglKpY8Fjq2XYkQ
zoonj4U1ly4cbg6OI88LUXo1tUtz48nzgnvgxEyt3DjyvFDTcqGten8MeV4Q
fhxl6uyIJc8LT7JP6kkfRpPnDZ/sFlNzyQbyvKHxi+fNzbkh5HnjrSSLJ6e2
kjxvmK8Y1L1iHEieNx5XFh70ObiUPDbu5kavrnQBeWy4eSQ+vn9rGnlshHA+
xh91MCePDcGwU7jxHB/y2Lhyc5pEbVYAeRykTp78wsYoYKxegwPXKMFyn2/X
x3wOnLISN/wZFDTWz+Ig2nBn698hYTSPg6zeuFO9HZFjXgoHr0Jn2LzMiaf5
HBxw8Gm50b55zKvjYNzwoSbbRdsoDwfurBc3futLG/Oec1BluKJko3Im5eNi
87nIxE77HMrHRdjunoEVqnmUjwvTO4EhNtvpeWNxoX14WYtWcD7l48KoedWb
W/5CysfFDO+j6Us5BZSPi5XJ6dZqYfR813EhH3xkWnBPIeXjgnE8eedLLxHl
4+Kv9i9Fo6IiysfDhxbDOuNCOk8aPMyanji9qExM+XjQkgw/US6l88ri4fOi
7Wl5YjHl48E81+u2xSfyUr7tWaEXq9fL3uc8PDnaZP7gvYjy8RCrfSqpfL+I
8vHgu+KSmLdLlo8Hga7LffmHsvPLx9DplvEZ+0SUj4+g4m36wybkWfCh66C9
KymC7pfFx1T3JfXJG2T/L3x0S16MhFfJfj8+wnW+nrkbLKR8fAgYda+booSU
j4/WvnEJc7lCysdH++lUP5fl9L54zof6YNH6sklC6f8BjCJ6RQ==
      "]]}, 
    {RGBColor[0.880722, 0.611041, 0.142051], PointSize[0.007333333333333334], 
     AbsoluteThickness[1.6], LineBox[CompressedData["
1:eJxd1nlUk1caBnAQpGqVVTahLLLIviQBhAR4kkCAhJBNBUZgVBZFwYUCYmkF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      "]]}}, {{}, {}}},
  AspectRatio->NCache[GoldenRatio^(-1), 0.6180339887498948],
  Axes->{True, True},
  AxesLabel->{None, None},
  AxesOrigin->{0, 0},
  DisplayFunction->Identity,
  Frame->{{False, False}, {False, False}},
  FrameLabel->{{None, None}, {None, None}},
  FrameTicks->{{Automatic, Automatic}, {Automatic, Automatic}},
  GridLines->{None, None},
  GridLinesStyle->Directive[
    GrayLevel[0.5, 0.4]],
  Method->{
   "OptimizePlotMarkers" -> True, "OptimizePlotMarkers" -> True, 
    "CoordinatesToolOptions" -> {"DisplayFunction" -> ({
        Identity[
         Part[#, 1]], 
        Identity[
         Part[#, 2]]}& ), "CopiedValueFunction" -> ({
        Identity[
         Part[#, 1]], 
        Identity[
         Part[#, 2]]}& )}},
  PlotRange->{{0, 79.6}, {-2251.7250537430536`, 9336.595633235114}},
  PlotRangeClipping->True,
  PlotRangePadding->{{
     Scaled[0.02], 
     Scaled[0.02]}, {
     Scaled[0.05], 
     Scaled[0.05]}},
  Ticks->{Automatic, Automatic}]], "Output",
 CellChangeTimes->{
  3.8739417875085087`*^9, {3.873941849545072*^9, 3.873941874239143*^9}, 
   3.8739419191151037`*^9, 3.87394194976112*^9, {3.873942011898774*^9, 
   3.8739420235953608`*^9}, {3.873942098565464*^9, 3.873942109569388*^9}},
 CellLabel->
  "Out[308]=",ExpressionUUID->"2b378a21-ca36-45ab-814e-788bcfe6071b"]
}, Open  ]]
}, Open  ]]
},
WindowSize->{1102, 921},
WindowMargins->{{Automatic, 91}, {Automatic, 7}},
FrontEndVersion->"12.3 for Mac OS X x86 (64-bit) (July 9, 2021)",
StyleDefinitions->"Default.nb",
ExpressionUUID->"160fb1bd-0138-4c3c-8411-6ebfbb3d04b9"
]
(* End of Notebook Content *)

(* Internal cache information *)
(*CellTagsOutline
CellTagsIndex->{}
*)
(*CellTagsIndex
CellTagsIndex->{}
*)
(*NotebookFileOutline
Notebook[{
Cell[CellGroupData[{
Cell[580, 22, 188, 3, 83, "Subsection",ExpressionUUID->"e836eb5e-a170-4a27-9e72-5c40d19d20e1"],
Cell[771, 27, 2406, 55, 692, "Input",ExpressionUUID->"0eb3f57c-547b-4fcd-a513-7c006788fd85"],
Cell[3180, 84, 264, 6, 67, "Input",ExpressionUUID->"9e7bd1cb-1b52-413e-a570-6145627e2944"],
Cell[CellGroupData[{
Cell[3469, 94, 712, 17, 212, "Input",ExpressionUUID->"3736c89a-f14f-48b8-82a0-50c6ef71a78b"],
Cell[4184, 113, 28283, 473, 334, "Output",ExpressionUUID->"56658caa-0e6e-4f40-9dd9-8c4aae1fa4ed"]
}, Open  ]],
Cell[CellGroupData[{
Cell[32504, 591, 802, 19, 212, "Input",ExpressionUUID->"861ee568-f68f-4a88-966a-5f05dbf645c5"],
Cell[33309, 612, 16057, 273, 329, "Output",ExpressionUUID->"0c88ac0b-a6ce-4202-9ff7-2ef56ca4c034"]
}, Open  ]],
Cell[CellGroupData[{
Cell[49403, 890, 216, 4, 67, "Input",ExpressionUUID->"c4b922b7-37fc-425a-86c7-ee7970f45bac"],
Cell[49622, 896, 14517, 327, 230, "Output",ExpressionUUID->"aadf5efb-9373-47b1-b35b-8dc36f0574f5"]
}, Open  ]]
}, Open  ]],
Cell[CellGroupData[{
Cell[64188, 1229, 230, 4, 83, "Subsection",ExpressionUUID->"fb39ea1f-78f0-4d4c-9d6a-00d6fc574f6a"],
Cell[64421, 1235, 855, 19, 212, "Input",ExpressionUUID->"8f84a382-e215-49eb-85f0-5971914befd5"],
Cell[65279, 1256, 1501, 43, 283, "Input",ExpressionUUID->"65406cf6-0e0d-4447-8f9d-fc641b08e5c9"],
Cell[66783, 1301, 1194, 39, 212, "Input",ExpressionUUID->"c0526c00-5eff-47b4-b825-9de58976406c"],
Cell[CellGroupData[{
Cell[68002, 1344, 438, 10, 116, "Input",ExpressionUUID->"26ea4d45-b2ba-4a63-a094-718cb4e2afad"],
Cell[68443, 1356, 7303, 137, 318, "Output",ExpressionUUID->"44e750a1-5b9a-46e1-805b-95b16d3dad96"]
}, Open  ]],
Cell[75761, 1496, 881, 18, 212, "Input",ExpressionUUID->"6c968fe9-cde8-4ece-9f33-141fa30bdba8"],
Cell[76645, 1516, 1194, 39, 212, "Input",ExpressionUUID->"77b7889b-e33e-4801-a6bc-ba26e8e07ac1"],
Cell[CellGroupData[{
Cell[77864, 1559, 438, 10, 116, "Input",ExpressionUUID->"6e2e886e-f132-4697-8960-7d3d4ffb8a94"],
Cell[78305, 1571, 7414, 138, 314, "Output",ExpressionUUID->"2b378a21-ca36-45ab-814e-788bcfe6071b"]
}, Open  ]]
}, Open  ]]
}
]
*)

